@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #ifndef __Platform_H_
@@ -66,10 +70,14 @@ namespace Ogre {
 #endif
 
 /* See if we can use __forceinline or if we need to use __inline instead */
-#if OGRE_COMPILER == OGRE_COMPILER_MSVC 
+#if OGRE_COMPILER == OGRE_COMPILER_MSVC
 #   if OGRE_COMP_VER >= 1200
 #       define FORCEINLINE __forceinline
 #   endif
+#elif defined(__MINGW32__)
+#   if !defined(FORCEINLINE)
+#       define FORCEINLINE __inline
+#   endif
 #else
 #   define FORCEINLINE __inline
 #endif
@@ -86,12 +94,13 @@ namespace Ogre {
 #   define OGRE_PLATFORM OGRE_PLATFORM_LINUX
 #endif
 
-/* Find the arch type */
-#if defined(__x86_64__)
+    /* Find the arch type */
+#if defined(__x86_64__) || defined(_M_X64)
 #   define OGRE_ARCH_TYPE OGRE_ARCHITECTURE_64
 #else
 #   define OGRE_ARCH_TYPE OGRE_ARCHITECTURE_32
 #endif
+
 // For generating compiler warnings - should work on any compiler
 // As a side note, if you start your message with 'Warning: ', the MSVC
 // IDE actually does catch a warning :)
@@ -105,7 +114,7 @@ namespace Ogre {
 
 // If we're not including this from a client build, specify that the stuff
 // should get exported. Otherwise, import it.
-#	if defined( __MINGW32__ )
+#	if defined( OGRE_STATIC_LIB )
 		// Linux compilers don't have symbol import/export directives.
 #   	define _OgreExport
 #   	define _OgrePrivate
@@ -113,7 +122,11 @@ namespace Ogre {
 #   	if defined( OGRE_NONCLIENT_BUILD )
 #       	define _OgreExport __declspec( dllexport )
 #   	else
-#       	define _OgreExport __declspec( dllimport )
+#           if defined( __MINGW32__ )
+#               define _OgreExport
+#           else
+#       	    define _OgreExport __declspec( dllimport )
+#           endif
 #   	endif
 #   	define _OgrePrivate
 #	endif
@@ -124,17 +137,23 @@ namespace Ogre {
 #       define OGRE_DEBUG_MODE 0
 #   endif
 
-#if defined( __MINGW32__ )
-    #define EXT_HASH
-#else
+#if !defined( __MINGW32__ )
     #define snprintf _snprintf
     #define vsnprintf _vsnprintf
 #endif
 
-#if OGRE_DEBUG_MODE
-    #define OGRE_PLATFORM_LIB "OgrePlatform_d.dll"
+// Disable unicode support on MingW at the moment, poorly supported in stdlibc++
+// STLPORT fixes this though so allow if found
+// MinGW C++ Toolkit supports unicode and sets the define __MINGW32_TOOLKIT_UNICODE__ in _mingw.h
+#if defined( __MINGW32__ ) && !defined(_STLPORT_VERSION)
+#   include<_mingw.h>
+#   if defined(__MINGW32_TOOLBOX_UNICODE__)
+#	    define OGRE_UNICODE_SUPPORT 1
+#   else
+#       define OGRE_UNICODE_SUPPORT 0
+#   endif
 #else
-    #define OGRE_PLATFORM_LIB "OgrePlatform.dll"
+#	define OGRE_UNICODE_SUPPORT 1
 #endif
 
 #endif
@@ -144,8 +163,8 @@ namespace Ogre {
 // Linux/Apple Settings
 #if OGRE_PLATFORM == OGRE_PLATFORM_LINUX || OGRE_PLATFORM == OGRE_PLATFORM_APPLE
 
-// Enable GCC 4.0 symbol visibility 
-#   if OGRE_COMP_VER >= 400
+// Enable GCC symbol visibility
+#   if defined( OGRE_GCC_VISIBILITY )
 #       define _OgreExport  __attribute__ ((visibility("default")))
 #       define _OgrePrivate __attribute__ ((visibility("hidden")))
 #   else
@@ -158,6 +177,8 @@ namespace Ogre {
 
 // Unlike the Win32 compilers, Linux compilers seem to use DEBUG for when
 // specifying a debug build.
+// (??? this is wrong, on Linux debug builds aren't marked in any way unless
+// you mark it yourself any way you like it -- zap ???)
 #   ifdef DEBUG
 #       define OGRE_DEBUG_MODE 1
 #   else
@@ -171,17 +192,15 @@ namespace Ogre {
     #define OGRE_PLATFORM_LIB "libOgrePlatform.so"
 #endif
 
+// Always enable unicode support for the moment
+// Perhaps disable in old versions of gcc if necessary
+#define OGRE_UNICODE_SUPPORT 1
+
 #endif
 
 //For apple, we always have a custom config.h file
 #if OGRE_PLATFORM == OGRE_PLATFORM_APPLE
 #    include "config.h"
-//SDL_main must be included in the file that contains
-//the application's main() function.
-#ifndef OGRE_NONCLIENT_BUILD
-#   include <SDL/SDL_main.h>
-#endif
-
 #endif
 
 //----------------------------------------------------------------------------
@@ -189,7 +208,7 @@ namespace Ogre {
 //----------------------------------------------------------------------------
 // Endian Settings
 // check for BIG_ENDIAN config flag, set OGRE_ENDIAN correctly
-#ifdef CONFIG_BIG_ENDIAN
+#ifdef OGRE_CONFIG_BIG_ENDIAN
 #    define OGRE_ENDIAN OGRE_ENDIAN_BIG
 #else
 #    define OGRE_ENDIAN OGRE_ENDIAN_LITTLE
