@@ -2,9 +2,9 @@
 -----------------------------------------------------------------------------
 This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
-For the latest info, see http://ogre.sourceforge.net/
+For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #ifndef __TextureUnitState_H__
@@ -65,7 +69,11 @@ namespace Ogre {
             /// Generate texture coords based on a frustum
             ET_PROJECTIVE_TEXTURE,
             /// Constant u/v scrolling effect
-            ET_SCROLL,
+            ET_UVSCROLL,
+			/// Constant u scrolling effect
+            ET_USCROLL,
+			/// Constant u/v scrolling effect
+            ET_VSCROLL,
             /// Constant rotation
             ET_ROTATE,
             /// More complex transform
@@ -116,9 +124,17 @@ namespace Ogre {
             /// Texture mirrors (flips) at joins over 1.0
             TAM_MIRROR,
             /// Texture clamps at 1.0
-            TAM_CLAMP
+            TAM_CLAMP,
+            /// Texture coordinates outside the range [0.0, 1.0] are set to the border colour
+            TAM_BORDER
         };
 
+		/** Texture addressing mode for each texture coordinate. */
+		struct UVWAddressingMode
+		{
+			TextureAddressingMode u, v, w;
+		};
+
         /** Enum identifying the frame indexes for faces of a cube map (not the composite 3D type.
         */
         enum TextureCubeFace
@@ -146,6 +162,9 @@ namespace Ogre {
             const Frustum* frustum;
         };
 
+        /** Texture effects in a multimap paired array
+        */
+        typedef std::multimap<TextureEffectType, TextureEffect> EffectMap;
 
         /** Default constructor.
         */
@@ -182,9 +201,9 @@ namespace Ogre {
         @note
         Applies to both fixed-function and programmable pipeline.
         */
-        void setTextureName( const String& name, TextureType ttype = TEX_TYPE_2D, int mipmaps = -1 );
+        void setTextureName( const String& name, TextureType ttype = TEX_TYPE_2D);
 
-        /** Sets this texture layer to use a combination of 6 texture maps, each one relating to a face of a cube.
+		/** Sets this texture layer to use a combination of 6 texture maps, each one relating to a face of a cube.
         @remarks
         Cubic textures are made up of 6 separate texture images. Each one of these is an orthoganal view of the
         world with a FOV of 90 degrees and an aspect ratio of 1:1. You can generate these from 3D Studio by
@@ -328,7 +347,7 @@ namespace Ogre {
 
         /** Returns the width and height of the texture in the given frame.
         */
-        std::pair< uint, uint > getTextureDimensions( unsigned int frame = 0 ) const;
+        std::pair< size_t, size_t > getTextureDimensions( unsigned int frame = 0 ) const;
 
         /** Changes the active frame in an animated or multi-image texture.
         @remarks
@@ -345,18 +364,87 @@ namespace Ogre {
         */
         unsigned int getCurrentFrame(void) const;
 
-        /** Gets the name of the texture associated with a frame.
+        /** Gets the name of the texture associated with a frame number.
+            Throws an exception if frameNumber exceeds the number of stored frames.
         @note
         Applies to both fixed-function and programmable pipeline.
         */
         const String& getFrameTextureName(unsigned int frameNumber) const;
 
+        /** Sets the name of the texture associated with a frame.
+        @param name The name of the texture
+        @param frameNumber The frame the texture name is to be placed in
+        @note
+        Throws an exception if frameNumber exceeds the number of stored frames.
+        Applies to both fixed-function and programmable pipeline.
+        */
+        void setFrameTextureName(const String& name, unsigned int frameNumber);
+
+        /** Add a Texture name to the end of the frame container.
+        @param name The name of the texture
+        @note
+        Applies to both fixed-function and programmable pipeline.
+        */
+        void addFrameTextureName(const String& name);
+        /** deletes a specific texture frame.  The texture used is not deleted but the
+            texture will no longer be used by the Texture Unit.  An exception is raised
+            if the frame number exceeds the number of actual frames.
+        @param frameNumber The frame number of the texture to be deleted.
+        @note
+        Applies to both fixed-function and programmable pipeline.
+        */
+        void deleteFrameTextureName(const size_t frameNumber);
         /** Gets the number of frames for a texture.
         @note
         Applies to both fixed-function and programmable pipeline.
         */
         unsigned int getNumFrames(void) const;
 
+
+		/** The type of unit to bind the texture settings to. */
+		enum BindingType
+		{
+			/** Regular fragment processing unit - the default. */
+			BT_FRAGMENT = 0,
+			/** Vertex processing unit - indicates this unit will be used for 
+				a vertex texture fetch.
+			*/
+			BT_VERTEX = 1
+		};
+		/** Enum identifying the type of content this texture unit contains.
+		*/
+		enum ContentType
+		{
+			/// Normal texture identified by name
+			CONTENT_NAMED = 0,
+			/// A shadow texture, automatically bound by engine
+			CONTENT_SHADOW = 1
+		};
+
+		/** Sets the type of unit these texture settings should be bound to. 
+		@remarks
+			Some render systems, when implementing vertex texture fetch, separate
+			the binding of textures for use in the vertex program versus those
+			used in fragment programs. This setting allows you to target the
+			vertex processing unit with a texture binding, in those cases. For
+			rendersystems which have a unified binding for the vertex and fragment
+			units, this setting makes no difference.
+		*/
+		void setBindingType(BindingType bt);
+
+		/** Gets the type of unit these texture settings should be bound to.  
+		*/
+		BindingType getBindingType(void) const;
+
+		/** Set the type of content this TextureUnitState references.
+		@remarks
+			The default is to reference a standard named texture, but this unit
+			can also reference automated content like a shadow texture.
+		*/
+		void setContentType(ContentType ct);
+		/** Get the type of content this TextureUnitState references. */
+		ContentType getContentType(void) const;
+
         /** Returns true if this texture unit is either a series of 6 2D textures, each
             in it's own frame, or is a full 3D cube map. You can tell which by checking
             getTextureType.
@@ -377,6 +465,30 @@ namespace Ogre {
         */
         TextureType getTextureType(void) const;
 
+        /** Sets the desired pixel format when load the texture.
+        */
+        void setDesiredFormat(PixelFormat desiredFormat);
+
+        /** Gets the desired pixel format when load the texture.
+        */
+        PixelFormat getDesiredFormat(void) const;
+
+        /** Sets how many mipmaps have been requested for the texture.
+		*/
+        void setNumMipmaps(int numMipmaps);
+
+        /** Gets how many mipmaps have been requested for the texture.
+		*/
+        int getNumMipmaps(void) const;
+
+		/** Sets whether this texture is requested to be loaded as alpha if single channel
+		*/
+        void setIsAlpha(bool isAlpha);
+
+		/** Gets whether this texture is requested to be loaded as alpha if single channel
+		*/
+        bool getIsAlpha(void) const;
+
         /** Gets the index of the set of texture co-ords this layer uses.
         @note
         Applies to both fixed-function and programmable pipeline.
@@ -497,21 +609,58 @@ namespace Ogre {
         // get texture rotation effects angle value
         const Radian& getTextureRotate(void) const;
 
-        /** Gets the texture addressing mode, i.e. what happens at uv values above 1.0.
+        /** Gets the texture addressing mode for a given coordinate, 
+		 	i.e. what happens at uv values above 1.0.
+        @note
+        	The default is TAM_WRAP i.e. the texture repeats over values of 1.0.
+        */
+        const UVWAddressingMode& getTextureAddressingMode(void) const;
+
+        /** Sets the texture addressing mode, i.e. what happens at uv values above 1.0.
         @note
         The default is TAM_WRAP i.e. the texture repeats over values of 1.0.
+		@note This is a shortcut method which sets the addressing mode for all
+			coordinates at once; you can also call the more specific method
+			to set the addressing mode per coordinate.
+        @note
+        This applies for both the fixed-function and programmable pipelines.
         */
-        TextureAddressingMode getTextureAddressingMode(void) const;
+        void setTextureAddressingMode( TextureAddressingMode tam);
 
         /** Sets the texture addressing mode, i.e. what happens at uv values above 1.0.
         @note
         The default is TAM_WRAP i.e. the texture repeats over values of 1.0.
         @note
         This applies for both the fixed-function and programmable pipelines.
-        */
-        void setTextureAddressingMode( TextureAddressingMode tam);
+		*/
+        void setTextureAddressingMode( TextureAddressingMode u, 
+			TextureAddressingMode v, TextureAddressingMode w);
 
-        /** Setting advanced blending options.
+        /** Sets the texture addressing mode, i.e. what happens at uv values above 1.0.
+        @note
+        The default is TAM_WRAP i.e. the texture repeats over values of 1.0.
+        @note
+        This applies for both the fixed-function and programmable pipelines.
+		*/
+        void setTextureAddressingMode( const UVWAddressingMode& uvw);
+
+        /** Sets the texture border colour.
+        @note
+            The default is ColourValue::Black, and this value only used when addressing mode
+            is TAM_BORDER.
+        @note
+            This applies for both the fixed-function and programmable pipelines.
+		*/
+        void setTextureBorderColour(const ColourValue& colour);
+
+        /** Sets the texture border colour.
+        @note
+            The default is ColourValue::Black, and this value only used when addressing mode
+            is TAM_BORDER.
+		*/
+        const ColourValue& getTextureBorderColour(void) const;
+
+		/** Setting advanced blending options.
         @remarks
         This is an extended version of the TextureUnitState::setColourOperation method which allows
         extremely detailed control over the blending applied between this and earlier layers.
@@ -803,8 +952,16 @@ namespace Ogre {
         */
         void setBlank(void);
 
+		/** Tests if the texture associated with this unit has failed to load.
+		*/
+		bool isTextureLoadFailing() const { return mTextureLoadFailed; }
+
+		/** Tells the unit to retry loading the texture if it had failed to load.
+		*/
+		void retryTextureLoad() { mTextureLoadFailed = false; }
+
         // get texture effects in a multimap paired array
-        std::multimap<TextureEffectType, TextureEffect> getEffects(void) const;
+        const EffectMap& getEffects(void) const;
         // get the animated-texture animation duration
         Real getAnimationDuration(void) const;
 
@@ -844,48 +1001,113 @@ namespace Ogre {
         // get this layer texture anisotropy level
         unsigned int getTextureAnisotropy() const;
 
+		/** Sets the bias value applied to the mipmap calculation.
+		@remarks
+			You can alter the mipmap calculation by biasing the result with a 
+			single floating point value. After the mip level has been calculated,
+			this bias value is added to the result to give the final mip level.
+			Lower mip levels are larger (higher detail), so a negative bias will
+			force the larger mip levels to be used, and a positive bias
+			will cause smaller mip levels to be used. The bias values are in 
+			mip levels, so a -1 bias will force mip levels one larger than by the
+			default calculation.
+		@param bias The bias value as described above, can be positive or negative.
+		*/
+		void setTextureMipmapBias(float bias) { mMipmapBias = bias; }
+		/** Gets the bias value applied to the mipmap calculation.
+		@see TextureUnitState::setTextureMipmapBias
+		*/
+		float getTextureMipmapBias(void) const { return mMipmapBias; }
+
         /// Gets the parent Pass object
-        Pass* getParent(void) { return mParent; }
+        Pass* getParent(void) const { return mParent; }
 
 		/** Internal method for loading this object as part of Material::load */
 		void _load(void);
 		/** Internal method for unloading this object as part of Material::unload */
 		void _unload(void);
         /// Returns whether this unit has texture coordinate generation that depends on the camera
-        bool hasViewRelativeTextureCoordinateGeneration(void);
+        bool hasViewRelativeTextureCoordinateGeneration(void) const;
 
         // Is this loaded?
-        bool isLoaded(void);
+        bool isLoaded(void) const;
         /** Tells the class that it needs recompilation. */
         void _notifyNeedsRecompile(void);
 
+        /** Set the name of the Texture Unit State
+        @remarks
+            The name of the Texture Unit State is optional.  Its usefull in material scripts where a material could inherit
+            from another material and only want to modify a particalar Texture Unit State.
+        */
+        void setName(const String& name);
+        /// get the name of the Texture Unit State
+        const String& getName(void) const { return mName; }
+
+        /** Set the alias name used for texture frame names
+        @param name can be any sequence of characters and does not have to be unique           
+        */
+        void setTextureNameAlias(const String& name);
+        /** gets the Texture Name Alias of the Texture Unit.
+        */
+        const String& getTextureNameAlias(void) const { return mTextureNameAlias;}
+
+        /** Applies texture names to Texture Unit State with matching texture name aliases.
+            If no matching aliases are found then the TUS state does not change.
+        @remarks
+            Cubic, 1d, 2d, and 3d textures are determined from current state of the Texture Unit.
+            Assumes animated frames are sequentially numbered in the name.
+            If matching texture aliases are found then true is returned.
+
+        @param
+            aliasList is a map container of texture alias, texture name pairs
+        @param
+            apply set true to apply the texture aliases else just test to see if texture alias matches are found.
+        @return
+            True if matching texture aliases were found in the Texture Unit State.
+        */
+        bool applyTextureAliases(const AliasTextureNamePairList& aliasList, const bool apply = true);
+
+		/** Notify this object that its parent has changed */
+		void _notifyParent(Pass* parent);
+
+		/** Get the texture pointer for the current frame. */
+		const TexturePtr& _getTexturePtr(void) const;
+		/** Get the texture pointer for a given frame. */
+		const TexturePtr& _getTexturePtr(size_t frame) const;
 	
+		/** Set the texture pointer for the current frame (internal use only!). */
+		void _setTexturePtr(const TexturePtr& texptr);
+		/** Set the texture pointer for a given frame (internal use only!). */
+		void _setTexturePtr(const TexturePtr& texptr, size_t frame);
+
+		/** Gets the animation controller (as created because of setAnimatedTexture)
+			if it exists.
+		*/
+		Controller<Real>* _getAnimController() const { return mAnimController; }
 protected:
         // State
-#define OGRE_MAX_TEXTURE_FRAMES 32
-
-        /// Number of frames of animation, or frames making up cubic
-        unsigned int mNumFrames;        
-        /// The xurrent animation frame.
+        /// The current animation frame.
         unsigned int mCurrentFrame;
-        // String mFrames[OGRE_MAX_TEXTURE_FRAMES] is at the end of the class                
 
         /// Duration of animation in seconds
         Real mAnimDuration;            
         bool mCubic; // is this a series of 6 2D textures to make up a cube?
 		
         TextureType mTextureType; 
+        PixelFormat mDesiredFormat;
 		int mTextureSrcMipmaps; // Request number of mipmaps
 
         unsigned int mTextureCoordSetIndex;
-        TextureAddressingMode mAddressMode;                
+        UVWAddressingMode mAddressMode;
+        ColourValue mBorderColour;
 
-        LayerBlendModeEx colourBlendMode;
-        SceneBlendFactor colourBlendFallbackSrc;
-        SceneBlendFactor colourBlendFallbackDest;
+        LayerBlendModeEx mColourBlendMode;
+        SceneBlendFactor mColourBlendFallbackSrc;
+        SceneBlendFactor mColourBlendFallbackDest;
 
-        LayerBlendModeEx alphaBlendMode;
-        bool mIsBlank;
+        LayerBlendModeEx mAlphaBlendMode;
+        mutable bool mTextureLoadFailed;
+        bool mIsAlpha;
 
         mutable bool mRecalcTexMatrix;
         Real mUMod, mVMod;
@@ -893,10 +1115,6 @@ protected:
         Radian mRotate;
         mutable Matrix4 mTexModMatrix;
 
-        // Animation, will be set up as Controllers
-        Real mUScrollAnim, mVScrollAnim;
-        Real mRotateAnim;
-
         /// Texture filtering - minification
         FilterOptions mMinFilter;
         /// Texture filtering - magnification
@@ -905,18 +1123,24 @@ protected:
         FilterOptions mMipFilter;
         ///Texture anisotropy
         unsigned int mMaxAniso;
+		/// Mipmap bias (always float, not Real)
+		float mMipmapBias;
 
         bool mIsDefaultAniso;
         bool mIsDefaultFiltering;
-
+		/// Binding type (fragment or vertex pipeline)
+		BindingType mBindingType;
+		/// Content type of texture (normal loaded texture, auto-texture)
+		ContentType mContentType;
 
         //-----------------------------------------------------------------------------
         // Complex members (those that can't be copied using memcpy) are at the end to 
         // allow for fast copying of the basic members.
         //
-        String mFrames[OGRE_MAX_TEXTURE_FRAMES]; // Names of frames
-
-        typedef std::multimap<TextureEffectType, TextureEffect> EffectMap;
+        std::vector<String> mFrames;
+		mutable std::vector<TexturePtr> mFramePtrs;
+        String mName;               // optional name for the TUS
+        String mTextureNameAlias;       // optional alias for texture frames
         EffectMap mEffects;
         //-----------------------------------------------------------------------------
 
@@ -941,6 +1165,9 @@ protected:
         */
         void createEffectController(TextureEffect& effect);
 
+		/** Internal method for ensuring the texture for a given frame is loaded. */
+		void ensureLoaded(size_t frame) const;
+
 
     };
 
