@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
@@ -28,6 +32,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreStringConverter.h"
 #include "OgreVertexIndexData.h"
 #include "OgreException.h"
+#include "OgreOptimisedUtil.h"
 
 namespace Ogre {
 
@@ -82,12 +87,28 @@ namespace Ogre {
     //---------------------------------------------------------------------
     void EdgeListBuilder::addVertexData(const VertexData* vertexData)
     {
+        if (vertexData->vertexStart != 0)
+        {
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+                "The base vertex index of the vertex data must be zero for build edge list.",
+                "EdgeListBuilder::addVertexData");
+        }
+
         mVertexDataList.push_back(vertexData);
     }
     //---------------------------------------------------------------------
     void EdgeListBuilder::addIndexData(const IndexData* indexData, 
         size_t vertexSet, RenderOperation::OperationType opType)
     {
+        if (opType != RenderOperation::OT_TRIANGLE_LIST &&
+            opType != RenderOperation::OT_TRIANGLE_FAN &&
+            opType != RenderOperation::OT_TRIANGLE_STRIP)
+        {
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+                "Only triangle list, fan and strip are supported to build edge list.",
+                "EdgeListBuilder::addIndexData");
+        }
+
         Geometry geometry;
         geometry.indexData = indexData;
         geometry.vertexSet = vertexSet;
@@ -151,7 +172,7 @@ namespace Ogre {
         */
 
         // Sort the geometries in the order of vertex set, so we can grouping
-        // triangles by vertex set easy, and improve memory access as well.
+        // triangles by vertex set easy.
         std::sort(mGeometryList.begin(), mGeometryList.end(), geometryLess());
         // Initialize edge data
         mEdgeData = new EdgeData();
@@ -162,6 +183,8 @@ namespace Ogre {
         {
             mEdgeData->edgeGroups[vSet].vertexSet = vSet;
             mEdgeData->edgeGroups[vSet].vertexData = mVertexDataList[vSet];
+            mEdgeData->edgeGroups[vSet].triStart = 0;
+            mEdgeData->edgeGroups[vSet].triCount = 0;
         }
 
         // Build triangles and edge list
@@ -172,11 +195,12 @@ namespace Ogre {
             buildTrianglesEdges(*i);
         }
 
-        // Log
-        //log(LogManager::getSingleton().createLog("EdgeListBuilder.log"));
+        // Allocate memory for light facing calculate
+        mEdgeData->triangleLightFacings.resize(mEdgeData->triangles.size());
+
+        // Record closed, ie the mesh is manifold
+        mEdgeData->isClosed = mEdgeMap.empty();
 
-		// Record closed - uncomment this when .mesh format includes isClosed
-		//mEdgeData->isClosed = mEdgeMap.empty();
         return mEdgeData;
     }
     //---------------------------------------------------------------------
@@ -199,10 +223,11 @@ namespace Ogre {
             iterations = indexData->indexCount - 2;
             break;
         default:
-            break;
+            return; // Just in case
         };
 
-
+        // The edge group now we are dealing with.
+        EdgeData::EdgeGroup& eg = mEdgeData->edgeGroups[vertexSet];
 
 		// locate position element & the buffer to go with it
         const VertexData* vertexData = mVertexDataList[vertexSet];
@@ -214,9 +239,13 @@ namespace Ogre {
 			vbuf->lock(HardwareBuffer::HBL_READ_ONLY));
 
         // Get the indexes ready for reading
+		bool idx32bit = (indexData->indexBuffer->getType() == HardwareIndexBuffer::IT_32BIT);
+		size_t indexSize = idx32bit ? sizeof(uint32) : sizeof(uint16);
 #if defined(_MSC_VER) && _MSC_VER <= 1300
         // NB: Can't use un-named union with VS.NET 2002 when /RTC1 compile flag enabled.
         void* pIndex = indexData->indexBuffer->lock(HardwareBuffer::HBL_READ_ONLY);
+		pIndex = static_cast<void*>(
+			static_cast<char*>(pIndex) + indexData->indexStart * indexSize);
         unsigned short* p16Idx = static_cast<unsigned short*>(pIndex);
         unsigned int* p32Idx = static_cast<unsigned int*>(pIndex);
 #else
@@ -226,16 +255,24 @@ namespace Ogre {
             unsigned int* p32Idx;
         };
         pIndex = indexData->indexBuffer->lock(HardwareBuffer::HBL_READ_ONLY);
+		pIndex = static_cast<void*>(
+			static_cast<char*>(pIndex) + indexData->indexStart * indexSize);
 #endif
-        bool idx32bit = (indexData->indexBuffer->getType() == HardwareIndexBuffer::IT_32BIT);
 
         // Iterate over all the groups of 3 indexes
         unsigned int index[3];
         // Get the triangle start, if we have more than one index set then this
         // will not be zero
-        size_t triStart = mEdgeData->triangles.size();
+        size_t triangleIndex = mEdgeData->triangles.size();
+        // If it's first time dealing with the edge group, setup triStart for it.
+        // Note that we are assume geometries sorted by vertex set.
+        if (!eg.triCount)
+        {
+            eg.triStart = triangleIndex;
+        }
         // Pre-reserve memory for less thrashing
-        mEdgeData->triangles.reserve(triStart + iterations);
+        mEdgeData->triangles.reserve(triangleIndex + iterations);
+        mEdgeData->triangleFaceNormals.reserve(triangleIndex + iterations);
         for (size_t t = 0; t < iterations; ++t)
         {
             EdgeData::Triangle tri;
@@ -301,22 +338,28 @@ namespace Ogre {
             {
                 // Calculate triangle normal (NB will require recalculation for 
                 // skeletally animated meshes)
-                tri.normal = Math::calculateFaceNormalWithoutNormalize(v[0], v[1], v[2]);
+                mEdgeData->triangleFaceNormals.push_back(
+                    Math::calculateFaceNormalWithoutNormalize(v[0], v[1], v[2]));
                 // Add triangle to list
                 mEdgeData->triangles.push_back(tri);
                 // Connect or create edges from common list
-                connectOrCreateEdge(vertexSet, triStart + t, 
+                connectOrCreateEdge(vertexSet, triangleIndex, 
                     tri.vertIndex[0], tri.vertIndex[1], 
                     tri.sharedVertIndex[0], tri.sharedVertIndex[1]);
-                connectOrCreateEdge(vertexSet, triStart + t, 
+                connectOrCreateEdge(vertexSet, triangleIndex, 
                     tri.vertIndex[1], tri.vertIndex[2], 
                     tri.sharedVertIndex[1], tri.sharedVertIndex[2]);
-                connectOrCreateEdge(vertexSet, triStart + t, 
+                connectOrCreateEdge(vertexSet, triangleIndex, 
                     tri.vertIndex[2], tri.vertIndex[0], 
                     tri.sharedVertIndex[2], tri.sharedVertIndex[0]);
+                ++triangleIndex;
             }
         }
 
+        // Update triCount for the edge group. Note that we are assume
+        // geometries sorted by vertex set.
+        eg.triCount = triangleIndex - eg.triStart;
+
         indexData->indexBuffer->unlock();
         vbuf->unlock();
     }
@@ -349,7 +392,7 @@ namespace Ogre {
 
             // Set only first tri, the other will be completed in connect existing edge
             e.triIndex[0] = triangleIndex;
-            e.triIndex[1] = ~0;
+            e.triIndex[1] = static_cast<size_t>(~0);
             e.sharedVertIndex[0] = sharedVertIndex0;
             e.sharedVertIndex[1] = sharedVertIndex1;
             e.vertIndex[0] = vertIndex0;
@@ -385,54 +428,37 @@ namespace Ogre {
     //---------------------------------------------------------------------
     void EdgeData::updateTriangleLightFacing(const Vector4& lightPos)
     {
-        // Iterate over the triangles, and determine if they are light facing
-        EdgeData::TriangleList::iterator ti, tiend;
-        tiend = triangles.end();
-        Vector3 vertToLight;
-        for (ti = triangles.begin(); ti != tiend; ++ti)
-        {
-            EdgeData::Triangle& t = *ti;
-            // Get pointer to positions, and reference the first position
-
-            Real dp = t.normal.dotProduct(lightPos);
-            t.lightFacing = (dp > 0);
-
-        }
+        // Triangle face normals should be 1:1 with light facing flags
+        assert(triangleFaceNormals.size() == triangleLightFacings.size());
 
+        // Use optimised util to determine if triangle's face normal are light facing
+        OptimisedUtil::getImplementation()->calculateLightFacing(
+            lightPos,
+            &triangleFaceNormals.front(),
+            &triangleLightFacings.front(),
+            triangleLightFacings.size());
     }
     //---------------------------------------------------------------------
     void EdgeData::updateFaceNormals(size_t vertexSet, 
-        HardwareVertexBufferSharedPtr positionBuffer)
+        const HardwareVertexBufferSharedPtr& positionBuffer)
     {
         assert (positionBuffer->getVertexSize() == sizeof(float) * 3
             && "Position buffer should contain only positions!");
 
+        // Triangle face normals should be 1:1 with triangles
+        assert(triangleFaceNormals.size() == triangles.size());
+
         // Lock buffer for reading
         float* pVert = static_cast<float*>(
             positionBuffer->lock(HardwareBuffer::HBL_READ_ONLY));
 
-        // Iterate over the triangles
-        EdgeData::TriangleList::iterator i, iend;
-        iend = triangles.end();
-        for (i = triangles.begin(); i != iend; ++i)
-        {
-            Triangle& t = *i;
-            // Only update tris which are using this vertex set
-            if (t.vertexSet == vertexSet)
-            {
-                size_t offset = t.vertIndex[0]*3;
-                Vector3 v1(pVert[offset], pVert[offset+1], pVert[offset+2]);
-
-                offset = t.vertIndex[1]*3;
-                Vector3 v2(pVert[offset], pVert[offset+1], pVert[offset+2]);
-
-                offset = t.vertIndex[2]*3;
-                Vector3 v3(pVert[offset], pVert[offset+1], pVert[offset+2]);
-
-                t.normal = Math::calculateFaceNormalWithoutNormalize(v1, v2, v3);
-            }
-        }
-
+        // Calculate triangles which are using this vertex set
+        const EdgeData::EdgeGroup& eg = edgeGroups[vertexSet];
+        OptimisedUtil::getImplementation()->calculateFaceNormals(
+            pVert,
+            &triangles[eg.triStart],
+            &triangleFaceNormals[eg.triStart],
+            eg.triCount);
 
         // unlock the buffer
         positionBuffer->unlock();
@@ -505,10 +531,13 @@ namespace Ogre {
                     if (mGeometryList[i].opType == RenderOperation::OT_TRIANGLE_LIST
                         || j == 0)
                     {
+		    	unsigned int n1 = *p32Idx++;
+		    	unsigned int n2 = *p32Idx++;
+		    	unsigned int n3 = *p32Idx++;
                         l->logMessage("Triangle " + StringConverter::toString(j) + 
-                            ": (" + StringConverter::toString(*p32Idx++) + 
-                            ", " + StringConverter::toString(*p32Idx++) + 
-                            ", " + StringConverter::toString(*p32Idx++) + ")");
+                            ": (" + StringConverter::toString(n1) + 
+                            ", " + StringConverter::toString(n2) + 
+                            ", " + StringConverter::toString(n3) + ")");
                         j += 3;
                     }
                     else
@@ -523,10 +552,13 @@ namespace Ogre {
                     if (mGeometryList[i].opType == RenderOperation::OT_TRIANGLE_LIST
                         || j == 0)
                     {
+		    	unsigned short n1 = *p16Idx++;
+		    	unsigned short n2 = *p16Idx++;
+		    	unsigned short n3 = *p16Idx++;
                         l->logMessage("Index " + StringConverter::toString(j) + 
-                            ": (" + StringConverter::toString(*p16Idx++) + 
-                            ", " + StringConverter::toString(*p16Idx++) + 
-                            ", " + StringConverter::toString(*p16Idx++) + ")");
+                            ": (" + StringConverter::toString(n1) + 
+                            ", " + StringConverter::toString(n2) + 
+                            ", " + StringConverter::toString(n3) + ")");
                         j += 3;
                     }
                     else
