@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
@@ -29,19 +33,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreImageCodec.h"
 #include "OgreColourValue.h"
 
-/* Use new scaling code when possible */
-#define NEWSCALING
-
-#ifdef NEWSCALING
 #include "OgreImageResampler.h"
-#endif
-
-#if OGRE_NO_DEVIL == 0
-#include "OgreILUtil.h"
-// Dependency on IL/ILU for resize
-#include <IL/il.h>
-#include <IL/ilu.h>
-#endif
 
 namespace Ogre {
 	ImageCodec::~ImageCodec() {
@@ -114,8 +106,6 @@ namespace Ogre {
 	//-----------------------------------------------------------------------------
 	Image & Image::flipAroundY()
 	{
-		OgreGuard( "Image::flipAroundY" );
-
 		if( !m_pBuffer )
 		{
 			OGRE_EXCEPT( 
@@ -169,10 +159,10 @@ namespace Ogre {
 			pTempBuffer3 = new uchar[m_uWidth * m_uHeight * 3];
 			for (y = 0; y < m_uHeight; y++)
 			{
-				uint offset = ((y * m_uWidth) + (m_uWidth - 1)) * 3;
+				size_t offset = ((y * m_uWidth) + (m_uWidth - 1)) * 3;
 				dst3 = pTempBuffer3;
 				dst3 += offset;
-				for (ushort x = 0; x < m_uWidth; x++)
+				for (size_t x = 0; x < m_uWidth; x++)
 				{
 					memcpy(dst3, src3, sizeof(uchar) * 3);
 					dst3 -= 3; src3 += 3;
@@ -204,14 +194,13 @@ namespace Ogre {
 			break;
 		}
 
-		OgreUnguardRet( *this );
+		return *this;
+
 	}
 
 	//-----------------------------------------------------------------------------
 	Image & Image::flipAroundX()
 	{
-		OgreGuard( "Image::flipAroundX" );
-
 		if( !m_pBuffer )
 		{
 			OGRE_EXCEPT( 
@@ -237,7 +226,7 @@ namespace Ogre {
 
 		delete [] pTempBuffer;
 
-		OgreUnguardRet( *this );
+		return *this;
 	}
 
 	//-----------------------------------------------------------------------------
@@ -246,7 +235,6 @@ namespace Ogre {
 		PixelFormat eFormat, bool autoDelete, 
 		size_t numFaces, size_t numMipMaps)
 	{
-		OgreGuard( "Image::loadDynamicImage" );
 
 		if( m_pBuffer && m_bAutoDelete )
 		{
@@ -258,7 +246,7 @@ namespace Ogre {
 		m_uHeight = uHeight;
 		m_uDepth = depth;
 		m_eFormat = eFormat;
-		m_ucPixelSize = PixelUtil::getNumElemBytes( m_eFormat );
+		m_ucPixelSize = static_cast<uchar>(PixelUtil::getNumElemBytes( m_eFormat ));
 		m_uNumMipmaps = numMipMaps;
 		m_uFlags = 0;
 		// Set flags
@@ -277,7 +265,8 @@ namespace Ogre {
 		m_pBuffer = pData;
 		m_bAutoDelete = autoDelete;
 
-		OgreUnguardRet( *this );
+		return *this;
+
 	}
 
 	//-----------------------------------------------------------------------------
@@ -287,7 +276,6 @@ namespace Ogre {
 		PixelFormat eFormat,
 		size_t numFaces, size_t numMipMaps)
 	{
-		OgreGuard( "Image::loadRawData" );
 
 		size_t size = calculateSize(numMipMaps, numFaces, uWidth, uHeight, uDepth, eFormat);
 		if (size != stream->size())
@@ -300,17 +288,15 @@ namespace Ogre {
 		uchar *buffer = new uchar[ size ];
 		stream->read(buffer, size);
 
-		loadDynamicImage(buffer,
+		return loadDynamicImage(buffer,
 			uWidth, uHeight, uDepth,
 			eFormat, true, numFaces, numMipMaps);
 
-		OgreUnguardRet( *this );
 	}
 
 	//-----------------------------------------------------------------------------
 	Image & Image::load(const String& strFileName, const String& group)
 	{
-		OgreGuard( "Image::load" );
 
 		if( m_pBuffer && m_bAutoDelete )
 		{
@@ -352,7 +338,7 @@ namespace Ogre {
 		m_uSize = pData->size;
 		m_eFormat = pData->format;
 		m_uNumMipmaps = pData->num_mipmaps;
-		m_ucPixelSize = PixelUtil::getNumElemBytes( m_eFormat );
+		m_ucPixelSize = static_cast<uchar>(PixelUtil::getNumElemBytes( m_eFormat ));
 		m_uFlags = pData->flags;
 
 		// re-use the decoded buffer
@@ -360,7 +346,8 @@ namespace Ogre {
 		// ensure we don't delete when stream is closed
 		res.first->setFreeOnClose(false);
 
-		OgreUnguardRet( *this );
+		return *this;
+
 	}
 	//-----------------------------------------------------------------------------
 	void Image::save(const String& filename)
@@ -404,7 +391,6 @@ namespace Ogre {
 	//-----------------------------------------------------------------------------
 	Image & Image::load(DataStreamPtr& stream, const String& type )
 	{
-		OgreGuard( "Image::load" );
 		if( m_pBuffer && m_bAutoDelete )
 		{
 			delete[] m_pBuffer;
@@ -434,13 +420,13 @@ namespace Ogre {
 
 		// Get the format and compute the pixel size
 		m_eFormat = pData->format;
-		m_ucPixelSize = PixelUtil::getNumElemBytes( m_eFormat );
+		m_ucPixelSize = static_cast<uchar>(PixelUtil::getNumElemBytes( m_eFormat ));
 		// Just use internal buffer of returned memory stream
 		m_pBuffer = res.first->getPtr();
 		// Make sure stream does not delete
 		res.first->setFreeOnClose(false);
 
-		OgreUnguardRet( *this );
+		return *this;
 	}
 
 	//-----------------------------------------------------------------------------
@@ -567,30 +553,6 @@ namespace Ogre {
 		}
 	}
 	//-----------------------------------------------------------------------------
-#if OGRE_NO_DEVIL == 0
-	// Local declaration of DevIL functions to prevent DevIL dependencies on header users
-	ILenum getILFilter(Image::Filter filter)
-	{
-		switch (filter)
-		{
-		case Image::FILTER_NEAREST:
-			return ILU_NEAREST;
-		case Image::FILTER_LINEAR:
-			return ILU_LINEAR;
-		case Image::FILTER_BILINEAR:
-			return ILU_BILINEAR;
-		case Image::FILTER_BOX:
-			return ILU_SCALE_BOX;
-		case Image::FILTER_TRIANGLE:
-			return ILU_SCALE_TRIANGLE;
-		case Image::FILTER_BICUBIC:
-			return ILU_SCALE_BSPLINE;
-		};
-		// keep compiler happy
-		return ILU_NEAREST;
-	}
-#endif
-	//-----------------------------------------------------------------------------
 	void Image::resize(ushort width, ushort height, Filter filter)
 	{
 		// resizing dynamic images is not supported
@@ -617,12 +579,14 @@ namespace Ogre {
 	{
 		assert(PixelUtil::isAccessible(src.format));
 		assert(PixelUtil::isAccessible(scaled.format));
-#ifdef NEWSCALING		
 		MemoryDataStreamPtr buf; // For auto-delete
 		PixelBox temp;
-		switch (filter) {
+		switch (filter) 
+		{
+		default:
 		case FILTER_NEAREST:
-			if(src.format == scaled.format) {
+			if(src.format == scaled.format) 
+			{
 				// No intermediate buffer needed
 				temp = scaled;
 			}
@@ -634,7 +598,8 @@ namespace Ogre {
 				temp.data = buf->getPtr();
 			}
 			// super-optimized: no conversion
-			switch (PixelUtil::getNumElemBytes(src.format)) {
+			switch (PixelUtil::getNumElemBytes(src.format)) 
+			{
 			case 1: NearestResampler<1>::scale(src, temp); break;
 			case 2: NearestResampler<2>::scale(src, temp); break;
 			case 3: NearestResampler<3>::scale(src, temp); break;
@@ -656,13 +621,15 @@ namespace Ogre {
 
 		case FILTER_LINEAR:
 		case FILTER_BILINEAR:
-			switch (src.format) {
+			switch (src.format) 
+			{
 			case PF_L8: case PF_A8: case PF_BYTE_LA:
 			case PF_R8G8B8: case PF_B8G8R8:
 			case PF_R8G8B8A8: case PF_B8G8R8A8:
 			case PF_A8B8G8R8: case PF_A8R8G8B8:
 			case PF_X8B8G8R8: case PF_X8R8G8B8:
-				if(src.format == scaled.format) {
+				if(src.format == scaled.format) 
+				{
 					// No intermediate buffer needed
 					temp = scaled;
 				}
@@ -674,7 +641,8 @@ namespace Ogre {
 					temp.data = buf->getPtr();
 				}
 				// super-optimized: byte-oriented math, no conversion
-				switch (PixelUtil::getNumElemBytes(src.format)) {
+				switch (PixelUtil::getNumElemBytes(src.format)) 
+				{
 				case 1: LinearResampler_Byte<1>::scale(src, temp); break;
 				case 2: LinearResampler_Byte<2>::scale(src, temp); break;
 				case 3: LinearResampler_Byte<3>::scale(src, temp); break;
@@ -703,40 +671,7 @@ namespace Ogre {
 				LinearResampler::scale(src, scaled);
 			}
 			break;
-		default:
-			// fall back to old, slow, wildly incorrect DevIL code
-#endif
-#if OGRE_NO_DEVIL == 0
-			ILuint ImageName;
-			ilGenImages( 1, &ImageName );
-			ilBindImage( ImageName );
-
-			// Convert image from OGRE to current IL image
-			ILUtil::fromOgre(src);
-
-			// set filter
-			iluImageParameter(ILU_FILTER, getILFilter(filter));
-
-			// do the scaling
-			if(!iluScale(scaled.getWidth(), scaled.getHeight(), scaled.getDepth())) {
-				OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR,
-					iluErrorString(ilGetError()),
-					"Image::scale" ) ;
-			}
-			ILUtil::toOgre(scaled);
-
-			ilDeleteImages(1, &ImageName);
-
-			// return to default filter
-			iluImageParameter(ILU_FILTER, ILU_NEAREST);
-#else
-			OGRE_EXCEPT( Exception::UNIMPLEMENTED_FEATURE,
-				"Scaling algorithm not implemented without DevIL",
-				"Image::scale" ) ;	
-#endif
-#ifdef NEWSCALING
 		}
-#endif
 	}
 
 	//-----------------------------------------------------------------------------    
@@ -752,8 +687,16 @@ namespace Ogre {
 
 	PixelBox Image::getPixelBox(size_t face, size_t mipmap) const
 	{
+		// Image data is arranged as:
+		// face 0, top level (mip 0)
+		// face 0, mip 1
+		// face 0, mip 2
+		// face 1, top level (mip 0)
+		// face 1, mip 1
+		// face 1, mip 2
+		// etc
 		if(mipmap > getNumMipmaps())
-			OGRE_EXCEPT( Exception::UNIMPLEMENTED_FEATURE,
+			OGRE_EXCEPT( Exception::ERR_NOT_IMPLEMENTED,
 			"Mipmap index out of range",
 			"Image::getPixelBox" ) ;
 		if(face >= getNumFaces())
@@ -761,23 +704,35 @@ namespace Ogre {
 			"Image::getPixelBox");
         // Calculate mipmap offset and size
         uint8 *offset = const_cast<uint8*>(getData());
+		// Base offset is number of full faces
         size_t width = getWidth(), height=getHeight(), depth=getDepth();
-        size_t faceSize; // Size of one face of the image
-        for(size_t mip=0; mip<mipmap; ++mip)
+		size_t numMips = getNumMipmaps();
+
+		// Figure out the offsets 
+		size_t fullFaceSize = 0;
+		size_t finalFaceSize = 0;
+		size_t finalWidth, finalHeight, finalDepth;
+		for(size_t mip=0; mip <= numMips; ++mip)
         {
-            faceSize = PixelUtil::getMemorySize(width, height, depth, getFormat());
-            /// Skip all faces of this mipmap
-            offset += faceSize*getNumFaces(); 
+			if (mip == mipmap)
+			{
+				finalFaceSize = fullFaceSize;
+				finalWidth = width;
+				finalHeight = height;
+				finalDepth = depth;
+			}
+            fullFaceSize += PixelUtil::getMemorySize(width, height, depth, getFormat());
+
             /// Half size in each dimension
             if(width!=1) width /= 2;
             if(height!=1) height /= 2;
             if(depth!=1) depth /= 2;
         }
-		// We have advanced to the desired mipmap, offset to right face
-        faceSize = PixelUtil::getMemorySize(width, height, depth, getFormat());
-        offset += faceSize*face;
+		// Advance pointer by number of full faces, plus mip offset into
+		offset += face * fullFaceSize;
+		offset += finalFaceSize;
 		// Return subface as pixelbox
-		PixelBox src(width, height, depth, getFormat(), offset);
+		PixelBox src(finalWidth, finalHeight, finalDepth, getFormat(), offset);
 		return src;
 	}
     //-----------------------------------------------------------------------------    
