@@ -4,7 +4,7 @@ This source file is a part of OGRE
 
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This library is free software; you can redistribute it and/or modify it
@@ -48,7 +48,14 @@ namespace Ogre
 {
 
     //-----------------------------------------------------------------------------
-    MemoryManager MemoryManager::sMemManager;
+    MemoryManager& MemoryManager::instance(void)
+    {
+        // Ensure MemoryManager construct before any memory allocate,
+        // and then destruct after all memory deallocated.
+        static MemoryManager sMemManager;
+
+        return sMemManager;
+    }
     //-----------------------------------------------------------------------------
 
 #if OGRE_DEBUG_MEMORY_MANAGER && OGRE_DEBUG_MODE
@@ -86,8 +93,8 @@ namespace Ogre
     // The BEOS assert added by Arvid Norberg <arvid@iname.com>.    
     #ifdef    WIN32
         #ifdef    _DEBUG
-            #if defined(_MSC_VER)
-                #define m_assert(x)  if( (x) == false ) __asm { int 3 } 
+            #if defined(_MSC_VER) 
+                    #define m_assert(x)  if( (x) == false ) __debugbreak();                
             #elif defined(__GNUC__)
                 #define m_assert(x)  if( (x) == false ) __asm ("int $3");  
             #endif
@@ -464,7 +471,7 @@ namespace Ogre
                     ptr->allocationNumber,
                     (size_t) ptr->reportedAddress, ptr->reportedSize,
                     (size_t) ptr->actualAddress, ptr->actualSize,
-                    MemoryManager::sMemManager.calcUnused(ptr),
+                    MemoryManager::instance().calcUnused(ptr),
                     allocationTypes[ptr->allocationType],
                     ptr->breakOnDealloc ? 'Y':'N',
                     ptr->breakOnRealloc ? 'Y':'N',
@@ -658,7 +665,7 @@ namespace Ogre
     {
         // If we don't have a process ID yet, get one now
         if( !gProcessID )
-            gProcessID = sMemManager._getProcessID();
+            gProcessID = _getProcessID();
 
         try
         {
@@ -849,7 +856,7 @@ namespace Ogre
     {
         // If we don't have a process ID yet, get one now
         if( !gProcessID )
-            gProcessID = sMemManager._getProcessID();
+            gProcessID = _getProcessID();
 
         try
         {
@@ -1074,6 +1081,10 @@ namespace Ogre
 
     void MemoryManager::dllocMem(const char *sourceFile, const unsigned int sourceLine, const char *sourceFunc, const unsigned int deallocationType, const void *reportedAddress, const unsigned processID )
     {
+		// early-out for NULL
+		if (!reportedAddress)
+			return;
+
         try
         {
             // Log the request
@@ -1430,7 +1441,7 @@ namespace Ogre
         for(;;)
         {
             // Try the allocation
-            void *ptr = MemoryManager::sMemManager.allocMem(file, line, func, m_alloc_new, reportedSize, processID );
+            void *ptr = allocMem(file, line, func, m_alloc_new, reportedSize, processID );
             if( ptr )
             {
                 return ptr;
@@ -1470,7 +1481,7 @@ namespace Ogre
         for(;;)
         {
             // Try the allocation
-            void    *ptr = MemoryManager::sMemManager.allocMem(file, line, func, m_alloc_new_array, reportedSize, processID );
+            void    *ptr = allocMem(file, line, func, m_alloc_new_array, reportedSize, processID );
             if( ptr )
             {
                 return ptr;
@@ -1515,7 +1526,7 @@ namespace Ogre
         {
             // Try the allocation
 
-            void    *ptr = MemoryManager::sMemManager.allocMem(sourceFile, sourceLine, "??", m_alloc_new, reportedSize, processID );
+            void    *ptr = allocMem(sourceFile, sourceLine, "??", m_alloc_new, reportedSize, processID );
             if (ptr)
             {
                 return ptr;
@@ -1557,7 +1568,7 @@ namespace Ogre
         for(;;)
         {
             // Try the allocation
-            void *ptr = MemoryManager::sMemManager.allocMem(
+            void *ptr = allocMem(
                 sourceFile, 
                 sourceLine, 
                 "??", 
@@ -1598,7 +1609,7 @@ namespace Ogre
     {
         // ANSI says: delete & delete[] allow NULL pointers (they do nothing)
         if( reportedAddress )
-            MemoryManager::sMemManager.dllocMem(sourceFile, sourceLine, sourceFunc, m_alloc_delete, reportedAddress, processID );
+            dllocMem(sourceFile, sourceLine, sourceFunc, m_alloc_delete, reportedAddress, processID );
         else if( alwaysLogAll )
             log("[-] ----- %8s of NULL                      by %s", 
             allocationTypes[m_alloc_delete], 
@@ -1615,7 +1626,7 @@ namespace Ogre
     {
         // ANSI says: delete & delete[] allow NULL pointers (they do nothing)
         if (reportedAddress) 
-            MemoryManager::sMemManager.dllocMem(
+            dllocMem(
                 sourceFile, 
                 sourceLine, 
                 sourceFunc, 
