@@ -1,11 +1,10 @@
-
 /*
 -----------------------------------------------------------------------------
 This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software you can redistribute it and/or modify it under
@@ -21,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
@@ -40,11 +43,17 @@ namespace Ogre {
     //---------------------------------------------------------------------
     OverlayContainer::~OverlayContainer()
     {
+		// remove from parent overlay if root
+		if (mOverlay && !mParent)
+		{
+			mOverlay->remove2D(this);
+		}
+
         OverlayContainer::ChildIterator ci = getChildIterator();
         while (ci.hasMoreElements())
         {
             OverlayElement* child = ci.getNext();
-            child->_setParent(0);
+			child->_notifyParent(0, 0);
         }
     }
     //---------------------------------------------------------------------
@@ -345,7 +354,7 @@ namespace Ogre {
 						 OverlayManager::getSingleton().createOverlayElement(
 							oldChildElement->getTypeName(), 
 							mName+"/"+oldChildElement->getName());
-					 oldChildElement->copyParametersTo(newChildElement);
+					 newChildElement->copyFromTemplate(oldChildElement);
 					 addChild((OverlayContainer*)newChildElement);
 				 }
 			 }
