@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
@@ -44,7 +48,12 @@ namespace Ogre {
         assert( ms_Singleton );  return ( *ms_Singleton );  
     }
 	String ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME = "General";
+	String ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME = "Internal";
 	String ResourceGroupManager::BOOTSTRAP_RESOURCE_GROUP_NAME = "Bootstrap";
+	String ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME = "Autodetect";
+	// A reference count of 3 means that only RGM and RM have references
+	// RGM has one (this one) and RM has 2 (by name and by handle)
+	size_t ResourceGroupManager::RESOURCE_SYSTEM_NUM_REFERENCE_COUNTS = 3;
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
     ResourceGroupManager::ResourceGroupManager()
@@ -52,6 +61,10 @@ namespace Ogre {
     {
         // Create the 'General' group
         createResourceGroup(DEFAULT_RESOURCE_GROUP_NAME);
+        // Create the 'Internal' group
+        createResourceGroup(INTERNAL_RESOURCE_GROUP_NAME);
+		// Create the 'Autodetect' group (only used for temp storage)
+		createResourceGroup(AUTODETECT_RESOURCE_GROUP_NAME);
         // default world group to the default group
         mWorldGroupName = DEFAULT_RESOURCE_GROUP_NAME;
     }
@@ -103,8 +116,8 @@ namespace Ogre {
 		if (!grp->initialised)
 		{
 			// Set current group
-			mCurrentGroup = grp;
 			parseResourceGroupScripts(grp);
+			mCurrentGroup = grp;
 			createDeclaredResources(grp);
 			grp->initialised = true;
 
@@ -186,17 +199,32 @@ namespace Ogre {
 			for (oi = grp->loadResourceOrderMap.begin(); 
 				oi != grp->loadResourceOrderMap.end(); ++oi)
 			{
+				size_t n = 0;
 				for (LoadUnloadResourceList::iterator l = oi->second->begin();
-					l != oi->second->end(); ++l)
+					l != oi->second->end(); ++l, ++n)
 				{
+					ResourcePtr res = *l;
+
+					// Fire resource events no matter whether resource is already
+					// loaded or not. This ensures that the number of callbacks
+					// matches the number originally estimated, which is important
+					// for progress bars.
+					fireResourceStarted(res);
+
 					// If loading one of these resources cascade-loads another resource, 
 					// the list will get longer! But these should be loaded immediately
-					if (!(*l)->isLoaded())
+					// Call load regardless, already loaded resources will be skipped
+					res->load();
+
+					// Did the resource change group? if so, our iterator will have
+					// been invalidated
+					if (res->getGroup() != name)
 					{
-						fireResourceStarted(*l);
-						(*l)->load();
-						fireResourceEnded();
+						l = oi->second->begin();
+						std::advance(l, n);
 					}
+
+					fireResourceEnded();
 				}
 			}
 		}
@@ -214,7 +242,7 @@ namespace Ogre {
 		LogManager::getSingleton().logMessage("Finished loading resource group " + name);
     }
     //-----------------------------------------------------------------------
-    void ResourceGroupManager::unloadResourceGroup(const String& name)
+    void ResourceGroupManager::unloadResourceGroup(const String& name, bool reloadableOnly)
     {
 		// Can only bulk-unload one group at a time (reasonable limitation I think)
 		OGRE_LOCK_AUTO_MUTEX
@@ -229,6 +257,7 @@ namespace Ogre {
 		}
 		// Set current group
 		mCurrentGroup = grp;
+		OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME) // lock group mutex 
 
 		// Count up resources for starting event
 		ResourceGroup::LoadResourceOrderMap::reverse_iterator oi;
@@ -238,7 +267,11 @@ namespace Ogre {
 			for (LoadUnloadResourceList::iterator l = oi->second->begin();
 				l != oi->second->end(); ++l)
 			{
-				(*l)->unload();
+				Resource* resource = l->get();
+				if (!reloadableOnly || resource->isReloadable())
+				{
+					resource->unload();
+				}
 			}
 		}
 
@@ -246,6 +279,50 @@ namespace Ogre {
 		mCurrentGroup = 0;
 		LogManager::getSingleton().logMessage("Finished unloading resource group " + name);
     }
+	//-----------------------------------------------------------------------
+	void ResourceGroupManager::unloadUnreferencedResourcesInGroup(
+		const String& name, bool reloadableOnly )
+	{
+		// Can only bulk-unload one group at a time (reasonable limitation I think)
+		OGRE_LOCK_AUTO_MUTEX
+
+		LogManager::getSingleton().logMessage(
+			"Unloading unused resources in resource group " + name);
+		ResourceGroup* grp = getResourceGroup(name);
+		if (!grp)
+		{
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+				"Cannot find a group named " + name, 
+				"ResourceGroupManager::unloadUnreferencedResourcesInGroup");
+		}
+		// Set current group
+		mCurrentGroup = grp;
+
+		ResourceGroup::LoadResourceOrderMap::reverse_iterator oi;
+		// unload in reverse order
+		for (oi = grp->loadResourceOrderMap.rbegin(); oi != grp->loadResourceOrderMap.rend(); ++oi)
+		{
+			for (LoadUnloadResourceList::iterator l = oi->second->begin();
+				l != oi->second->end(); ++l)
+			{
+				// A use count of 3 means that only RGM and RM have references
+				// RGM has one (this one) and RM has 2 (by name and by handle)
+				if (l->useCount() == RESOURCE_SYSTEM_NUM_REFERENCE_COUNTS)
+				{
+					Resource* resource = l->get();
+					if (!reloadableOnly || resource->isReloadable())
+					{
+						resource->unload();
+					}
+				}
+			}
+		}
+
+		// reset current group
+		mCurrentGroup = 0;
+		LogManager::getSingleton().logMessage(
+			"Finished unloading unused resources in resource group " + name);
+	}
 	//-----------------------------------------------------------------------
 	void ResourceGroupManager::clearResourceGroup(const String& name)
 	{
@@ -285,7 +362,7 @@ namespace Ogre {
 		}
 		// set current group
 		mCurrentGroup = grp;
-        unloadResourceGroup(name); // will throw an exception if name not valid
+        unloadResourceGroup(name, false); // will throw an exception if name not valid
 		dropGroupContents(grp);
 		deleteGroup(grp);
         mResourceGroupMap.erase(mResourceGroupMap.find(name));
@@ -401,6 +478,14 @@ namespace Ogre {
     void ResourceGroupManager::declareResource(const String& name, 
         const String& resourceType, const String& groupName,
 		const NameValuePairList& loadParameters)
+    {
+        declareResource(name, resourceType, groupName, 0, loadParameters);
+    }
+    //-----------------------------------------------------------------------
+    void ResourceGroupManager::declareResource(const String& name, 
+        const String& resourceType, const String& groupName,
+        ManualResourceLoader* loader,
+        const NameValuePairList& loadParameters)
     {
 		ResourceGroup* grp = getResourceGroup(groupName);
 		if (!grp)
@@ -413,6 +498,7 @@ namespace Ogre {
 		OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME) // lock group mutex
 
 		ResourceDeclaration dcl;
+		dcl.loader = loader;
 		dcl.parameters = loadParameters;
 		dcl.resourceName = name;
 		dcl.resourceType = resourceType;
@@ -444,14 +530,16 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     DataStreamPtr ResourceGroupManager::openResource(
-        const String& resourceName, const String& groupName)
+        const String& resourceName, const String& groupName, 
+		bool searchGroupsIfNotFound, Resource* resourceBeingLoaded)
     {
 		// Try to find in resource index first
 		ResourceGroup* grp = getResourceGroup(groupName);
 		if (!grp)
 		{
 			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
-				"Cannot locate a resource group called '" + groupName + "'", 
+				"Cannot locate a resource group called '" + groupName + 
+				"' for resource '" + resourceName + "'" , 
 				"ResourceGroupManager::openResource");
 		}
 
@@ -496,6 +584,25 @@ namespace Ogre {
 
 		
 		// Not found
+		if (searchGroupsIfNotFound)
+		{
+			ResourceGroup* grp = findGroupContainingResourceImpl(resourceName); 
+			if (grp)
+			{
+				if (resourceBeingLoaded)
+				{
+					resourceBeingLoaded->changeGroupOwnership(grp->name);
+				}
+				return openResource(resourceName, grp->name, false);
+			}
+			else
+			{
+				OGRE_EXCEPT(Exception::ERR_FILE_NOT_FOUND, 
+					"Cannot locate resource " + resourceName + 
+					" in resource group " + groupName + " or any other group.", 
+					"ResourceGroupManager::openResource");
+			}
+		}
 		OGRE_EXCEPT(Exception::ERR_FILE_NOT_FOUND, "Cannot locate resource " + 
 			resourceName + " in resource group " + groupName + ".", 
 			"ResourceGroupManager::openResource");
@@ -678,7 +785,7 @@ namespace Ogre {
                             su->parseScript(stream, grp->name);
                         }
                     }
-				    fireScriptEnded();
+				    fireScriptEnded(fii->filename);
 			    }
             }
 		}
@@ -698,9 +805,8 @@ namespace Ogre {
 			// Retrieve the appropriate manager
 			ResourceManager* mgr = _getResourceManager(dcl.resourceType);
 			// Create the resource
-			ResourcePtr res = mgr->create(dcl.resourceName, grp->name);
-			// Set custom parameters
-			res->setParameterList(dcl.parameters);
+			ResourcePtr res = mgr->create(dcl.resourceName, grp->name,
+                dcl.loader != 0, dcl.loader, &dcl.parameters);
 			// Add resource to load list
 			ResourceGroup::LoadResourceOrderMap::iterator li = 
 				grp->loadResourceOrderMap.find(mgr->getLoadingOrder());
@@ -722,7 +828,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
 	void ResourceGroupManager::_notifyResourceCreated(ResourcePtr& res)
 	{
-		if (mCurrentGroup)
+		if (mCurrentGroup && res->getGroup() == mCurrentGroup->name)
 		{
 			// Use current group (batch loading)
 			addCreatedResource(res, *mCurrentGroup);
@@ -773,6 +879,36 @@ namespace Ogre {
 		}
 	}
 	//-----------------------------------------------------------------------
+	void ResourceGroupManager::_notifyResourceGroupChanged(const String& oldGroup, 
+		Resource* res)
+	{
+		OGRE_LOCK_AUTO_MUTEX
+	
+		// New group
+		ResourceGroup* newGrp = getResourceGroup(res->getGroup());
+		// find old entry
+		ResourceGroupMap::iterator grpi = mResourceGroupMap.find(oldGroup);
+
+		assert(grpi != mResourceGroupMap.end());
+		ResourceGroup* grp = grpi->second;
+		Real order = res->getCreator()->getLoadingOrder();
+		ResourceGroup::LoadResourceOrderMap::iterator i = 
+			grp->loadResourceOrderMap.find(order);
+		assert(i != grp->loadResourceOrderMap.end());
+		LoadUnloadResourceList* loadList = i->second;
+		for (LoadUnloadResourceList::iterator l = loadList->begin(); 
+			l != loadList->end(); ++l)
+		{
+			if ((*l).getPointer() == res)
+			{
+				addCreatedResource(*l, *newGrp);
+				loadList->erase(l);
+				break;
+			}
+		}
+
+	}
+	//-----------------------------------------------------------------------
 	void ResourceGroupManager::_notifyAllResourcesRemoved(ResourceManager* manager)
 	{
 		OGRE_LOCK_AUTO_MUTEX
@@ -887,22 +1023,24 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	void ResourceGroupManager::deleteGroup(ResourceGroup* grp)
 	{
-		OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME)
-		// delete all the load list entries
-		ResourceGroup::LoadResourceOrderMap::iterator j, jend;
-		jend = grp->loadResourceOrderMap.end();
-		for (j = grp->loadResourceOrderMap.begin(); j != jend; ++j)
-		{
-			// Don't iterate over resources to drop with ResourceManager
-			// Assume this is being done anyway since this is a shutdown method
-			delete j->second;
-		}
-		// Drop location list
-		for (LocationList::iterator ll = grp->locationList.begin();
-			ll != grp->locationList.end(); ++ll)
-		{
-			delete *ll;
-		}
+        {
+		    OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME)
+		    // delete all the load list entries
+		    ResourceGroup::LoadResourceOrderMap::iterator j, jend;
+		    jend = grp->loadResourceOrderMap.end();
+		    for (j = grp->loadResourceOrderMap.begin(); j != jend; ++j)
+		    {
+			    // Don't iterate over resources to drop with ResourceManager
+			    // Assume this is being done anyway since this is a shutdown method
+			    delete j->second;
+		    }
+		    // Drop location list
+		    for (LocationList::iterator ll = grp->locationList.begin();
+			    ll != grp->locationList.end(); ++ll)
+		    {
+			    delete *ll;
+		    }
+        }
 
 		// delete ResourceGroup
 		delete grp;
@@ -928,13 +1066,13 @@ namespace Ogre {
 		}
 	}
     //-----------------------------------------------------------------------
-    void ResourceGroupManager::fireScriptEnded(void)
+    void ResourceGroupManager::fireScriptEnded(const String& scriptName)
     {
         OGRE_LOCK_AUTO_MUTEX
             for (ResourceGroupListenerList::iterator l = mResourceGroupListenerList.begin();
                 l != mResourceGroupListenerList.end(); ++l)
             {
-                (*l)->scriptParseEnded();
+                (*l)->scriptParseEnded(scriptName);
             }
     }
 	//-----------------------------------------------------------------------
@@ -1020,7 +1158,7 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    StringVectorPtr ResourceGroupManager::listResourceNames(const String& groupName)
+    StringVectorPtr ResourceGroupManager::listResourceNames(const String& groupName, bool dirs)
     {
         OGRE_LOCK_AUTO_MUTEX
         StringVectorPtr vec(new StringVector());
@@ -1041,7 +1179,7 @@ namespace Ogre {
         iend = grp->locationList.end();
         for (i = grp->locationList.begin(); i != iend; ++i)
         {
-            StringVectorPtr lst = (*i)->archive->list((*i)->recursive);
+            StringVectorPtr lst = (*i)->archive->list((*i)->recursive, dirs);
             vec->insert(vec->end(), lst->begin(), lst->end());
         }
 
@@ -1050,7 +1188,7 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------
-    FileInfoListPtr ResourceGroupManager::listResourceFileInfo(const String& groupName)
+    FileInfoListPtr ResourceGroupManager::listResourceFileInfo(const String& groupName, bool dirs)
     {
         OGRE_LOCK_AUTO_MUTEX
         FileInfoListPtr vec(new FileInfoList());
@@ -1071,7 +1209,7 @@ namespace Ogre {
         iend = grp->locationList.end();
         for (i = grp->locationList.begin(); i != iend; ++i)
         {
-            FileInfoListPtr lst = (*i)->archive->listFileInfo((*i)->recursive);
+            FileInfoListPtr lst = (*i)->archive->listFileInfo((*i)->recursive, dirs);
             vec->insert(vec->end(), lst->begin(), lst->end());
         }
 
@@ -1080,7 +1218,7 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     StringVectorPtr ResourceGroupManager::findResourceNames(const String& groupName, 
-        const String& pattern)
+        const String& pattern, bool dirs)
     {
         OGRE_LOCK_AUTO_MUTEX
         StringVectorPtr vec(new StringVector());
@@ -1101,7 +1239,7 @@ namespace Ogre {
         iend = grp->locationList.end();
         for (i = grp->locationList.begin(); i != iend; ++i)
         {
-            StringVectorPtr lst = (*i)->archive->find(pattern, (*i)->recursive);
+            StringVectorPtr lst = (*i)->archive->find(pattern, (*i)->recursive, dirs);
             vec->insert(vec->end(), lst->begin(), lst->end());
         }
 
@@ -1109,7 +1247,7 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     FileInfoListPtr ResourceGroupManager::findResourceFileInfo(const String& groupName, 
-        const String& pattern)
+        const String& pattern, bool dirs)
     {
         OGRE_LOCK_AUTO_MUTEX
         FileInfoListPtr vec(new FileInfoList());
@@ -1130,7 +1268,7 @@ namespace Ogre {
         iend = grp->locationList.end();
         for (i = grp->locationList.begin(); i != iend; ++i)
         {
-            FileInfoListPtr lst = (*i)->archive->findFileInfo(pattern, (*i)->recursive);
+            FileInfoListPtr lst = (*i)->archive->findFileInfo(pattern, (*i)->recursive, dirs);
             vec->insert(vec->end(), lst->begin(), lst->end());
         }
 
@@ -1150,6 +1288,12 @@ namespace Ogre {
                 "ResourceGroupManager::resourceExists");
         }
 
+		return resourceExists(grp, resourceName);
+	}
+    //-----------------------------------------------------------------------
+	bool ResourceGroupManager::resourceExists(ResourceGroup* grp, const String& resourceName)
+	{
+
 		OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME) // lock group mutex
 
 		// Try indexes first
@@ -1189,6 +1333,40 @@ namespace Ogre {
 		return false;
 
 	}
+    //-----------------------------------------------------------------------
+	ResourceGroupManager::ResourceGroup* 
+	ResourceGroupManager::findGroupContainingResourceImpl(const String& filename)
+	{
+        OGRE_LOCK_AUTO_MUTEX
+
+			// Iterate over resource groups and find
+		for (ResourceGroupMap::iterator i = mResourceGroupMap.begin();
+			i != mResourceGroupMap.end(); ++i)
+		{
+	        ResourceGroup* grp = i->second;
+
+			OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME) // lock group mutex
+				
+			if (resourceExists(grp, filename))
+				return grp;
+		}
+		// Not found
+		return 0;
+	}
+    //-----------------------------------------------------------------------
+	const String& ResourceGroupManager::findGroupContainingResource(const String& filename)
+	{
+		ResourceGroup* grp = findGroupContainingResourceImpl(filename);
+		if (!grp)
+		{
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,
+				"Unable to derive resource group for " + 
+				filename + " automatically since the resource was not "
+				"found.", 
+				"ResourceGroupManager::findGroupContainingResource");
+		}
+		return grp->name;
+	}
     //-----------------------------------------------------------------------
     void ResourceGroupManager::linkWorldGeometryToResourceGroup(const String& group, 
         const String& worldGeometry, SceneManager* sceneManager)
@@ -1245,13 +1423,13 @@ namespace Ogre {
         {
             OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
                 "Cannot locate a resource group called '" + group + "'", 
-                "ResourceGroupManager::unlinkWorldGeometryFromResourceGroup");
+                "ResourceGroupManager::getResourceDeclarationList");
         }
 
 		OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME) // lock group mutex
 		return grp->resourceDeclarations;
 	}
-    //-----------------------------------------------------------------------
+	//-----------------------------------------------------------------------
 	ScriptLoader::~ScriptLoader()
 	{
 	}
