@@ -2,9 +2,9 @@
 -----------------------------------------------------------------------------
 This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
-For the latest info, see http://ogre.sourceforge.net/
+For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 // Ogre includes
@@ -47,16 +51,31 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreErrorDialog.h"
 #include "OgreConfigDialog.h"
 #include "OgreStringConverter.h"
-#include "OgrePlatformManager.h"
 #include "OgreArchiveManager.h"
+#include "OgrePlugin.h"
 #include "OgreZip.h"
 #include "OgreFileSystem.h"
 #include "OgreShadowVolumeExtrudeProgram.h"
 #include "OgreResourceBackgroundQueue.h"
-
+#include "OgreEntity.h"
+#include "OgreBillboardSet.h"
+#include "OgreBillboardChain.h"
+#include "OgreRibbonTrail.h"
+#include "OgreLight.h"
+#include "OgreManualObject.h"
+#include "OgreRenderQueueInvocation.h"
+#include "OgrePlatformInformation.h"
+#include "OgreConvexBody.h"
+	
+#if OGRE_NO_FREEIMAGE == 0
+#include "OgreFreeImageCodec.h"
+#endif
 #if OGRE_NO_DEVIL == 0
 #include "OgreILCodecs.h"
 #endif
+#if OGRE_NO_DDS_CODEC == 0
+#include "OgreDDSCodec.h"
+#endif
 
 #include "OgreFontManager.h"
 #include "OgreHardwareBufferManager.h"
@@ -65,17 +84,11 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreHighLevelGpuProgramManager.h"
 
 #include "OgreExternalTextureSourceManager.h"
+#include "OgreCompositorManager.h"
 
-#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
+#include "OgreWindowEventUtilities.h"
 
-#   define WIN32_LEAN_AND_MEAN
-#   include <direct.h>
-#   include <windows.h>
-
-#endif
-
-
-namespace Ogre {    
+namespace Ogre {
     //-----------------------------------------------------------------------
     template<> Root* Singleton<Root>::ms_Singleton = 0;
     Root* Root::getSingletonPtr(void)
@@ -83,8 +96,8 @@ namespace Ogre {
         return ms_Singleton;
     }
     Root& Root::getSingleton(void)
-    {  
-        assert( ms_Singleton );  return ( *ms_Singleton );  
+    {
+        assert( ms_Singleton );  return ( *ms_Singleton );
     }
 
     typedef void (*DLL_START_PLUGIN)(void);
@@ -92,53 +105,25 @@ namespace Ogre {
 
 
     //-----------------------------------------------------------------------
-    // Termination handler
-    extern "C" _OgreExport void handleTerminate(void)
+    Root::Root(const String& pluginFileName, const String& configFileName, 
+		const String& logFileName)
+      : mLogManager(0), mCurrentFrame(0), mFrameSmoothingTime(0.0f),
+	  mNextMovableObjectTypeFlag(1), mIsInitialised(false)
     {
-        LogManager::getSingleton().logMessage("Termination handler: uncaught exception!", LML_CRITICAL);
-
-        Root::getSingleton().shutdown();
-
-        ErrorDialog* dlg = PlatformManager::getSingleton().createErrorDialog();
-
-        Exception* e = Exception::getLastException();
-
-        if (e)
-            dlg->display(e->getFullDescription());
-        else
-            dlg->display("Unknown");
-
-        // Abort
-        exit(-1);
-
-    }
-
-    void Root::termHandler()
-    {
-        handleTerminate();
-    }
-
-
-    //-----------------------------------------------------------------------
-    Root::Root(const String& pluginFileName, const String& configFileName, const String& logFileName)
-      : mLogManager(0), mCurrentFrame(0)
-    {
-        // First create new exception handler
-        SET_TERM_HANDLER;
-
         // superclass will do singleton checking
         String msg;
 
         // Init
         mActiveRenderer = 0;
-        mVersion = StringConverter::toString(OGRE_VERSION_MAJOR) + "." + 
-            StringConverter::toString(OGRE_VERSION_MINOR) + "." + 
-            StringConverter::toString(OGRE_VERSION_PATCH) + " " + 
+        mVersion = StringConverter::toString(OGRE_VERSION_MAJOR) + "." +
+            StringConverter::toString(OGRE_VERSION_MINOR) + "." +
+            StringConverter::toString(OGRE_VERSION_PATCH) + 
+			OGRE_VERSION_SUFFIX + " " +
             "(" + OGRE_VERSION_NAME + ")";
 		mConfigFileName = configFileName;
 
 		// Create log manager and default log file if there is no log manager yet
-		if(LogManager::getSingletonPtr() == 0) 
+		if(LogManager::getSingletonPtr() == 0)
 		{
 			mLogManager = new LogManager();
 			mLogManager->createLog(logFileName, true, true);
@@ -159,6 +144,8 @@ namespace Ogre {
         mSceneManagerEnum = new SceneManagerEnumerator();
         mCurrentSceneManager = NULL;
 
+		mShadowTextureManager = new ShadowTextureManager();
+
         // ..material manager
         mMaterialManager = new MaterialManager();
 
@@ -171,11 +158,7 @@ namespace Ogre {
         // ..particle system manager
         mParticleManager = new ParticleSystemManager();
 
-        // Platform manager
-        mPlatformManager = new PlatformManager();
-
-        // Timer
-        mTimer = mPlatformManager->createTimer();
+        mTimer = new Timer();
 
         // Overlay manager
         mOverlayManager = new OverlayManager();
@@ -200,21 +183,44 @@ namespace Ogre {
         ArchiveManager::getSingleton().addArchiveFactory( mFileSystemArchiveFactory );
         mZipArchiveFactory = new ZipArchiveFactory();
         ArchiveManager::getSingleton().addArchiveFactory( mZipArchiveFactory );
+#if OGRE_NO_FREEIMAGE == 0
+		// Register image codecs
+		FreeImageCodec::startup();
+#endif
 #if OGRE_NO_DEVIL == 0
 	    // Register image codecs
-	    ILCodecs::registerCodecs();	
+	    ILCodecs::registerCodecs();
+#endif
+#if OGRE_NO_DDS_CODEC == 0
+		// Register image codecs
+		DDSCodec::startup();
 #endif
 
         mHighLevelGpuProgramManager = new HighLevelGpuProgramManager();
 
 		mExternalTextureSourceManager = new ExternalTextureSourceManager();
+        mCompositorManager = new CompositorManager();
 
         // Auto window
         mAutoWindow = 0;
 
-        // Load plugins
+		// instantiate and register base movable factories
+		mEntityFactory = new EntityFactory();
+		addMovableObjectFactory(mEntityFactory);
+		mLightFactory = new LightFactory();
+		addMovableObjectFactory(mLightFactory);
+		mBillboardSetFactory = new BillboardSetFactory();
+		addMovableObjectFactory(mBillboardSetFactory);
+		mManualObjectFactory = new ManualObjectFactory();
+		addMovableObjectFactory(mManualObjectFactory);
+		mBillboardChainFactory = new BillboardChainFactory();
+		addMovableObjectFactory(mBillboardChainFactory);
+		mRibbonTrailFactory = new RibbonTrailFactory();
+		addMovableObjectFactory(mRibbonTrailFactory);
+
+		// Load plugins
         if (!pluginFileName.empty())
-            loadPlugins(pluginFileName);        
+            loadPlugins(pluginFileName);
 
 		LogManager::getSingleton().logMessage("*-*-* OGRE Initialising");
         msg = "*-*-* Version " + mVersion;
@@ -230,14 +236,22 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Root::~Root()
     {
-		shutdownPlugins();
         shutdown();
         delete mSceneManagerEnum;
+		delete mShadowTextureManager;
 
+		destroyAllRenderQueueInvocationSequences();
+        delete mCompositorManager;
 		delete mExternalTextureSourceManager;
+#if OGRE_NO_FREEIMAGE == 0
+		FreeImageCodec::shutdown();
+#endif
 #if OGRE_NO_DEVIL == 0
         ILCodecs::deleteCodecs();
 #endif
+#if OGRE_NO_DDS_CODEC == 0
+		DDSCodec::shutdown();
+#endif
 #if OGRE_PROFILING
         delete mProfiler;
 #endif
@@ -260,18 +274,26 @@ namespace Ogre {
         delete mPanelFactory;
 
         unloadPlugins();
-        delete mMaterialManager;        
+        delete mMaterialManager;
         Pass::processPendingPassUpdates(); // make sure passes are cleaned
 		delete mResourceBackgroundQueue;
         delete mResourceGroupManager;
 
+		delete mEntityFactory;
+		delete mLightFactory;
+		delete mBillboardSetFactory;
+		delete mManualObjectFactory;
+		delete mBillboardChainFactory;
+		delete mRibbonTrailFactory;
 
+	delete mTimer;
 
-        mPlatformManager->destroyTimer(mTimer);
-        delete mPlatformManager;
         delete mDynLibManager;
         delete mLogManager;
 
+		mAutoWindow = 0;
+		mFirstTimePostWindowInit = false;
+
 
         StringInterface::cleanupDictionary ();
     }
@@ -279,6 +301,9 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Root::saveConfig(void)
     {
+        if (mConfigFileName.empty ())
+            return;
+
 		std::ofstream of(mConfigFileName.c_str());
 
         if (!of)
@@ -287,17 +312,23 @@ namespace Ogre {
 
         if (mActiveRenderer)
         {
-            of << "Render System\t" << mActiveRenderer->getName() << std::endl;;
-
-            ConfigOptionMap& opts = mActiveRenderer->getConfigOptions();
-            for(  ConfigOptionMap::iterator pOpt = opts.begin(); pOpt != opts.end(); ++pOpt )
-            {
-				of << pOpt->first << "\t" << pOpt->second.currentValue << std::endl;;
-            }
+            of << "Render System=" << mActiveRenderer->getName() << std::endl;
         }
         else
         {
-            of << "Render System\t ";
+            of << "Render System=" << std::endl;
+        }
+
+        for (RenderSystemList::const_iterator pRend = getAvailableRenderers()->begin(); pRend != getAvailableRenderers()->end(); ++pRend)
+        {
+            RenderSystem* rs = *pRend;
+            of << std::endl;
+            of << "[" << rs->getName() << "]" << std::endl;
+            const ConfigOptionMap& opts = rs->getConfigOptions();
+            for (ConfigOptionMap::const_iterator pOpt = opts.begin(); pOpt != opts.end(); ++pOpt)
+            {
+				of << pOpt->first << "=" << pOpt->second.currentValue << std::endl;
+            }
         }
 
         of.close();
@@ -306,13 +337,15 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     bool Root::restoreConfig(void)
     {
+        if (mConfigFileName.empty ())
+            return true;
+
         // Restores configuration from saved state
         // Returns true if a valid saved configuration is
         //   available, and false if no saved config is
         //   stored, or if there has been a problem
         ConfigFile cfg;
-        String renderSystem;
-        RenderSystemList::iterator pRend;
+        //RenderSystemList::iterator pRend;
 
         try {
             // Don't trim whitespace
@@ -330,42 +363,34 @@ namespace Ogre {
             }
         }
 
-        renderSystem = cfg.getSetting("Render System");
-        if(renderSystem.empty())
+        ConfigFile::SectionIterator iSection = cfg.getSectionIterator();
+        while (iSection.hasMoreElements())
         {
-            // No render system entry - error
-            return false;
-        }
-		
-        pRend = getAvailableRenderers()->begin();
-        while (pRend != getAvailableRenderers()->end())
-        {
-            String rName = (*pRend)->getName();
-            if (rName == renderSystem)
-                break;
-            pRend++;
+            const String& renderSystem = iSection.peekNextKey();
+            const ConfigFile::SettingsMultiMap& settings = *iSection.getNext();
+
+            RenderSystem* rs = getRenderSystemByName(renderSystem);
+            if (!rs)
+            {
+                // Unrecognised render system
+                continue;
+            }
+
+            ConfigFile::SettingsMultiMap::const_iterator i;
+            for (i = settings.begin(); i != settings.end(); ++i)
+            {
+                rs->setConfigOption(i->first, i->second);
+            }
         }
 
-        if (pRend == getAvailableRenderers()->end())
+        RenderSystem* rs = getRenderSystemByName(cfg.getSetting("Render System"));
+        if (!rs)
         {
             // Unrecognised render system
             return false;
         }
 
-        setRenderSystem(*pRend);
-
-        ConfigFile::SettingsIterator i = cfg.getSettingsIterator();
-
-        String optName, optVal;
-        while (i.hasMoreElements())
-        {
-            optName = i.peekNextKey();
-            optVal = i.getNext();
-            if(optName != "Render System")
-            {
-                mActiveRenderer->setConfigOption(optName, optVal);
-            }
-        }
+        setRenderSystem(rs);
 
         // Successful load
         return true;
@@ -380,14 +405,15 @@ namespace Ogre {
         ConfigDialog* dlg;
         bool isOk;
 
-        dlg = mPlatformManager->createConfigDialog();
+        restoreConfig();
 
-        isOk = dlg->display();
+        dlg = new ConfigDialog();
 
-        mPlatformManager->destroyConfigDialog(dlg);
+        if ((isOk = dlg->display()))
+            saveConfig();
 
+        delete dlg;
         return isOk;
-
     }
 
     //-----------------------------------------------------------------------
@@ -399,6 +425,27 @@ namespace Ogre {
 
     }
 
+    //-----------------------------------------------------------------------
+    RenderSystem* Root::getRenderSystemByName(const String& name)
+    {
+        if (name.empty())
+        {
+            // No render system
+            return NULL;
+        }
+
+        RenderSystemList::const_iterator pRend;
+        for (pRend = getAvailableRenderers()->begin(); pRend != getAvailableRenderers()->end(); ++pRend)
+        {
+            RenderSystem* rs = *pRend;
+            if (rs->getName() == name)
+                return rs;
+        }
+
+        // Unrecognised render system
+        return NULL;
+    }
+
     //-----------------------------------------------------------------------
     void Root::setRenderSystem(RenderSystem* system)
     {
@@ -424,11 +471,10 @@ namespace Ogre {
         mRenderers.push_back(newRend);
     }
     //-----------------------------------------------------------------------
-    void Root::setSceneManager(SceneType sType, SceneManager *sm)
-    {
-        SceneManagerEnumerator::getSingleton().setSceneManager(sType, sm);
-    }
-
+	void Root::_setCurrentSceneManager(SceneManager* sm)
+	{
+		mCurrentSceneManager = sm;
+	}
     //-----------------------------------------------------------------------
     RenderSystem* Root::getRenderSystem(void)
     {
@@ -441,15 +487,16 @@ namespace Ogre {
     RenderWindow* Root::initialise(bool autoCreateWindow, const String& windowTitle)
     {
         if (!mActiveRenderer)
-            OGRE_EXCEPT(Exception::ERR_NO_RENDERSYSTEM_SELECTED,
+            OGRE_EXCEPT(Exception::ERR_INVALID_STATE,
             "Cannot initialise - no render "
             "system has been selected.", "Root::initialise");
 
-        mControllerManager = new ControllerManager();
+        if (!mControllerManager)
+			mControllerManager = new ControllerManager();
 
-        mAutoWindow =  mActiveRenderer->initialise(autoCreateWindow, windowTitle);
+		PlatformInformation::log(LogManager::getSingleton().getDefaultLog());
+		mAutoWindow =  mActiveRenderer->initialise(autoCreateWindow, windowTitle);
 
-		mResourceBackgroundQueue->initialise();
 
         if (autoCreateWindow && !mFirstTimePostWindowInit)
         {
@@ -459,6 +506,12 @@ namespace Ogre {
 
         // Initialise timer
         mTimer->reset();
+
+		// Init pools
+		ConvexBody::_initialisePool();
+
+		mIsInitialised = true;
+
         return mAutoWindow;
 
     }
@@ -473,12 +526,55 @@ namespace Ogre {
             return "";
 
     }
-    //-----------------------------------------------------------------------
-    SceneManager* Root::getSceneManager(SceneType sceneType)
-    {
-        // Delegate
-        return mSceneManagerEnum->getSceneManager(sceneType);
-    }
+	//-----------------------------------------------------------------------
+	void Root::addSceneManagerFactory(SceneManagerFactory* fact)
+	{
+		mSceneManagerEnum->addFactory(fact);
+	}
+	//-----------------------------------------------------------------------
+	void Root::removeSceneManagerFactory(SceneManagerFactory* fact)
+	{
+		mSceneManagerEnum->removeFactory(fact);
+	}
+	//-----------------------------------------------------------------------
+	const SceneManagerMetaData* Root::getSceneManagerMetaData(const String& typeName) const
+	{
+		return mSceneManagerEnum->getMetaData(typeName);
+	}
+	//-----------------------------------------------------------------------
+	SceneManagerEnumerator::MetaDataIterator 
+	Root::getSceneManagerMetaDataIterator(void) const
+	{
+		return mSceneManagerEnum->getMetaDataIterator();
+
+	}
+	//-----------------------------------------------------------------------
+	SceneManager* Root::createSceneManager(const String& typeName, 
+		const String& instanceName)
+	{
+		return mSceneManagerEnum->createSceneManager(typeName, instanceName);
+	}
+	//-----------------------------------------------------------------------
+	SceneManager* Root::createSceneManager(SceneTypeMask typeMask, 
+		const String& instanceName)
+	{
+		return mSceneManagerEnum->createSceneManager(typeMask, instanceName);
+	}
+	//-----------------------------------------------------------------------
+	void Root::destroySceneManager(SceneManager* sm)
+	{
+		mSceneManagerEnum->destroySceneManager(sm);
+	}
+	//-----------------------------------------------------------------------
+	SceneManager* Root::getSceneManager(const String& instanceName) const
+	{
+		return mSceneManagerEnum->getSceneManager(instanceName);
+	}
+	//-----------------------------------------------------------------------
+	SceneManagerEnumerator::SceneManagerIterator Root::getSceneManagerIterator(void)
+	{
+		return mSceneManagerEnum->getSceneManagerIterator();
+	}
     //-----------------------------------------------------------------------
     TextureManager* Root::getTextureManager(void)
     {
@@ -492,9 +588,14 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Root::addFrameListener(FrameListener* newListener)
     {
-        // Insert, unique only (set)
-        mFrameListeners.insert(newListener);
+		// Check if the specified listener is scheduled for removal
+		std::set<FrameListener *>::iterator i = mRemovedFrameListeners.find(newListener);
 
+		// If yes, cancel the removal. Otherwise add it to other listeners.
+		if (i != mRemovedFrameListeners.end())
+			mRemovedFrameListeners.erase(*i);
+		else
+			mFrameListeners.insert(newListener); // Insert, unique only (set)
     }
 
     //-----------------------------------------------------------------------
@@ -508,7 +609,7 @@ namespace Ogre {
     {
         // Increment frame number
         ++mCurrentFrame;
-        
+
         // Remove all marked listeners
         std::set<FrameListener*>::iterator i;
         for (i = mRemovedFrameListeners.begin();
@@ -539,7 +640,7 @@ namespace Ogre {
             mFrameListeners.erase(*i);
         }
         mRemovedFrameListeners.clear();
-        
+
         // Tell all listeners
 		bool ret = true;
         for (i= mFrameListeners.begin(); i != mFrameListeners.end(); ++i)
@@ -555,6 +656,9 @@ namespace Ogre {
         if (HardwareBufferManager::getSingletonPtr())
             HardwareBufferManager::getSingleton()._releaseBufferCopies();
 
+		// Also tell the ResourceBackgroundQueue to propagate background load events
+		ResourceBackgroundQueue::getSingleton()._fireBackgroundLoadingComplete();
+
         return ret;
     }
     //-----------------------------------------------------------------------
@@ -581,7 +685,7 @@ namespace Ogre {
     Real Root::calculateEventTime(unsigned long now, FrameEventTimeType type)
     {
         // Calculate the average time passed between events of the given type
-        // during the last 0.5 seconds.
+        // during the last mFrameSmoothingTime seconds.
 
         std::deque<unsigned long>& times = mEventTimes[type];
         times.push_back(now);
@@ -589,15 +693,16 @@ namespace Ogre {
         if(times.size() == 1)
             return 0;
 
-        // Times up to 0.5 seconds old should be kept
-        unsigned long discardLimit = now - 500;
+        // Times up to mFrameSmoothingTime seconds old should be kept
+        unsigned long discardThreshold =
+			static_cast<unsigned long>(mFrameSmoothingTime * 1000.0f);
 
         // Find the oldest time to keep
         std::deque<unsigned long>::iterator it = times.begin(),
             end = times.end()-2; // We need at least two times
         while(it != end)
         {
-            if(*it < discardLimit)
+            if (now - *it > discardThreshold)
                 ++it;
             else
                 break;
@@ -612,7 +717,7 @@ namespace Ogre {
     void Root::queueEndRendering(void)
     {
 	    mQueuedEnd = true;
-    }	
+    }
     //-----------------------------------------------------------------------
     void Root::startRendering(void)
     {
@@ -625,25 +730,17 @@ namespace Ogre {
             mEventTimes[i].clear();
 
         // Infinite loop, until broken out of by frame listeners
-		// or break out by calling queueEndRendering()
-		mQueuedEnd = false;
-		while( !mQueuedEnd )
-		{
-#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
-            // Pump events on Win32
-    		MSG  msg;
-			while( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
-			{
-				TranslateMessage( &msg );
-				DispatchMessage( &msg );
-			}
-#endif
+        // or break out by calling queueEndRendering()
+        mQueuedEnd = false;
 
-            if (!renderOneFrame())
+        while( !mQueuedEnd )
+        {
+			//Pump messages in all registered RenderWindow windows
+			WindowEventUtilities::messagePump();
+
+			if (!renderOneFrame())
                 break;
-		}
-
-
+        }
     }
     //-----------------------------------------------------------------------
     bool Root::renderOneFrame(void)
@@ -655,14 +752,23 @@ namespace Ogre {
 
         return _fireFrameEnded();
     }
+
     //-----------------------------------------------------------------------
     void Root::shutdown(void)
     {
-        SceneManagerEnumerator::getSingleton().shutdownAll();
+		SceneManagerEnumerator::getSingleton().shutdownAll();
+		shutdownPlugins();
+
         ShadowVolumeExtrudeProgram::shutdown();
 		mResourceBackgroundQueue->shutdown();
         ResourceGroupManager::getSingleton().shutdownAll();
 
+		// Destroy pools
+		ConvexBody::_destroyPool();
+
+
+		mIsInitialised = false;
+
 		LogManager::getSingleton().logMessage("*-*-* OGRE Shutdown");
     }
     //-----------------------------------------------------------------------
@@ -703,41 +809,48 @@ namespace Ogre {
     //-----------------------------------------------------------------------
 	void Root::shutdownPlugins(void)
 	{
-		std::vector<DynLib*>::reverse_iterator i;
-
 		// NB Shutdown plugins in reverse order to enforce dependencies
-		for (i = mPluginLibs.rbegin(); i != mPluginLibs.rend(); ++i)
+		for (PluginInstanceList::reverse_iterator i = mPlugins.rbegin(); i != mPlugins.rend(); ++i)
 		{
-			// Call plugin shutdown (optional)
-			DLL_STOP_PLUGIN pFunc = (DLL_STOP_PLUGIN)(*i)->getSymbol("dllShutdownPlugin");
-			if (pFunc)
-			{
-				pFunc();
-			}
-
+			(*i)->shutdown();
+		}
+	}
+	//-----------------------------------------------------------------------
+	void Root::initialisePlugins(void)
+	{
+		for (PluginInstanceList::reverse_iterator i = mPlugins.rbegin(); i != mPlugins.rend(); ++i)
+		{
+			(*i)->initialise();
 		}
 	}
 	//-----------------------------------------------------------------------
 	void Root::unloadPlugins(void)
     {
-        std::vector<DynLib*>::reverse_iterator i;
-
-        // NB Unload plugins in reverse order to enforce dependencies
-        for (i = mPluginLibs.rbegin(); i != mPluginLibs.rend(); ++i)
+		// unload dynamic libs first
+        for (PluginLibList::reverse_iterator i = mPluginLibs.rbegin(); i != mPluginLibs.rend(); ++i)
         {
             // Call plugin shutdown
             DLL_STOP_PLUGIN pFunc = (DLL_STOP_PLUGIN)(*i)->getSymbol("dllStopPlugin");
+			// this will call uninstallPlugin
             pFunc();
             // Unload library & destroy
             DynLibManager::getSingleton().unload(*i);
 
         }
+		mPluginLibs.clear();
 
-        mPluginLibs.clear();
+		// now deal with any remaining plugins that were registered through other means
+		for (PluginInstanceList::reverse_iterator i = mPlugins.rbegin(); i != mPlugins.rend(); ++i)
+		{
+			// Note this does NOT call uninstallPlugin - this shutdown is for the 
+			// detail objects
+			(*i)->uninstall();
+		}
+		mPlugins.clear();
 
     }
     //-----------------------------------------------------------------------
-    void Root::addResourceLocation(const String& name, const String& locType, 
+    void Root::addResourceLocation(const String& name, const String& locType,
 		const String& groupName, bool recursive)
     {
 		ResourceGroupManager::getSingleton().addResourceLocation(
@@ -761,12 +874,12 @@ namespace Ogre {
         return mAutoWindow;
     }
     //-----------------------------------------------------------------------
-	RenderWindow* Root::createRenderWindow(const String &name, unsigned int width, unsigned int height, 
+	RenderWindow* Root::createRenderWindow(const String &name, unsigned int width, unsigned int height,
 			bool fullScreen, const NameValuePairList *miscParams)
 	{
         if (!mActiveRenderer)
         {
-            OGRE_EXCEPT(Exception::ERR_NO_RENDERSYSTEM_SELECTED,
+            OGRE_EXCEPT(Exception::ERR_INVALID_STATE,
             "Cannot create window - no render "
             "system has been selected.", "Root::createRenderWindow");
         }
@@ -788,7 +901,7 @@ namespace Ogre {
     {
         if (!mActiveRenderer)
         {
-            OGRE_EXCEPT(Exception::ERR_NO_RENDERSYSTEM_SELECTED,
+            OGRE_EXCEPT(Exception::ERR_INVALID_STATE,
             "Cannot create window - no render "
             "system has been selected.", "Root::destroyRenderWindow");
         }
@@ -800,7 +913,7 @@ namespace Ogre {
     {
         if (!mActiveRenderer)
         {
-            OGRE_EXCEPT(Exception::ERR_NO_RENDERSYSTEM_SELECTED,
+            OGRE_EXCEPT(Exception::ERR_INVALID_STATE,
             "Cannot create window - no render "
             "system has been selected.", "Root::destroyRenderWindow");
         }
@@ -812,13 +925,45 @@ namespace Ogre {
     {
         if (!mActiveRenderer)
         {
-            OGRE_EXCEPT(Exception::ERR_NO_RENDERSYSTEM_SELECTED,
+            OGRE_EXCEPT(Exception::ERR_INVALID_STATE,
             "Cannot create window - no render "
             "system has been selected.", "Root::getRenderWindow");
         }
 
         return mActiveRenderer->getRenderTarget(name);
     }
+	//---------------------------------------------------------------------
+	void Root::installPlugin(Plugin* plugin)
+	{
+		LogManager::getSingleton().logMessage("Installing plugin: " + plugin->getName());
+
+		mPlugins.push_back(plugin);
+		plugin->install();
+
+		// if rendersystem is already initialised, call rendersystem init too
+		if (mIsInitialised)
+		{
+			plugin->initialise();
+		}
+
+		LogManager::getSingleton().logMessage("Plugin successfully installed");
+	}
+	//---------------------------------------------------------------------
+	void Root::uninstallPlugin(Plugin* plugin)
+	{
+		LogManager::getSingleton().logMessage("Uninstalling plugin: " + plugin->getName());
+		PluginInstanceList::iterator i = 
+			std::find(mPlugins.begin(), mPlugins.end(), plugin);
+		if (i != mPlugins.end())
+		{
+			if (mIsInitialised)
+				plugin->shutdown();
+			plugin->uninstall();
+			mPlugins.erase(i);
+		}
+		LogManager::getSingleton().logMessage("Plugin successfully uninstalled");
+
+	}
     //-----------------------------------------------------------------------
 	void Root::loadPlugin(const String& pluginName)
 	{
@@ -832,13 +977,16 @@ namespace Ogre {
 
 		if (!pFunc)
 			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Cannot find symbol dllStartPlugin in library " + pluginName,
-				"Root::loadPlugins");
+				"Root::loadPlugin");
+
+		// This must call installPlugin
 		pFunc();
+
 	}
     //-----------------------------------------------------------------------
 	void Root::unloadPlugin(const String& pluginName)
 	{
-        std::vector<DynLib*>::iterator i;
+        PluginLibList::iterator i;
 
         for (i = mPluginLibs.begin(); i != mPluginLibs.end(); ++i)
         {
@@ -846,6 +994,7 @@ namespace Ogre {
 			{
 				// Call plugin shutdown
 				DLL_STOP_PLUGIN pFunc = (DLL_STOP_PLUGIN)(*i)->getSymbol("dllStopPlugin");
+				// this must call uninstallPlugin
 				pFunc();
 				// Unload library (destroyed by DynLibManager)
 				DynLibManager::getSingleton().unload(*i);
@@ -865,12 +1014,16 @@ namespace Ogre {
     {
         if (!mFirstTimePostWindowInit)
         {
+			// Background loader
+			mResourceBackgroundQueue->initialise();
 			// Initialise material manager
 			mMaterialManager->initialise();
             // Init particle systems manager
             mParticleManager->_initialise();
 			// Init mesh manager
 			MeshManager::getSingleton()._initialise();
+			// Init plugins - after window creation so rsys resources available
+			initialisePlugins();
             mFirstTimePostWindowInit = true;
         }
 
@@ -888,4 +1041,149 @@ namespace Ogre {
 		for(int i=0; i<3; ++i)
 			mEventTimes[i].clear();
 	}
+	//---------------------------------------------------------------------
+	void Root::addMovableObjectFactory(MovableObjectFactory* fact,
+		bool overrideExisting)
+	{
+		MovableObjectFactoryMap::iterator facti = mMovableObjectFactoryMap.find(
+			fact->getType());
+		if (!overrideExisting && facti != mMovableObjectFactoryMap.end())
+		{
+			OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM,
+				"A factory of type '" + fact->getType() + "' already exists.",
+				"Root::addMovableObjectFactory");
+		}
+
+		if (fact->requestTypeFlags())
+		{
+			if (facti != mMovableObjectFactoryMap.end() && facti->second->requestTypeFlags())
+			{
+				// Copy type flags from the factory we're replacing
+				fact->_notifyTypeFlags(facti->second->getTypeFlags());
+			}
+			else
+			{
+				// Allocate new
+				fact->_notifyTypeFlags(_allocateNextMovableObjectTypeFlag());
+			}
+		}
+
+		// Save
+		mMovableObjectFactoryMap[fact->getType()] = fact;
+
+		LogManager::getSingleton().logMessage("MovableObjectFactory for type '" +
+			fact->getType() + "' registered.");
+
+	}
+	//---------------------------------------------------------------------
+	bool Root::hasMovableObjectFactory(const String& typeName) const
+	{
+		return !(mMovableObjectFactoryMap.find(typeName) == mMovableObjectFactoryMap.end());
+	}
+	//---------------------------------------------------------------------
+	MovableObjectFactory* Root::getMovableObjectFactory(const String& typeName)
+	{
+		MovableObjectFactoryMap::iterator i =
+			mMovableObjectFactoryMap.find(typeName);
+		if (i == mMovableObjectFactoryMap.end())
+		{
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,
+				"MovableObjectFactory of type " + typeName + " does not exist",
+				"Root::getMovableObjectFactory");
+		}
+		return i->second;
+	}
+	//---------------------------------------------------------------------
+	uint32 Root::_allocateNextMovableObjectTypeFlag(void)
+	{
+		if (mNextMovableObjectTypeFlag == SceneManager::USER_TYPE_MASK_LIMIT)
+		{
+			OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM,
+				"Cannot allocate a type flag since "
+				"all the available flags have been used.",
+				"Root::_allocateNextMovableObjectTypeFlag");
+
+		}
+		uint32 ret = mNextMovableObjectTypeFlag;
+		mNextMovableObjectTypeFlag <<= 1;
+		return ret;
+
+	}
+	//---------------------------------------------------------------------
+	void Root::removeMovableObjectFactory(MovableObjectFactory* fact)
+	{
+		MovableObjectFactoryMap::iterator i = mMovableObjectFactoryMap.find(
+			fact->getType());
+		if (i != mMovableObjectFactoryMap.end())
+		{
+			mMovableObjectFactoryMap.erase(i);
+		}
+
+	}
+	//---------------------------------------------------------------------
+	Root::MovableObjectFactoryIterator
+	Root::getMovableObjectFactoryIterator(void) const
+	{
+		return MovableObjectFactoryIterator(mMovableObjectFactoryMap.begin(),
+			mMovableObjectFactoryMap.end());
+
+	}
+	//---------------------------------------------------------------------
+	RenderQueueInvocationSequence* Root::createRenderQueueInvocationSequence(
+		const String& name)
+	{
+		RenderQueueInvocationSequenceMap::iterator i =
+			mRQSequenceMap.find(name);
+		if (i != mRQSequenceMap.end())
+		{
+			OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM,
+				"RenderQueueInvocationSequence with the name " + name +
+					" already exists.",
+				"Root::createRenderQueueInvocationSequence");
+		}
+		RenderQueueInvocationSequence* ret = new RenderQueueInvocationSequence(name);
+		mRQSequenceMap[name] = ret;
+		return ret;
+	}
+	//---------------------------------------------------------------------
+	RenderQueueInvocationSequence* Root::getRenderQueueInvocationSequence(
+		const String& name)
+	{
+		RenderQueueInvocationSequenceMap::iterator i =
+			mRQSequenceMap.find(name);
+		if (i == mRQSequenceMap.end())
+		{
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,
+				"RenderQueueInvocationSequence with the name " + name +
+				" not found.",
+				"Root::getRenderQueueInvocationSequence");
+		}
+		return i->second;
+	}
+	//---------------------------------------------------------------------
+	void Root::destroyRenderQueueInvocationSequence(
+		const String& name)
+	{
+		RenderQueueInvocationSequenceMap::iterator i =
+			mRQSequenceMap.find(name);
+		if (i != mRQSequenceMap.end())
+		{
+			delete i->second;
+			mRQSequenceMap.erase(i);
+		}
+	}
+	//---------------------------------------------------------------------
+	void Root::destroyAllRenderQueueInvocationSequences(void)
+	{
+		for (RenderQueueInvocationSequenceMap::iterator i = mRQSequenceMap.begin();
+			i != mRQSequenceMap.end(); ++i)
+		{
+			delete i->second;
+		}
+		mRQSequenceMap.clear();
+	}
+	//---------------------------------------------------------------------
+
+
+
 }
