@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
@@ -37,16 +41,72 @@ namespace Ogre {
     /// stream overhead = ID + size
     const size_t STREAM_OVERHEAD_SIZE = sizeof(uint16) + sizeof(uint32);
     const uint16 HEADER_STREAM_ID = 0x1000;
+    const uint16 OTHER_ENDIAN_HEADER_STREAM_ID = 0x0010;
     //---------------------------------------------------------------------
     Serializer::Serializer()
     {
         // Version number
         mVersion = "[Serializer_v1.00]";
+		mFlipEndian = false;
     }
     //---------------------------------------------------------------------
     Serializer::~Serializer()
     {
     }
+    //---------------------------------------------------------------------
+	void Serializer::determineEndianness(DataStreamPtr& stream)
+	{
+		if (stream->tell() != 0)
+		{
+			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+				"Can only determine the endianness of the input stream if it "
+				"is at the start", "Serializer::determineEndianness");
+		}
+				
+		uint16 dest;
+		// read header id manually (no conversion)
+        stream->read(&dest, sizeof(uint16));
+		// skip back
+		stream->skip(0 - sizeof(uint16));
+		if (dest == HEADER_STREAM_ID)
+		{
+			mFlipEndian = false;
+		}
+		else if (dest == OTHER_ENDIAN_HEADER_STREAM_ID)
+		{
+			mFlipEndian = true;
+		}
+		else
+		{
+			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+				"Can't find a header chunk to determine endianness",
+				"Serializer::determineEndianness");
+		}
+	}
+    //---------------------------------------------------------------------
+	void Serializer::determineEndianness(Endian requestedEndian)
+	{
+		switch(requestedEndian)
+		{
+		case ENDIAN_NATIVE:
+			mFlipEndian = false;
+			break;
+		case ENDIAN_BIG:
+#if OGRE_ENDIAN == OGRE_ENDIAN_BIG
+			mFlipEndian = false;
+#else
+			mFlipEndian = true;
+#endif
+			break;
+		case ENDIAN_LITTLE:
+#if OGRE_ENDIAN == OGRE_ENDIAN_BIG
+			mFlipEndian = true;
+#else
+			mFlipEndian = false;
+#endif
+			break;
+		}
+	}
     //---------------------------------------------------------------------
     void Serializer::writeFileHeader(void)
     {
@@ -58,15 +118,17 @@ namespace Ogre {
 
     }
     //---------------------------------------------------------------------
-    void Serializer::writeChunkHeader(uint16 id, uint32 size)
+    void Serializer::writeChunkHeader(uint16 id, size_t size)
     {
         writeShorts(&id, 1);
-        writeInts(&size, 1);
+		uint32 uint32size = static_cast<uint32>(size);
+        writeInts(&uint32size, 1);
     }
     //---------------------------------------------------------------------
     void Serializer::writeFloats(const float* const pFloat, size_t count)
     {
-#	if OGRE_ENDIAN == OGRE_ENDIAN_BIG
+		if (mFlipEndian)
+		{
             float * pFloatToWrite = (float *)malloc(sizeof(float) * count);
             memcpy(pFloatToWrite, pFloat, sizeof(float) * count);
             
@@ -74,9 +136,11 @@ namespace Ogre {
             writeData(pFloatToWrite, sizeof(float), count);
             
             free(pFloatToWrite);
-# 	else
+		}
+		else
+		{
             writeData(pFloat, sizeof(float), count);
-#	endif
+		}
     }
     //---------------------------------------------------------------------
     void Serializer::writeFloats(const double* const pDouble, size_t count)
@@ -87,18 +151,22 @@ namespace Ogre {
 		{
 			tmp[i] = static_cast<float>(pDouble[i]);
 		}
-#	if OGRE_ENDIAN == OGRE_ENDIAN_BIG
+		if(mFlipEndian)
+		{
             flipToLittleEndian(tmp, sizeof(float), count);
             writeData(tmp, sizeof(float), count);
-# 	else
+		}
+		else
+		{
             writeData(tmp, sizeof(float), count);
-#	endif
+		}
 		delete [] tmp;
     }
     //---------------------------------------------------------------------
     void Serializer::writeShorts(const uint16* const pShort, size_t count = 1)
     {
-#	if OGRE_ENDIAN == OGRE_ENDIAN_BIG
+		if(mFlipEndian)
+		{
             unsigned short * pShortToWrite = (unsigned short *)malloc(sizeof(unsigned short) * count);
             memcpy(pShortToWrite, pShort, sizeof(unsigned short) * count);
             
@@ -106,14 +174,17 @@ namespace Ogre {
             writeData(pShortToWrite, sizeof(unsigned short), count);
             
             free(pShortToWrite);
-# 	else
+		}
+		else
+		{
             writeData(pShort, sizeof(unsigned short), count);
-#	endif
+		}
     }
     //---------------------------------------------------------------------
     void Serializer::writeInts(const uint32* const pInt, size_t count = 1)
     {
-#	if OGRE_ENDIAN == OGRE_ENDIAN_BIG
+		if(mFlipEndian)
+		{
             unsigned int * pIntToWrite = (unsigned int *)malloc(sizeof(unsigned int) * count);
             memcpy(pIntToWrite, pInt, sizeof(unsigned int) * count);
             
@@ -121,9 +192,11 @@ namespace Ogre {
             writeData(pIntToWrite, sizeof(unsigned int), count);
             
             free(pIntToWrite);
-# 	else
+		}
+		else
+		{
             writeData(pInt, sizeof(unsigned int), count);
-#	endif
+		}
     }
     //---------------------------------------------------------------------
     //---------------------------------------------------------------------
@@ -261,49 +334,46 @@ namespace Ogre {
     //---------------------------------------------------------------------
     void Serializer::writeObject(const Vector3& vec)
     {
-        writeFloats(&vec.x, 1);
-        writeFloats(&vec.y, 1);
-        writeFloats(&vec.z, 1);
-
+        writeFloats(vec.ptr(), 3);
     }
     //---------------------------------------------------------------------
     void Serializer::writeObject(const Quaternion& q)
     {
-        writeFloats(&q.x, 1);
-        writeFloats(&q.y, 1);
-        writeFloats(&q.z, 1);
-        writeFloats(&q.w, 1);
+        float tmp[4] = { q.x, q.y, q.z, q.w };
+        writeFloats(tmp, 4);
     }
     //---------------------------------------------------------------------
     void Serializer::readObject(DataStreamPtr& stream, Vector3& pDest)
     {
-        readFloats(stream, &pDest.x, 1);
-        readFloats(stream, &pDest.y, 1);
-        readFloats(stream, &pDest.z, 1);
+        readFloats(stream, pDest.ptr(), 3);
     }
     //---------------------------------------------------------------------
     void Serializer::readObject(DataStreamPtr& stream, Quaternion& pDest)
     {
-        readFloats(stream, &pDest.x, 1);
-        readFloats(stream, &pDest.y, 1);
-        readFloats(stream, &pDest.z, 1);
-        readFloats(stream, &pDest.w, 1);
+        float tmp[4];
+        readFloats(stream, tmp, 4);
+        pDest.x = tmp[0];
+        pDest.y = tmp[1];
+        pDest.z = tmp[2];
+        pDest.w = tmp[3];
     }
     //---------------------------------------------------------------------
 
 
     void Serializer::flipToLittleEndian(void* pData, size_t size, size_t count)
     {
-#	if OGRE_ENDIAN == OGRE_ENDIAN_BIG
-        flipEndian(pData, size, count);
-#	endif
+		if(mFlipEndian)
+		{
+	        flipEndian(pData, size, count);
+		}
     }
     
     void Serializer::flipFromLittleEndian(void* pData, size_t size, size_t count)
     {
-#	if OGRE_ENDIAN == OGRE_ENDIAN_BIG
-        flipEndian(pData, size, count);
-#	endif
+		if(mFlipEndian)
+		{
+	        flipEndian(pData, size, count);
+		}
     }
     
     void Serializer::flipEndian(void * pData, size_t size, size_t count)
