@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
@@ -31,18 +35,19 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreRenderSystem.h"
 #include "OgreRenderSystemCapabilities.h"
 #include "OgreGpuProgramManager.h"
+#include "OgreMaterialManager.h"
 
 
 namespace Ogre {
     //-----------------------------------------------------------------------------
     Technique::Technique(Material* parent)
-        : mParent(parent), mIsSupported(false), mIlluminationPassesCompilationPhase(IPS_NOT_COMPILED), mLodIndex(0)
+        : mParent(parent), mIsSupported(false), mIlluminationPassesCompilationPhase(IPS_NOT_COMPILED), mLodIndex(0), mSchemeIndex(0)
     {
         // See above, defaults to unsupported until examined
     }
     //-----------------------------------------------------------------------------
     Technique::Technique(Material* parent, const Technique& oth)
-        : mParent(parent), mLodIndex(0)
+        : mParent(parent), mLodIndex(0), mSchemeIndex(0)
     {
         // Copy using operator=
         *this = oth;
@@ -59,81 +64,160 @@ namespace Ogre {
         return mIsSupported;
     }
     //-----------------------------------------------------------------------------
-    void Technique::_compile(bool autoManageTextureUnits)
+    String Technique::_compile(bool autoManageTextureUnits)
     {
+		StringUtil::StrStreamType compileErrors;
+
 		// assume not supported
 		mIsSupported = false;
         // Go through each pass, checking requirements
-        Passes::iterator i, iend;
-        iend = mPasses.end();
-        for (i = mPasses.begin(); i != iend; ++i)
+        Passes::iterator i;
+		unsigned short passNum = 0;
+		const RenderSystemCapabilities* caps =
+			Root::getSingleton().getRenderSystem()->getCapabilities();
+		unsigned short numTexUnits = caps->getNumTextureUnits();
+        for (i = mPasses.begin(); i != mPasses.end(); ++i, ++passNum)
         {
             Pass* currPass = *i;
+			// Adjust pass index
+			currPass->_notifyIndex(passNum);
             // Check texture unit requirements
             size_t numTexUnitsRequested = currPass->getNumTextureUnitStates();
-            const RenderSystemCapabilities* caps = 
-                Root::getSingleton().getRenderSystem()->getCapabilities();
-            unsigned short numTexUnits = caps->getNumTextureUnits();
+			// Don't trust getNumTextureUnits for programmable
+			if(!currPass->hasFragmentProgram())
+			{
+	#if defined(OGRE_PRETEND_TEXTURE_UNITS) && OGRE_PRETEND_TEXTURE_UNITS > 0
+				if (numTexUnits > OGRE_PRETEND_TEXTURE_UNITS)
+					numTexUnits = OGRE_PRETEND_TEXTURE_UNITS;
+	#endif
+				if (numTexUnitsRequested > numTexUnits)
+				{
+					if (!autoManageTextureUnits)
+					{
+						// The user disabled auto pass split
+						compileErrors << "Pass " << passNum << 
+							": Too many texture units for the current hardware and no splitting allowed."
+							<< std::endl;
+						return compileErrors.str();
+					}
+					else if (currPass->hasVertexProgram())
+					{
+						// Can't do this one, and can't split a programmable pass
+						compileErrors << "Pass " << passNum << 
+							": Too many texture units for the current hardware and "
+							"cannot split programmable passes."
+							<< std::endl;
+						return compileErrors.str();
+					}
+				}
+			}
 
+			if (currPass->hasVertexProgram())
+			{
+				// Check vertex program version
+				if (!currPass->getVertexProgram()->isSupported() )
+				{
+					// Can't do this one
+					compileErrors << "Pass " << passNum << 
+						": Vertex program " << currPass->getVertexProgram()->getName()
+						<< " cannot be used - ";
+					if (currPass->getVertexProgram()->hasCompileError())
+						compileErrors << "compile error.";
+					else
+						compileErrors << "not supported.";
+
+					compileErrors << std::endl;
+					return compileErrors.str();
+				}
+			}
             if (currPass->hasFragmentProgram())
             {
-                // Check texture units
-                if (numTexUnitsRequested > numTexUnits)
-                {
-                    // Can't do this one, and can't split a fragment pass
-                    return;
-                }
                 // Check fragment program version
                 if (!currPass->getFragmentProgram()->isSupported())
                 {
                     // Can't do this one
-                    return;
+					compileErrors << "Pass " << passNum << 
+						": Fragment program " << currPass->getFragmentProgram()->getName()
+						<< " cannot be used - ";
+					if (currPass->getFragmentProgram()->hasCompileError())
+						compileErrors << "compile error.";
+					else
+						compileErrors << "not supported.";
+
+					compileErrors << std::endl;
+					return compileErrors.str();
                 }
             }
             else
             {
 				// Check a few fixed-function options in texture layers
                 Pass::TextureUnitStateIterator texi = currPass->getTextureUnitStateIterator();
+				size_t texUnit = 0;
 				while (texi.hasMoreElements())
 				{
 					TextureUnitState* tex = texi.getNext();
-					// Any Cube textures? NB we make the assumption that any 
+					// Any Cube textures? NB we make the assumption that any
 					// card capable of running fragment programs can support
 					// cubic textures, which has to be true, surely?
 					if (tex->is3D() && !caps->hasCapability(RSC_CUBEMAPPING))
 					{
 						// Fail
-						return;
+						compileErrors << "Pass " << passNum << 
+							" Tex " << texUnit <<
+							": Cube maps not supported by current environment."
+							<< std::endl;
+						return compileErrors.str();
+					}
+					// Any 3D textures? NB we make the assumption that any
+					// card capable of running fragment programs can support
+					// 3D textures, which has to be true, surely?
+					if (tex->getTextureType() == TEX_TYPE_3D && !caps->hasCapability(RSC_TEXTURE_3D))
+					{
+						// Fail
+						compileErrors << "Pass " << passNum << 
+							" Tex " << texUnit <<
+							": Volume textures not supported by current environment."
+							<< std::endl;
+						return compileErrors.str();
 					}
 					// Any Dot3 blending?
 					if (tex->getColourBlendMode().operation == LBX_DOTPRODUCT &&
 							!caps->hasCapability(RSC_DOT3))
 					{
 						// Fail
-						return;
+						compileErrors << "Pass " << passNum << 
+							" Tex " << texUnit <<
+							": DOT3 blending not supported by current environment."
+							<< std::endl;
+						return compileErrors.str();
+					}
+					++texUnit;
+				}
+
+				// We're ok on operations, now we need to check # texture units
+				if (!currPass->hasFragmentProgram())
+				{
+					// Keep splitting this pass so long as units requested > gpu units
+					while (numTexUnitsRequested > numTexUnits)
+					{
+						// chop this pass into many passes
+						currPass = currPass->_split(numTexUnits);
+						numTexUnitsRequested = currPass->getNumTextureUnitStates();
+						// Advance pass number
+						++passNum;
+						// Reset iterator
+						i = mPasses.begin() + passNum;
+						// Move the new pass to the right place (will have been created
+						// at the end, may be other passes in between)
+						assert(mPasses.back() == currPass);
+						std::copy_backward(i, (mPasses.end()-1), mPasses.end());
+						*i = currPass;
+						// Adjust pass index
+						currPass->_notifyIndex(passNum);
 					}
 				}
-				
-				// We're ok on operations, now we need to check # texture units
-				// Keep splitting this pass so long as units requested > gpu units
-                while (numTexUnitsRequested > numTexUnits)
-                {
-                    // chop this pass into many passes
-                    currPass = currPass->_split(numTexUnits);
-                    numTexUnitsRequested = currPass->getNumTextureUnitStates();
-                }
             }
 
-            if (currPass->hasVertexProgram())
-            {
-                // Check vertex program version
-                if (!currPass->getVertexProgram()->isSupported() )
-                {
-                    // Can't do this one
-                    return;
-                }
-            }
-		
 		}
         // If we got this far, we're ok
         mIsSupported = true;
@@ -142,6 +226,8 @@ namespace Ogre {
         clearIlluminationPasses();
         mIlluminationPassesCompilationPhase = IPS_NOT_COMPILED;
 
+		return StringUtil::BLANK;
+
     }
     //-----------------------------------------------------------------------------
     Pass* Technique::createPass(void)
@@ -157,6 +243,26 @@ namespace Ogre {
 		return mPasses[index];
     }
     //-----------------------------------------------------------------------------
+    Pass* Technique::getPass(const String& name)
+    {
+        Passes::iterator i    = mPasses.begin();
+        Passes::iterator iend = mPasses.end();
+        Pass* foundPass = 0;
+
+        // iterate through techniques to find a match
+        while (i != iend)
+        {
+            if ( (*i)->getName() == name )
+            {
+                foundPass = (*i);
+                break;
+            }
+            ++i;
+        }
+
+        return foundPass;
+    }
+    //-----------------------------------------------------------------------------
     unsigned short Technique::getNumPasses(void) const
     {
 		return static_cast<unsigned short>(mPasses.size());
@@ -167,7 +273,12 @@ namespace Ogre {
 		assert(index < mPasses.size() && "Index out of bounds");
 		Passes::iterator i = mPasses.begin() + index;
 		(*i)->queueForDeletion();
-		mPasses.erase(i);
+		i = mPasses.erase(i);
+		// Adjust passes index
+		for (; i != mPasses.end(); ++i, ++index)
+		{
+			(*i)->_notifyIndex(index);
+		}
     }
     //-----------------------------------------------------------------------------
     void Technique::removeAllPasses(void)
@@ -180,6 +291,52 @@ namespace Ogre {
         }
         mPasses.clear();
     }
+
+    //-----------------------------------------------------------------------------
+    bool Technique::movePass(const unsigned short sourceIndex, const unsigned short destinationIndex)
+    {
+        bool moveSuccessful = false;
+
+        // don't move the pass if source == destination
+        if (sourceIndex == destinationIndex) return true;
+
+        if( (sourceIndex < mPasses.size()) && (destinationIndex < mPasses.size()))
+        {
+            Passes::iterator i = mPasses.begin() + sourceIndex;
+            //Passes::iterator DestinationIterator = mPasses.begin() + destinationIndex;
+
+            Pass* pass = (*i);
+            mPasses.erase(i);
+
+            i = mPasses.begin() + destinationIndex;
+
+            // compensate for source erase if destination is greater than source
+            if (destinationIndex > sourceIndex) --i;
+
+            mPasses.insert(i, pass);
+
+			// Adjust passes index
+			unsigned short beginIndex, endIndex;
+			if (destinationIndex > sourceIndex)
+			{
+				beginIndex = sourceIndex;
+				endIndex = destinationIndex;
+			}
+			else
+			{
+				beginIndex = destinationIndex;
+				endIndex = sourceIndex;
+			}
+			for (unsigned short index = beginIndex; index <= endIndex; ++index)
+			{
+				mPasses[index]->_notifyIndex(index);
+			}
+            moveSuccessful = true;
+        }
+
+        return moveSuccessful;
+    }
+
     //-----------------------------------------------------------------------------
     const Technique::PassIterator Technique::getPassIterator(void)
     {
@@ -188,8 +345,10 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     Technique& Technique::operator=(const Technique& rhs)
     {
+        mName = rhs.mName;
 		this->mIsSupported = rhs.mIsSupported;
         this->mLodIndex = rhs.mLodIndex;
+		this->mSchemeIndex = rhs.mSchemeIndex;
 		// copy passes
 		removeAllPasses();
 		Passes::const_iterator i, iend;
@@ -244,6 +403,19 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------------
+    bool Technique::hasColourWriteDisabled(void) const
+    {
+        if (mPasses.empty())
+        {
+            return true;
+        }
+        else
+        {
+            // Base decision on the colour write settings of the first pass
+            return !mPasses[0]->getColourWriteEnabled();
+        }
+    }
+    //-----------------------------------------------------------------------------
     void Technique::_load(void)
     {
 		assert (mIsSupported && "This technique is not supported");
@@ -281,6 +453,17 @@ namespace Ogre {
         return mParent->isLoaded() && mIsSupported;
     }
     //-----------------------------------------------------------------------
+    void Technique::setPointSize(Real ps)
+    {
+        Passes::iterator i, iend;
+        iend = mPasses.end();
+        for (i = mPasses.begin(); i != iend; ++i)
+        {
+            (*i)->setPointSize(ps);
+        }
+
+    }
+    //-----------------------------------------------------------------------
     void Technique::setAmbient(Real red, Real green, Real blue)
     {
         Passes::iterator i, iend;
@@ -443,13 +626,13 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void Technique::setDepthBias(ushort bias)
+    void Technique::setDepthBias(float constantBias, float slopeScaleBias)
     {
         Passes::iterator i, iend;
         iend = mPasses.end();
         for (i = mPasses.begin(); i != iend; ++i)
         {
-            (*i)->setDepthBias(bias);
+            (*i)->setDepthBias(constantBias, slopeScaleBias);
         }
     }
     //-----------------------------------------------------------------------
@@ -483,7 +666,7 @@ namespace Ogre {
         }
     }
     // --------------------------------------------------------------------
-    void Technique::setSceneBlending( const SceneBlendFactor sourceFactor, 
+    void Technique::setSceneBlending( const SceneBlendFactor sourceFactor,
         const SceneBlendFactor destFactor)
     {
         Passes::iterator i, iend;
@@ -494,6 +677,13 @@ namespace Ogre {
         }
     }
 
+    // --------------------------------------------------------------------
+    void Technique::setName(const String& name)
+    {
+        mName = name;
+    }
+
+
     //-----------------------------------------------------------------------
     void Technique::_notifyNeedsRecompile(void)
     {
@@ -509,24 +699,32 @@ namespace Ogre {
         mLodIndex = index;
         _notifyNeedsRecompile();
     }
+    //-----------------------------------------------------------------------
+	void Technique::setSchemeName(const String& schemeName)
+	{
+		mSchemeIndex = MaterialManager::getSingleton()._getSchemeIndex(schemeName);
+        _notifyNeedsRecompile();
+	}
+    //-----------------------------------------------------------------------
+	const String& Technique::getSchemeName(void) const
+	{
+		return MaterialManager::getSingleton()._getSchemeName(mSchemeIndex);
+	}
+    //-----------------------------------------------------------------------
+	unsigned short Technique::_getSchemeIndex(void) const
+	{
+		return mSchemeIndex;
+	}
     //-----------------------------------------------------------------------
     void Technique::_compileIlluminationPasses(void)
     {
         clearIlluminationPasses();
 
-        if (isTransparent())
-        {
-            // Don't need to split transparents since they are rendered separately
-            return;
-        }
-
         Passes::iterator i, iend;
         iend = mPasses.end();
         i = mPasses.begin();
-        
-        IlluminationStage iStage = IS_AMBIENT;
 
-        // Disable requirement to recompile during compile
+        IlluminationStage iStage = IS_AMBIENT;
 
         bool haveAmbient = false;
         while (i != iend)
@@ -553,27 +751,40 @@ namespace Ogre {
                 {
                     // Split off any ambient part
                     if (p->getAmbient() != ColourValue::Black ||
-                        p->getSelfIllumination() != ColourValue::Black)
+                        p->getSelfIllumination() != ColourValue::Black ||
+                        p->getAlphaRejectFunction() != CMPF_ALWAYS_PASS)
                     {
                         // Copy existing pass
                         Pass* newPass = new Pass(this, p->getIndex(), *p);
-                        // Remove any texture units
-                        newPass->removeAllTextureUnitStates();
+                        if (newPass->getAlphaRejectFunction() != CMPF_ALWAYS_PASS)
+                        {
+                            // Alpha rejection passes must retain their transparency, so
+                            // we allow the texture units, but override the colour functions
+                            Pass::TextureUnitStateIterator tusi = newPass->getTextureUnitStateIterator();
+                            while (tusi.hasMoreElements())
+                            {
+                                TextureUnitState* tus = tusi.getNext();
+                                tus->setColourOperationEx(LBX_SOURCE1, LBS_CURRENT);
+                            }
+                        }
+                        else
+                        {
+                            // Remove any texture units
+                            newPass->removeAllTextureUnitStates();
+                        }
                         // Remove any fragment program
                         if (newPass->hasFragmentProgram())
                             newPass->setFragmentProgram("");
                         // We have to leave vertex program alone (if any) and
-                        // just trust that the author is using light bindings, which 
+                        // just trust that the author is using light bindings, which
                         // we will ensure there are none in the ambient pass
-                        newPass->setDiffuse(ColourValue::Black);
+                        newPass->setDiffuse(0, 0, 0, newPass->getDiffuse().a);  // Preserving alpha
                         newPass->setSpecular(ColourValue::Black);
 
-                        // If ambient & emissive are zero, then no colour write
-                        if (newPass->getAmbient() == ColourValue::Black && 
-                            newPass->getSelfIllumination() == ColourValue::Black)
-                        {
-                            newPass->setColourWriteEnabled(false);
-                        }
+                        // Calculate hash value for new pass, because we are compiling
+                        // illumination passes on demand, which will loss hash calculate
+                        // before it add to render queue first time.
+                        newPass->_recalculateHash();
 
                         iPass = new IlluminationPass();
                         iPass->destroyOnShutdown = true;
@@ -583,15 +794,21 @@ namespace Ogre {
 
                         mIlluminationPasses.push_back(iPass);
                         haveAmbient = true;
-                        
+
                     }
-                    
+
                     if (!haveAmbient)
                     {
                         // Make up a new basic pass
                         Pass* newPass = new Pass(this, p->getIndex());
                         newPass->setAmbient(ColourValue::Black);
                         newPass->setDiffuse(ColourValue::Black);
+
+                        // Calculate hash value for new pass, because we are compiling
+                        // illumination passes on demand, which will loss hash calculate
+                        // before it add to render queue first time.
+                        newPass->_recalculateHash();
+
                         iPass = new IlluminationPass();
                         iPass->destroyOnShutdown = true;
                         iPass->originalPass = p;
@@ -605,7 +822,7 @@ namespace Ogre {
                 }
                 break;
             case IS_PER_LIGHT:
-                if (p->getRunOncePerLight())
+                if (p->getIteratePerLight())
                 {
                     // If this is per-light already, use it directly
                     iPass = new IlluminationPass();
@@ -619,14 +836,28 @@ namespace Ogre {
                 else
                 {
                     // Split off per-light details (can only be done for one)
-                    if (p->getLightingEnabled() && 
+                    if (p->getLightingEnabled() &&
                         (p->getDiffuse() != ColourValue::Black ||
                         p->getSpecular() != ColourValue::Black))
                     {
                         // Copy existing pass
                         Pass* newPass = new Pass(this, p->getIndex(), *p);
-                        // remove texture units
-                        newPass->removeAllTextureUnitStates();
+                        if (newPass->getAlphaRejectFunction() != CMPF_ALWAYS_PASS)
+                        {
+                            // Alpha rejection passes must retain their transparency, so
+                            // we allow the texture units, but override the colour functions
+                            Pass::TextureUnitStateIterator tusi = newPass->getTextureUnitStateIterator();
+                            while (tusi.hasMoreElements())
+                            {
+                                TextureUnitState* tus = tusi.getNext();
+                                tus->setColourOperationEx(LBX_SOURCE1, LBS_CURRENT);
+                            }
+                        }
+                        else
+                        {
+                            // remove texture units
+                            newPass->removeAllTextureUnitStates();
+                        }
                         // remove fragment programs
                         if (newPass->hasFragmentProgram())
                             newPass->setFragmentProgram("");
@@ -637,6 +868,11 @@ namespace Ogre {
                         // must be additive
                         newPass->setSceneBlending(SBF_ONE, SBF_ONE);
 
+                        // Calculate hash value for new pass, because we are compiling
+                        // illumination passes on demand, which will loss hash calculate
+                        // before it add to render queue first time.
+                        newPass->_recalculateHash();
+
                         iPass = new IlluminationPass();
                         iPass->destroyOnShutdown = true;
                         iPass->originalPass = p;
@@ -669,13 +905,18 @@ namespace Ogre {
                         // Copy the pass and tweak away the lighting parts
                         Pass* newPass = new Pass(this, p->getIndex(), *p);
                         newPass->setAmbient(ColourValue::Black);
-                        newPass->setDiffuse(ColourValue::Black);
+                        newPass->setDiffuse(0, 0, 0, newPass->getDiffuse().a);  // Preserving alpha
                         newPass->setSpecular(ColourValue::Black);
                         newPass->setSelfIllumination(ColourValue::Black);
                         newPass->setLightingEnabled(false);
                         // modulate
                         newPass->setSceneBlending(SBF_DEST_COLOUR, SBF_ZERO);
 
+                        // Calculate hash value for new pass, because we are compiling
+                        // illumination passes on demand, which will loss hash calculate
+                        // before it add to render queue first time.
+                        newPass->_recalculateHash();
+
                         // NB there is nothing we can do about vertex & fragment
                         // programs here, so people will just have to make their
                         // programs friendly-like if they want to use this technique
@@ -711,20 +952,21 @@ namespace Ogre {
         mIlluminationPasses.clear();
     }
     //-----------------------------------------------------------------------
-    const Technique::IlluminationPassIterator 
+    const Technique::IlluminationPassIterator
     Technique::getIlluminationPassIterator(void)
     {
-        if (mIlluminationPassesCompilationPhase == IPS_NOT_COMPILED)
+        IlluminationPassesState targetState = IPS_COMPILED;
+        if (mIlluminationPassesCompilationPhase != targetState)
         {
             // prevents parent->_notifyNeedsRecompile() call during compile
             mIlluminationPassesCompilationPhase = IPS_COMPILE_DISABLED;
             // Splitting the passes into illumination passes
             _compileIlluminationPasses();
             // Mark that illumination passes compilation finished
-            mIlluminationPassesCompilationPhase = IPS_COMPILED;
+            mIlluminationPassesCompilationPhase = targetState;
         }
 
-        return IlluminationPassIterator(mIlluminationPasses.begin(), 
+        return IlluminationPassIterator(mIlluminationPasses.begin(),
             mIlluminationPasses.end());
     }
     //-----------------------------------------------------------------------
@@ -733,5 +975,21 @@ namespace Ogre {
 		return mParent->getGroup();
 	}
 
+    //-----------------------------------------------------------------------
+    bool Technique::applyTextureAliases(const AliasTextureNamePairList& aliasList, const bool apply) const
+    {
+        // iterate through passes and apply texture alias
+        Passes::const_iterator i, iend;
+        iend = mPasses.end();
+        bool testResult = false;
+
+        for(i = mPasses.begin(); i != iend; ++i)
+        {
+            if ((*i)->applyTextureAliases(aliasList, apply))
+                testResult = true;
+        }
+
+        return testResult;
+    }
 
 }
