@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
@@ -29,41 +33,47 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreRenderTarget.h"
 #include "OgreCamera.h"
 #include "OgreMath.h"
+#include "OgreRoot.h"
+#include "OgreMaterialManager.h"
+
 
 namespace Ogre {
     //---------------------------------------------------------------------
     Viewport::Viewport(Camera* cam, RenderTarget* target, Real left, Real top, Real width, Real height, int ZOrder)
+        : mCamera(cam)
+        , mTarget(target)
+        , mRelLeft(left)
+        , mRelTop(top)
+        , mRelWidth(width)
+        , mRelHeight(height)
+        // Actual dimensions will update later
+        , mZOrder(ZOrder)
+        , mBackColour(ColourValue::Black)
+        , mClearEveryFrame(true)
+		, mClearBuffers(FBT_COLOUR | FBT_DEPTH)
+        , mUpdated(false)
+        , mShowOverlays(true)
+        , mShowSkies(true)
+		, mShowShadows(true)
+		, mVisibilityMask(0xFFFFFFFF)
+		, mRQSequence(0)
+		, mMaterialSchemeName(MaterialManager::DEFAULT_SCHEME_NAME)
     {
 
 		StringUtil::StrStreamType msg;
 
         msg << "Creating viewport on target '" << target->getName() << "'"
-			<< ", rendering from camera '" << cam->getName() << "'"
+			<< ", rendering from camera '" << (cam != 0 ? cam->getName() : "NULL") << "'"
 			<< ", relative dimensions "	<< std::fixed << std::setprecision(2) 
 			<< "L: " << left << " T: " << top << " W: " << width << " H: " << height
 			<< " ZOrder: " << ZOrder;
         LogManager::getSingleton().logMessage(msg.str());
-        mCamera = cam;
-        mTarget = target;
-
-        mRelLeft = left;
-        mRelTop = top;
-        mRelWidth = width;
-        mRelHeight = height;
-        mZOrder = ZOrder;
-
-        mBackColour = ColourValue::Black;
-        mClearEveryFrame = true;
-
 
         // Calculate actual dimensions
         _updateDimensions();
 
-        mUpdated = true;
-        mShowOverlays = true;
-
         // notify camera
-        cam->_notifyViewport(this);
+        if(cam) cam->_notifyViewport(this);
     }
     //---------------------------------------------------------------------
     Viewport::~Viewport()
@@ -97,14 +107,14 @@ namespace Ogre {
         // This allows cameras to be used to render to many viewports,
         // which can have their own dimensions and aspect ratios.
 
-        if (mCamera->getAutoAspectRatio()) 
+        if (mCamera && mCamera->getAutoAspectRatio()) 
         {
             mCamera->setAspectRatio((Real) mActWidth / (Real) mActHeight);
         }
 
 		StringUtil::StrStreamType msg;
 
-        msg << "Viewport for camera '" << mCamera->getName() << "'"
+		msg << "Viewport for camera '" << (mCamera != 0 ? mCamera->getName() : "NULL") << "'"
 			<< ", actual dimensions "	<< std::fixed << std::setprecision(2) 
 			<< "L: " << mActLeft << " T: " << mActTop << " W: " << mActWidth << " H: " << mActHeight;
 
@@ -196,9 +206,10 @@ namespace Ogre {
         return mBackColour;
     }
     //---------------------------------------------------------------------
-    void Viewport::setClearEveryFrame(bool clear)
+    void Viewport::setClearEveryFrame(bool clear, unsigned int buffers)
     {
         mClearEveryFrame = clear;
+		mClearBuffers = buffers;
     }
     //---------------------------------------------------------------------
     bool Viewport::getClearEveryFrame(void) const
@@ -206,6 +217,11 @@ namespace Ogre {
         return mClearEveryFrame;
     }
     //---------------------------------------------------------------------
+    unsigned int Viewport::getClearBuffers(void) const
+    {
+        return mClearBuffers;
+    }
+    //---------------------------------------------------------------------
     void Viewport::getActualDimensions(int &left, int&top, int &width, int &height) const
     {
         left = mActLeft;
@@ -217,13 +233,18 @@ namespace Ogre {
     //---------------------------------------------------------------------
     unsigned int Viewport::_getNumRenderedFaces(void) const
     {
-        return mCamera->_getNumRenderedFaces();
+		return mCamera ? mCamera->_getNumRenderedFaces() : 0;
+    }
+    //---------------------------------------------------------------------
+    unsigned int Viewport::_getNumRenderedBatches(void) const
+    {
+		return mCamera ? mCamera->_getNumRenderedBatches() : 0;
     }
     //---------------------------------------------------------------------
     void Viewport::setCamera(Camera* cam)
     {
         mCamera = cam;
-
+		if(cam) mCamera->_notifyViewport(this);
     }
     //---------------------------------------------------------------------
     void Viewport::setOverlaysEnabled(bool enabled)
@@ -235,4 +256,50 @@ namespace Ogre {
     {
         return mShowOverlays;
     }
+    //---------------------------------------------------------------------
+    void Viewport::setSkiesEnabled(bool enabled)
+    {
+        mShowSkies = enabled;
+    }
+    //---------------------------------------------------------------------
+    bool Viewport::getSkiesEnabled(void) const
+    {
+        return mShowSkies;
+    }
+    //---------------------------------------------------------------------
+    void Viewport::setShadowsEnabled(bool enabled)
+    {
+        mShowShadows = enabled;
+    }
+    //---------------------------------------------------------------------
+    bool Viewport::getShadowsEnabled(void) const
+    {
+        return mShowShadows;
+    }
+	//-----------------------------------------------------------------------
+	void Viewport::setRenderQueueInvocationSequenceName(const String& sequenceName)
+	{
+		mRQSequenceName = sequenceName;
+		if (mRQSequenceName.empty())
+		{
+			mRQSequence = 0;
+		}
+		else
+		{
+			mRQSequence =
+				Root::getSingleton().getRenderQueueInvocationSequence(mRQSequenceName);
+		}
+	}
+	//-----------------------------------------------------------------------
+	const String& Viewport::getRenderQueueInvocationSequenceName(void) const
+	{
+		return mRQSequenceName;
+	}
+	//-----------------------------------------------------------------------
+	RenderQueueInvocationSequence* Viewport::_getRenderQueueInvocationSequence(void)
+	{
+		return mRQSequence;
+	}
+	//-----------------------------------------------------------------------
+
 }
