@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 /***************************************************************************
@@ -52,26 +56,21 @@ namespace Ogre
     #define DELTA_BINDING 1
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
-    TerrainBufferCache TerrainRenderable::msIndexCache;
     String TerrainRenderable::mType = "TerrainMipMap";
-    LevelArray TerrainRenderable::mLevelIndex;
-    bool TerrainRenderable::mLevelInit = false;
-    const TerrainOptions* TerrainRenderable::msOptions = 0;
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
 
     //-----------------------------------------------------------------------
-    TerrainRenderable::TerrainRenderable(const String& name)
-        : Renderable(), mTerrain(0), mName(name),  mDeltaBuffers(0), mPositionBuffer(0)
+    TerrainRenderable::TerrainRenderable(const String& name, TerrainSceneManager* tsm)
+        : Renderable(), MovableObject(name), mSceneManager(tsm), mTerrain(0), mDeltaBuffers(0), mPositionBuffer(0)
     {
         mForcedRenderLevel = -1;
         mLastNextLevel = -1;
-        // Default query flags to top bit so users can exclude it if they wish
-        mQueryFlags = SceneManager::WORLD_GEOMETRY_QUERY_MASK;
 
         mMinLevelDistSqr = 0;
 
         mInit = false;
+        mLightListDirty = true;
 		MovableObject::mCastShadows = false;
 
         for ( int i = 0; i < 4; i++ )
@@ -79,7 +78,8 @@ namespace Ogre
             mNeighbors[ i ] = 0;
         }
 
-        _initLevelIndexes();
+		mOptions = &(mSceneManager->getOptions());
+
 
     }
     //-----------------------------------------------------------------------
@@ -87,8 +87,13 @@ namespace Ogre
     {
 
         deleteGeometry();
-        _destroyLevelIndexes();
     }
+	//-----------------------------------------------------------------------
+	uint32 TerrainRenderable::getTypeFlags(void) const
+	{
+		// return world flag
+		return SceneManager::WORLD_GEOMETRY_TYPE_MASK;
+	}
     //-----------------------------------------------------------------------
     void TerrainRenderable::deleteGeometry()
     {
@@ -108,14 +113,12 @@ namespace Ogre
     void TerrainRenderable::initialise(int startx, int startz,  
         Real* pageHeightData)
     {
-        if (!msOptions)
-            msOptions = &(TerrainSceneManager::getOptions());
 
-        if ( msOptions->maxGeoMipMapLevel != 0 )
+        if ( mOptions->maxGeoMipMapLevel != 0 )
         {
-            int i = ( int ) 1 << ( msOptions->maxGeoMipMapLevel - 1 ) ;
+            int i = ( int ) 1 << ( mOptions->maxGeoMipMapLevel - 1 ) ;
 
-            if ( ( i + 1 ) > msOptions->tileSize )
+            if ( ( i + 1 ) > mOptions->tileSize )
             {
                 printf( "Invalid maximum mipmap specifed, must be n, such that 2^(n-1)+1 < tileSize \n" );
                 return ;
@@ -129,7 +132,7 @@ namespace Ogre
 
         mTerrain = new VertexData;
         mTerrain->vertexStart = 0;
-        mTerrain->vertexCount = msOptions->tileSize * msOptions->tileSize;
+        mTerrain->vertexCount = mOptions->tileSize * mOptions->tileSize;
 
         VertexDeclaration* decl = mTerrain->vertexDeclaration;
         VertexBufferBinding* bind = mTerrain->vertexBufferBinding;
@@ -138,7 +141,7 @@ namespace Ogre
         size_t offset = 0;
         decl->addElement(MAIN_BINDING, offset, VET_FLOAT3, VES_POSITION);
         offset += VertexElement::getTypeSize(VET_FLOAT3);
-        if (TerrainSceneManager::getOptions().lit)
+        if (mOptions->lit)
         {
             decl->addElement(MAIN_BINDING, offset, VET_FLOAT3, VES_NORMAL);
             offset += VertexElement::getTypeSize(VET_FLOAT3);
@@ -148,7 +151,7 @@ namespace Ogre
         offset += VertexElement::getTypeSize(VET_FLOAT2);
         decl->addElement(MAIN_BINDING, offset, VET_FLOAT2, VES_TEXTURE_COORDINATES, 1);
         offset += VertexElement::getTypeSize(VET_FLOAT2);
-        if (TerrainSceneManager::getOptions().coloured)
+        if (mOptions->coloured)
         {
             decl->addElement(MAIN_BINDING, offset, VET_COLOUR, VES_DIFFUSE);
             offset += VertexElement::getTypeSize(VET_COLOUR);
@@ -165,7 +168,7 @@ namespace Ogre
 
         bind->setBinding(MAIN_BINDING, mMainBuffer);
 
-        if (msOptions->lodMorph)
+        if (mOptions->lodMorph)
         {
             // Create additional element for delta
             decl->addElement(DELTA_BINDING, 0, VET_FLOAT1, VES_BLEND_WEIGHTS);
@@ -177,11 +180,11 @@ namespace Ogre
 
         mRenderLevel = 1;
 
-        mMinLevelDistSqr = new Real[ msOptions->maxGeoMipMapLevel ];
+        mMinLevelDistSqr = new Real[ mOptions->maxGeoMipMapLevel ];
 
-        int endx = startx + msOptions->tileSize;
+        int endx = startx + mOptions->tileSize;
 
-        int endz = startz + msOptions->tileSize;
+        int endz = startz + mOptions->tileSize;
 
         Vector3 left, down, here;
 
@@ -201,18 +204,18 @@ namespace Ogre
                 texelem0->baseVertexPointerToElement(pBase, &pTex0);
                 texelem1->baseVertexPointerToElement(pBase, &pTex1);
     
-                Real height = pageHeightData[j * msOptions->pageSize + i];
-                height = height * msOptions->scale.y; // scale height 
+                Real height = pageHeightData[j * mOptions->pageSize + i];
+                height = height * mOptions->scale.y; // scale height 
 
-                *pSysPos++ = *pPos++ = ( float ) i * msOptions->scale.x; //x
+                *pSysPos++ = *pPos++ = ( float ) i * mOptions->scale.x; //x
                 *pSysPos++ = *pPos++ = height; // y
-                *pSysPos++ = *pPos++ = ( float ) j * msOptions->scale.z; //z
+                *pSysPos++ = *pPos++ = ( float ) j * mOptions->scale.z; //z
 
-                *pTex0++ = ( float ) i / ( float ) ( msOptions->pageSize - 1 );
-                *pTex0++ = ( float ) j / ( float ) ( msOptions->pageSize - 1 );
+                *pTex0++ = ( float ) i / ( float ) ( mOptions->pageSize - 1 );
+                *pTex0++ = ( float ) j / ( float ) ( mOptions->pageSize - 1 );
 
-                *pTex1++ = ( ( float ) i / ( float ) ( msOptions->tileSize - 1 ) ) * msOptions->detailTile;
-                *pTex1++ = ( ( float ) j / ( float ) ( msOptions->tileSize - 1 ) ) * msOptions->detailTile;
+                *pTex1++ = ( ( float ) i / ( float ) ( mOptions->tileSize - 1 ) ) * mOptions->detailTile;
+                *pTex1++ = ( ( float ) j / ( float ) ( mOptions->tileSize - 1 ) ) * mOptions->detailTile;
 
                 if ( height < min )
                     min = ( Real ) height;
@@ -227,23 +230,28 @@ namespace Ogre
         mMainBuffer->unlock();
 
         mBounds.setExtents( 
-            ( Real ) startx * msOptions->scale.x, 
+            ( Real ) startx * mOptions->scale.x, 
             min, 
-            ( Real ) startz * msOptions->scale.z,
-            ( Real ) ( endx - 1 ) * msOptions->scale.x, 
+            ( Real ) startz * mOptions->scale.z,
+            ( Real ) ( endx - 1 ) * mOptions->scale.x, 
             max, 
-            ( Real ) ( endz - 1 ) * msOptions->scale.z );
+            ( Real ) ( endz - 1 ) * mOptions->scale.z );
 
 
-        mCenter = Vector3( ( startx * msOptions->scale.x + (endx - 1) * msOptions->scale.x ) / 2,
+        mCenter = Vector3( ( startx * mOptions->scale.x + (endx - 1) * mOptions->scale.x ) / 2,
             ( min + max ) / 2,
-            ( startz * msOptions->scale.z + (endz - 1) * msOptions->scale.z ) / 2 );
+            ( startz * mOptions->scale.z + (endz - 1) * mOptions->scale.z ) / 2 );
+
+        mBoundingRadius = Math::Sqrt(
+            Math::Sqr(max - min) +
+            Math::Sqr((endx - 1 - startx) * mOptions->scale.x) +
+            Math::Sqr((endz - 1 - startz) * mOptions->scale.z)) / 2;
 
         // Create delta buffer list if required to morph
-        if (msOptions->lodMorph)
+        if (mOptions->lodMorph)
         {
             // Create delta buffer for all except the lowest mip
-            mDeltaBuffers = new HardwareVertexBufferSharedPtr[msOptions->maxGeoMipMapLevel - 1];
+            mDeltaBuffers = new HardwareVertexBufferSharedPtr[mOptions->maxGeoMipMapLevel - 1];
         }
 
         Real C = _calculateCFactor();
@@ -255,7 +263,7 @@ namespace Ogre
     void TerrainRenderable::_getNormalAt( float x, float z, Vector3 * result )
     {
 
-        assert(msOptions->lit && "No normals present");
+        assert(mOptions->lit && "No normals present");
 
         Vector3 here, left, down;
         here.x = x;
@@ -290,7 +298,7 @@ namespace Ogre
 
         Vector3 norm;
 
-        assert (msOptions->lit && "No normals present");
+        assert (mOptions->lit && "No normals present");
 
         HardwareVertexBufferSharedPtr vbuf = 
             mTerrain->vertexBufferBinding->getBuffer(MAIN_BINDING);
@@ -298,9 +306,9 @@ namespace Ogre
         unsigned char* pBase = static_cast<unsigned char*>( vbuf->lock(HardwareBuffer::HBL_DISCARD) );
         float* pNorm;
 
-        for ( size_t j = 0; j < msOptions->tileSize; j++ )
+        for ( size_t j = 0; j < mOptions->tileSize; j++ )
         {
-            for ( size_t i = 0; i < msOptions->tileSize; i++ )
+            for ( size_t i = 0; i < mOptions->tileSize; i++ )
             {
 
                 _getNormalAt( _vertex( i, j, 0 ), _vertex( i, j, 2 ), &norm );
@@ -319,6 +327,7 @@ namespace Ogre
     //-----------------------------------------------------------------------
     void TerrainRenderable::_notifyCurrentCamera( Camera* cam )
     {
+		MovableObject::_notifyCurrentCamera(cam);
 
         if ( mForcedRenderLevel >= 0 )
         {
@@ -328,13 +337,16 @@ namespace Ogre
 
 
         Vector3 cpos = cam -> getDerivedPosition();
-        Vector3 diff = mCenter - cpos;
+        const AxisAlignedBox& aabb = getWorldBoundingBox(true);
+        Vector3 diff(0, 0, 0);
+        diff.makeFloor(cpos - aabb.getMinimum());
+        diff.makeCeil(cpos - aabb.getMaximum());
 
         Real L = diff.squaredLength();
 
         mRenderLevel = -1;
 
-        for ( int i = 0; i < msOptions->maxGeoMipMapLevel; i++ )
+        for ( int i = 0; i < mOptions->maxGeoMipMapLevel; i++ )
         {
             if ( mMinLevelDistSqr[ i ] > L )
             {
@@ -344,9 +356,9 @@ namespace Ogre
         }
 
         if ( mRenderLevel < 0 )
-            mRenderLevel = msOptions->maxGeoMipMapLevel - 1;
+            mRenderLevel = mOptions->maxGeoMipMapLevel - 1;
 
-        if (msOptions->lodMorph)
+        if (mOptions->lodMorph)
         {
             // Get the next LOD level down
             int nextLevel = mNextLevelDown[mRenderLevel];
@@ -364,8 +376,8 @@ namespace Ogre
                 {
                     Real percent = (L - mMinLevelDistSqr[mRenderLevel]) / range;
                     // scale result so that msLODMorphStart == 0, 1 == 1, clamp to 0 below that
-                    Real rescale = 1.0f / (1.0f - msOptions->lodMorphStart);
-                    mLODMorphFactor = std::max((percent - msOptions->lodMorphStart) * rescale, 
+                    Real rescale = 1.0f / (1.0f - mOptions->lodMorphStart);
+                    mLODMorphFactor = std::max((percent - mOptions->lodMorphStart) * rescale, 
 						static_cast<Real>(0.0));
                 }
                 else
@@ -401,7 +413,10 @@ namespace Ogre
     //-----------------------------------------------------------------------
     void TerrainRenderable::_updateRenderQueue( RenderQueue* queue )
     {
-        queue->addRenderable( this );
+        // Notify need to calculate light list when our sending to render queue
+        mLightListDirty = true;
+
+        queue->addRenderable(this, mRenderQueueID);
     }
     //-----------------------------------------------------------------------
     void TerrainRenderable::getRenderOperation( RenderOperation& op )
@@ -411,7 +426,7 @@ namespace Ogre
         assert( mInit && "Uninitialized" );
 
         op.useIndexes = true;
-        op.operationType = msOptions->useTriStrips ? 
+        op.operationType = mOptions->useTriStrips ? 
             RenderOperation::OT_TRIANGLE_STRIP : RenderOperation::OT_TRIANGLE_LIST;
         op.vertexData = mTerrain;
         op.indexData = getIndexData();
@@ -452,7 +467,7 @@ namespace Ogre
 
         int i, j;
 
-        for ( int level = 1; level < msOptions->maxGeoMipMapLevel; level++ )
+        for ( int level = 1; level < mOptions->maxGeoMipMapLevel; level++ )
         {
             mMinLevelDistSqr[ level ] = 0;
 
@@ -461,7 +476,7 @@ namespace Ogre
             int higherstep = step >> 1;
 
             float* pDeltas = 0;
-            if (msOptions->lodMorph)
+            if (mOptions->lodMorph)
             {
                 // Create a set of delta values (store at index - 1 since 0 has none)
                 mDeltaBuffers[level - 1]  = createDeltaBuffer();
@@ -470,9 +485,9 @@ namespace Ogre
                     mDeltaBuffers[level - 1]->lock(HardwareBuffer::HBL_NORMAL));
             }
 
-            for ( j = 0; j < msOptions->tileSize - step; j += step )
+            for ( j = 0; j < mOptions->tileSize - step; j += step )
             {
-                for ( i = 0; i < msOptions->tileSize - step; i += step )
+                for ( i = 0; i < mOptions->tileSize - step; i += step )
                 {
                     /* Form planes relating to the lower detail tris to be produced
                     For tri lists and even tri strip rows, they are this shape:
@@ -492,7 +507,7 @@ namespace Ogre
 
                     Plane t1, t2;
                     bool backwardTri = false;
-                    if (!msOptions->useTriStrips || j % 2 == 0)
+                    if (!mOptions->useTriStrips || j % 2 == 0)
                     {
                         t1.redefine(v1, v3, v2);
                         t2.redefine(v2, v3, v4);
@@ -505,11 +520,11 @@ namespace Ogre
                     }
 
                     // include the bottommost row of vertices if this is the last row
-                    int zubound = (j == (msOptions->tileSize - step)? step : step - 1);
+                    int zubound = (j == (mOptions->tileSize - step)? step : step - 1);
                     for ( int z = 0; z <= zubound; z++ )
                     {
                         // include the rightmost col of vertices if this is the last col
-                        int xubound = (i == (msOptions->tileSize - step)? step : step - 1);
+                        int xubound = (i == (mOptions->tileSize - step)? step : step - 1);
                         for ( int x = 0; x <= xubound; x++ )
                         {
                             int fulldetailx = i + x;
@@ -559,12 +574,12 @@ namespace Ogre
 
                             // Should be save height difference?
                             // Don't morph along edges
-                            if (msOptions->lodMorph && 
-                                fulldetailx != 0  && fulldetailx != (msOptions->tileSize - 1) && 
-                                fulldetailz != 0  && fulldetailz != (msOptions->tileSize - 1) )
+                            if (mOptions->lodMorph && 
+                                fulldetailx != 0  && fulldetailx != (mOptions->tileSize - 1) && 
+                                fulldetailz != 0  && fulldetailz != (mOptions->tileSize - 1) )
                             {
                                 // Save height difference 
-                                pDeltas[fulldetailx + (fulldetailz * msOptions->tileSize)] = 
+                                pDeltas[fulldetailx + (fulldetailz * mOptions->tileSize)] = 
                                     interp_h - actual_h;
                             }
 
@@ -575,7 +590,7 @@ namespace Ogre
             }
 
             // Unlock morph deltas if required
-            if (msOptions->lodMorph)
+            if (mOptions->lodMorph)
             {
                 mDeltaBuffers[level - 1]->unlock();
             }
@@ -584,7 +599,7 @@ namespace Ogre
 
 
         // Post validate the whole set
-        for ( i = 1; i < msOptions->maxGeoMipMapLevel; i++ )
+        for ( i = 1; i < mOptions->maxGeoMipMapLevel; i++ )
         {
 
             // Make sure no LOD transition within the tile
@@ -606,9 +621,9 @@ namespace Ogre
         // Now reverse traverse the list setting the 'next level down'
         Real lastDist = -1;
         int lastIndex = 0;
-        for (i = msOptions->maxGeoMipMapLevel - 1; i >= 0; --i)
+        for (i = mOptions->maxGeoMipMapLevel - 1; i >= 0; --i)
         {
-            if (i == msOptions->maxGeoMipMapLevel - 1)
+            if (i == mOptions->maxGeoMipMapLevel - 1)
             {
                 // Last one is always 0
                 lastIndex = i;
@@ -628,39 +643,6 @@ namespace Ogre
         }
 
 
-    }
-    //-----------------------------------------------------------------------
-    void TerrainRenderable::_initLevelIndexes()
-    {
-        if ( mLevelInit )
-            return ;
-
-
-        if ( mLevelIndex.size() == 0 )
-        {
-            for ( int i = 0; i < 16; i++ )
-            {
-
-                mLevelIndex.push_back( new IndexMap() );
-
-            }
-
-        }
-
-        mLevelInit = true;
-    }
-    //-----------------------------------------------------------------------
-    void TerrainRenderable::_destroyLevelIndexes()
-    {
-        if ( mLevelInit )
-        {
-            for ( int i = 0; i < 16; i++ )
-            {
-                delete mLevelIndex[i];
-            }
-            mLevelIndex.clear();
-            mLevelInit = false;
-        }
     }
     //-----------------------------------------------------------------------
     void TerrainRenderable::_adjustRenderLevel( int i )
@@ -673,8 +655,7 @@ namespace Ogre
     {
         Real A, T;
 
-        const TerrainOptions& opts = TerrainSceneManager::getOptions();
-        if (!opts.primaryCamera)
+        if (!mOptions->primaryCamera)
         {
             OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
                 "You have not created a camera yet!", 
@@ -685,9 +666,9 @@ namespace Ogre
         // Turn off detail compression at higher FOVs
         A = 1.0f;
 
-        int vertRes = opts.primaryCamera->getViewport()->getActualHeight();
+        int vertRes = mOptions->primaryCamera->getViewport()->getActualHeight();
 
-        T = 2 * ( Real ) opts.maxPixelError / ( Real ) vertRes;
+        T = 2 * ( Real ) mOptions->maxPixelError / ( Real ) vertRes;
 
         return A / T;
     }
@@ -701,9 +682,9 @@ namespace Ogre
         start.y = _vertex( 0, 0, 1 );
         start.z = _vertex( 0, 0, 2 );
 
-        end.x = _vertex( msOptions->tileSize - 1, msOptions->tileSize - 1, 0 );
-        end.y = _vertex( msOptions->tileSize - 1, msOptions->tileSize - 1, 1 );
-        end.z = _vertex( msOptions->tileSize - 1, msOptions->tileSize - 1, 2 );
+        end.x = _vertex( mOptions->tileSize - 1, mOptions->tileSize - 1, 0 );
+        end.y = _vertex( mOptions->tileSize - 1, mOptions->tileSize - 1, 1 );
+        end.z = _vertex( mOptions->tileSize - 1, mOptions->tileSize - 1, 2 );
 
         /* Safety catch, if the point asked for is outside
         * of this tile, it will ask the appropriate tile
@@ -746,14 +727,14 @@ namespace Ogre
         float x_pct = ( x - start.x ) / ( end.x - start.x );
         float z_pct = ( z - start.z ) / ( end.z - start.z );
 
-        float x_pt = x_pct * ( float ) ( msOptions->tileSize - 1 );
-        float z_pt = z_pct * ( float ) ( msOptions->tileSize - 1 );
+        float x_pt = x_pct * ( float ) ( mOptions->tileSize - 1 );
+        float z_pt = z_pct * ( float ) ( mOptions->tileSize - 1 );
 
         int x_index = ( int ) x_pt;
         int z_index = ( int ) z_pt;
 
         // If we got to the far right / bottom edge, move one back
-        if (x_index == msOptions->tileSize - 1)
+        if (x_index == mOptions->tileSize - 1)
         {
             --x_index;
             x_pct = 1.0f;
@@ -763,7 +744,7 @@ namespace Ogre
             // get remainder
             x_pct = x_pt - x_index;
         }
-        if (z_index == msOptions->tileSize - 1)
+        if (z_index == mOptions->tileSize - 1)
         {
             --z_index;
             z_pct = 1.0f;
@@ -819,16 +800,15 @@ namespace Ogre
         //dir.y *= mScale.y;
         //dir.z *= mScale.z;
 
-        const Vector3 * corners = getBoundingBox().getAllCorners();
-
+        const AxisAlignedBox& box = getBoundingBox();
         //start with the next one...
         ray += dir;
 
 
-        while ( ! ( ( ray.x < corners[ 0 ].x ) ||
-            ( ray.x > corners[ 4 ].x ) ||
-            ( ray.z < corners[ 0 ].z ) ||
-            ( ray.z > corners[ 4 ].z ) ) )
+        while ( ! ( ( ray.x < box.getMinimum().x ) ||
+            ( ray.x > box.getMaximum().x ) ||
+            ( ray.z < box.getMinimum().z ) ||
+            ( ray.z > box.getMaximum().z ) ) )
         {
 
 
@@ -849,13 +829,13 @@ namespace Ogre
 
         }
 
-        if ( ray.x < corners[ 0 ].x && mNeighbors[ WEST ] != 0 )
+        if ( ray.x < box.getMinimum().x && mNeighbors[ WEST ] != 0 )
             return mNeighbors[ WEST ] ->intersectSegment( ray, end, result );
-        else if ( ray.z < corners[ 0 ].z && mNeighbors[ NORTH ] != 0 )
+        else if ( ray.z < box.getMinimum().z && mNeighbors[ NORTH ] != 0 )
             return mNeighbors[ NORTH ] ->intersectSegment( ray, end, result );
-        else if ( ray.x > corners[ 4 ].x && mNeighbors[ EAST ] != 0 )
+        else if ( ray.x > box.getMaximum().x && mNeighbors[ EAST ] != 0 )
             return mNeighbors[ EAST ] ->intersectSegment( ray, end, result );
-        else if ( ray.z > corners[ 4 ].z && mNeighbors[ SOUTH ] != 0 )
+        else if ( ray.z > box.getMaximum().z && mNeighbors[ SOUTH ] != 0 )
             return mNeighbors[ SOUTH ] ->intersectSegment( ray, end, result );
         else
         {
@@ -866,7 +846,7 @@ namespace Ogre
         }
     }
     //-----------------------------------------------------------------------
-    void TerrainRenderable::_generateVertexLighting( const Vector3 &sun, ColourValue ambient )
+    void TerrainRenderable::_generateVertexLighting( const Vector3 &sunlight, ColourValue ambient )
     {
 
         Vector3 pt;
@@ -877,20 +857,20 @@ namespace Ogre
             mTerrain->vertexBufferBinding->getBuffer(MAIN_BINDING);
         const VertexElement* elem = mTerrain->vertexDeclaration->findElementBySemantic(VES_DIFFUSE);
         //for each point in the terrain, see if it's in the line of sight for the sun.
-        for ( size_t i = 0; i < msOptions->tileSize; i++ )
+        for ( size_t i = 0; i < mOptions->tileSize; i++ )
         {
-            for ( size_t j = 0; j < msOptions->tileSize; j++ )
+            for ( size_t j = 0; j < mOptions->tileSize; j++ )
             {
                 //  printf( "Checking %f,%f,%f ", pt.x, pt.y, pt.z );
                 pt.x = _vertex( i, j, 0 );
                 pt.y = _vertex( i, j, 1 );
                 pt.z = _vertex( i, j, 2 );
 
-                light = sun - pt;
+                light = sunlight - pt;
 
                 light.normalise();
 
-                if ( ! intersectSegment( pt, sun, 0 ) )
+                if ( ! intersectSegment( pt, sunlight, 0 ) )
                 {
                     //
                     _getNormalAt( _vertex( i, j, 0 ), _vertex( i, j, 2 ), &normal );
@@ -948,7 +928,13 @@ namespace Ogre
     //-----------------------------------------------------------------------
     const LightList& TerrainRenderable::getLights(void) const
     {
-        return getParentSceneNode()->findLights(this->getBoundingRadius());
+        if (mLightListDirty)
+        {
+            getParentSceneNode()->getCreator()->_populateLightList(
+                mCenter, this->getBoundingRadius(), mLightList);
+            mLightListDirty = false;
+        }
+        return mLightList;
     }
     //-----------------------------------------------------------------------
     IndexData* TerrainRenderable::getIndexData(void)
@@ -984,12 +970,13 @@ namespace Ogre
         }
 
         // Check preexisting
-        IndexMap::iterator ii = mLevelIndex[ mRenderLevel ]->find( stitchFlags );
+		LevelArray& levelIndex = mSceneManager->_getLevelIndex();
+        IndexMap::iterator ii = levelIndex[ mRenderLevel ]->find( stitchFlags );
         IndexData* indexData;
-        if ( ii == mLevelIndex[ mRenderLevel ]->end())
+        if ( ii == levelIndex[ mRenderLevel ]->end())
         {
             // Create
-            if (msOptions->useTriStrips)
+            if (mOptions->useTriStrips)
             {
                 indexData = generateTriStripIndexes(stitchFlags);
             }
@@ -997,7 +984,7 @@ namespace Ogre
             {
                 indexData = generateTriListIndexes(stitchFlags);
             }
-            mLevelIndex[ mRenderLevel ]->insert(
+            levelIndex[ mRenderLevel ]->insert(
                 IndexMap::value_type(stitchFlags, indexData));
         }
         else
@@ -1023,9 +1010,9 @@ namespace Ogre
         // Calculate the number of indexes required
         // This is the number of 'cells' at this detail level x 2
         // plus 3 degenerates to turn corners
-        int numTrisAcross = (((msOptions->tileSize-1) / step) * 2) + 3;
+        int numTrisAcross = (((mOptions->tileSize-1) / step) * 2) + 3;
         // Num indexes is number of tris + 2
-        int new_length = numTrisAcross * ((msOptions->tileSize-1) / step) + 2;
+        int new_length = numTrisAcross * ((mOptions->tileSize-1) / step) + 2;
         //this is the maximum for a level.  It wastes a little, but shouldn't be a problem.
 
         IndexData* indexData = new IndexData;
@@ -1034,7 +1021,7 @@ namespace Ogre
             HardwareIndexBuffer::IT_16BIT,
             new_length, HardwareBuffer::HBU_STATIC_WRITE_ONLY);//, false);
 
-        msIndexCache.mCache.push_back( indexData );
+        mSceneManager->_getIndexCache().mCache.push_back( indexData );
 
         unsigned short* pIdx = static_cast<unsigned short*>(
             indexData->indexBuffer->lock(0, 
@@ -1042,12 +1029,12 @@ namespace Ogre
             HardwareBuffer::HBL_DISCARD));
 
         // Stripified mesh
-        for ( int j = 0; j < msOptions->tileSize - 1; j += step )
+        for ( int j = 0; j < mOptions->tileSize - 1; j += step )
         {
             int i;
             // Forward strip
             // We just do the |/ here, final | done after
-            for ( i = 0; i < msOptions->tileSize - 1; i += step )
+            for ( i = 0; i < mOptions->tileSize - 1; i += step )
             {
                 int x[4], y[4];
                 x[0] = x[1] = i;
@@ -1085,7 +1072,7 @@ namespace Ogre
                         y[1] -= step;
                     }
                 }
-                if (i == (msOptions->tileSize - 1 - step) && (stitchFlags & STITCH_EAST))
+                if (i == (mOptions->tileSize - 1 - step) && (stitchFlags & STITCH_EAST))
                 {
                     // East tiling means rounding y[2] & y[3]
                     if (y[2] % lowstep != 0)
@@ -1107,11 +1094,11 @@ namespace Ogre
                 *pIdx++ = _index( x[1], y[1] ); numIndexes++;
                 *pIdx++ = _index( x[2], y[2] ); numIndexes++;
 
-                if (i == msOptions->tileSize - 1 - step)
+                if (i == mOptions->tileSize - 1 - step)
                 {
                     // Emit extra index to finish row
                     *pIdx++ = _index( x[3], y[3] ); numIndexes++;
-                    if (j < msOptions->tileSize - 1 - step)
+                    if (j < mOptions->tileSize - 1 - step)
                     {
                         // Emit this index twice more (this is to turn around without
                         // artefacts)
@@ -1125,7 +1112,7 @@ namespace Ogre
             // Increment row
             j += step;
             // Backward strip
-            for ( i = msOptions->tileSize - 1; i > 0 ; i -= step )
+            for ( i = mOptions->tileSize - 1; i > 0 ; i -= step )
             {
                 int x[4], y[4];
                 x[0] = x[1] = i;
@@ -1135,7 +1122,7 @@ namespace Ogre
 
                 // Never get a north tiling on a backward strip (always
                 // start on a forward strip)
-                if (j == (msOptions->tileSize - 1 - step) && (stitchFlags & STITCH_SOUTH))
+                if (j == (mOptions->tileSize - 1 - step) && (stitchFlags & STITCH_SOUTH))
                 {
                     // South reduction means rounding x[1] / x[3]
                     if (x[1] % lowstep != 0)
@@ -1160,7 +1147,7 @@ namespace Ogre
                         y[3] -= step;
                     }
                 }
-                if (i == msOptions->tileSize - 1 && (stitchFlags & STITCH_EAST))
+                if (i == mOptions->tileSize - 1 && (stitchFlags & STITCH_EAST))
                 {
                     // East tiling means rounding y[0] and y[1] on backward strip
                     if (y[0] % lowstep != 0)
@@ -1174,7 +1161,7 @@ namespace Ogre
                 }
 
                 //triangles
-                if (i == msOptions->tileSize)
+                if (i == mOptions->tileSize)
                 {
                     // Starter
                     *pIdx++ = _index( x[0], y[0] ); numIndexes++;
@@ -1186,7 +1173,7 @@ namespace Ogre
                 {
                     // Emit extra index to finish row
                     *pIdx++ = _index( x[3], y[3] ); numIndexes++;
-                    if (j < msOptions->tileSize - 1 - step)
+                    if (j < mOptions->tileSize - 1 - step)
                     {
                         // Emit this index once more (this is to turn around)
                         *pIdx++ = _index( x[3], y[3] ); numIndexes++;
@@ -1217,7 +1204,7 @@ namespace Ogre
         int east = stitchFlags & STITCH_EAST ? step : 0;
         int west = stitchFlags & STITCH_WEST ? step : 0;
 
-        int new_length = ( msOptions->tileSize / step ) * ( msOptions->tileSize / step ) * 2 * 2 * 2 ;
+        int new_length = ( mOptions->tileSize / step ) * ( mOptions->tileSize / step ) * 2 * 2 * 2 ;
         //this is the maximum for a level.  It wastes a little, but shouldn't be a problem.
 
         indexData = new IndexData;
@@ -1226,7 +1213,7 @@ namespace Ogre
             HardwareIndexBuffer::IT_16BIT,
             new_length, HardwareBuffer::HBU_STATIC_WRITE_ONLY);//, false);
 
-        msIndexCache.mCache.push_back( indexData );
+        mSceneManager->_getIndexCache().mCache.push_back( indexData );
 
         unsigned short* pIdx = static_cast<unsigned short*>(
             indexData->indexBuffer->lock(0, 
@@ -1234,9 +1221,9 @@ namespace Ogre
             HardwareBuffer::HBL_DISCARD));
 
         // Do the core vertices, minus stitches
-        for ( int j = north; j < msOptions->tileSize - 1 - south; j += step )
+        for ( int j = north; j < mOptions->tileSize - 1 - south; j += step )
         {
-            for ( int i = west; i < msOptions->tileSize - 1 - east; i += step )
+            for ( int i = west; i < mOptions->tileSize - 1 - east; i += step )
             {
                 //triangles
                 *pIdx++ = _index( i, j ); numIndexes++;
@@ -1288,11 +1275,11 @@ namespace Ogre
         HardwareVertexBufferSharedPtr buf = 
             HardwareBufferManager::getSingleton().createVertexBuffer(
             VertexElement::getTypeSize(VET_FLOAT1), 
-            msOptions->tileSize * msOptions->tileSize,
+            mOptions->tileSize * mOptions->tileSize,
             HardwareBuffer::HBU_STATIC_WRITE_ONLY);
         // Fill the buffer with zeros, we will only fill in delta
         void* pVoid = buf->lock(HardwareBuffer::HBL_DISCARD);
-        memset(pVoid, 0, msOptions->tileSize * msOptions->tileSize * sizeof(float));
+        memset(pVoid, 0, mOptions->tileSize * mOptions->tileSize * sizeof(float));
         buf->unlock();
 
         return buf;
@@ -1306,7 +1293,7 @@ namespace Ogre
         if (constantEntry.data == MORPH_CUSTOM_PARAM_ID)
         {
             // Update morph LOD factor
-            params->setConstant(constantEntry.index, mLODMorphFactor);
+            params->_writeRawConstant(constantEntry.physicalIndex, mLODMorphFactor);
         }
         else
         {
@@ -1367,13 +1354,13 @@ namespace Ogre
         {
         case NORTH:
             startx = starty = 0;
-            endx = msOptions->tileSize - 1;
+            endx = mOptions->tileSize - 1;
             rowstep = step;
             horizontal = true;
             break;
         case SOUTH:
             // invert x AND y direction, helps to keep same winding
-            startx = starty = msOptions->tileSize - 1;
+            startx = starty = mOptions->tileSize - 1;
             endx = 0;
             rowstep = -step;
             step = -step;
@@ -1383,13 +1370,13 @@ namespace Ogre
             break;
         case EAST:
             startx = 0;
-            endx = msOptions->tileSize - 1;
-            starty = msOptions->tileSize - 1;
+            endx = mOptions->tileSize - 1;
+            starty = mOptions->tileSize - 1;
             rowstep = -step;
             horizontal = false;
             break;
         case WEST:
-            startx = msOptions->tileSize - 1;
+            startx = mOptions->tileSize - 1;
             endx = 0;
             starty = 0;
             rowstep = step;
