@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 /***************************************************************************
@@ -48,16 +52,19 @@ Enhancements 2003 - 2004 (C) The OGRE Team
 #include "OgreTerrainVertexProgram.h"
 #include "OgreTerrainPage.h"
 #include "OgreLogManager.h"
+#include "OgreResourceGroupManager.h"
+#include "OgreMaterialManager.h"
+#include "OgreHeightmapTerrainPageSource.h"
+#include <fstream>
 
 #define TERRAIN_MATERIAL_NAME "TerrainSceneManager/Terrain"
 
 namespace Ogre
 {
     //-------------------------------------------------------------------------
-    TerrainOptions TerrainSceneManager::mOptions;
     //-------------------------------------------------------------------------
-    //-------------------------------------------------------------------------
-    TerrainSceneManager::TerrainSceneManager() : OctreeSceneManager( )
+    TerrainSceneManager::TerrainSceneManager(const String& name) 
+		: OctreeSceneManager(name)
     {
         //setDisplaySceneNodes( true );
         //setShowBoxes( true );
@@ -69,38 +76,46 @@ namespace Ogre
         mActivePageSource = 0;
         mPagingEnabled = false;
         mLivePageMargin = 0;
-        mBufferedPageMargin = 0;
-		// Construct listener manager singleton
-		new TerrainPageSourceListenerManager();
-		
+        mBufferedPageMargin = 0;		
 
 
     }
+	//-------------------------------------------------------------------------
+	const String& TerrainSceneManager::getTypeName(void) const
+	{
+		return TerrainSceneManagerFactory::FACTORY_TYPE_NAME;
+	}
 	//-------------------------------------------------------------------------
 	void TerrainSceneManager::shutdown(void)
 	{
 		// Make sure the indexes are destroyed during orderly shutdown
 		// and not when statics are destroyed (may be too late)
-		TerrainRenderable::_getIndexCache().shutdown();
+		mIndexCache.shutdown();
+		destroyLevelIndexes();
 
 		// Make sure we free up material (static)
 		mOptions.terrainMaterial.setNull();
 
-		// destroy listener manager
-		delete TerrainPageSourceListenerManager::getSingletonPtr();
+		// Shut down page source to free terrain pages
+		if (mActivePageSource)
+		{
+			mActivePageSource->shutdown();
+		}
+
 	}
     //-------------------------------------------------------------------------
     TerrainSceneManager::~TerrainSceneManager()
     {
+		shutdown();
     }
     //-------------------------------------------------------------------------
-    void TerrainSceneManager::loadConfig(const String& filename)
+    void TerrainSceneManager::loadConfig(DataStreamPtr& stream)
     {
         /* Set up the options */
         ConfigFile config;
         String val;
 
-        config.load( filename );
+        config.load( stream );
 
         val = config.getSetting( "DetailTile" );
         if ( !val.empty() )
@@ -141,8 +156,8 @@ namespace Ogre
             v.z = atof( val.c_str() );
 
         // Scale x/z relative to pagesize
-        v.x /= mOptions.pageSize;
-        v.z /= mOptions.pageSize;
+        v.x /= mOptions.pageSize - 1;
+        v.z /= mOptions.pageSize - 1;
         setScale(v);
 
         val = config.getSetting( "MaxPixelError" );
@@ -268,10 +283,14 @@ namespace Ogre
             // define our own material
             mOptions.terrainMaterial = 
                 MaterialManager::getSingleton().getByName(TERRAIN_MATERIAL_NAME);
+			// Make unique terrain material name
+			StringUtil::StrStreamType s;
+			s << mName << "/Terrain";
+			mOptions.terrainMaterial = MaterialManager::getSingleton().getByName(s.str());
             if (mOptions.terrainMaterial.isNull())
             {
                 mOptions.terrainMaterial = MaterialManager::getSingleton().create(
-                    "TerrainSceneManager/Terrain",
+                    s.str(),
                     ResourceGroupManager::getSingleton().getWorldResourceGroupName());
 
             }
@@ -444,6 +463,29 @@ namespace Ogre
     }
     //-------------------------------------------------------------------------
     void TerrainSceneManager::setWorldGeometry( const String& filename )
+    {
+		// try to open in the current folder first
+		std::ifstream fs;
+		fs.open(filename.c_str(), std::ios::in | std::ios::binary);
+		if (fs)
+		{
+			// Wrap as a stream
+			DataStreamPtr stream(
+				new FileStreamDataStream(filename, &fs, false));
+			setWorldGeometry(stream);
+		}
+		else
+		{
+			// otherwise try resource system
+			DataStreamPtr stream = 
+				ResourceGroupManager::getSingleton().openResource(filename, 
+					ResourceGroupManager::getSingleton().getWorldResourceGroupName());
+				
+			setWorldGeometry(stream);
+		}
+	}
+    //-------------------------------------------------------------------------
+    void TerrainSceneManager::setWorldGeometry(DataStreamPtr& stream, const String& typeName )
     {
         // Clear out any existing world resources (if not default)
         if (ResourceGroupManager::getSingleton().getWorldResourceGroupName() != 
@@ -452,9 +494,11 @@ namespace Ogre
             ResourceGroupManager::getSingleton().clearResourceGroup(
                 ResourceGroupManager::getSingleton().getWorldResourceGroupName());
         }
+		destroyLevelIndexes();
         mTerrainPages.clear();
         // Load the configuration
-        loadConfig(filename);
+        loadConfig(stream);
+		initLevelIndexes();
 
         // Resize the octree, allow for 1 page for now
         float max_x = mOptions.scale.x * mOptions.pageSize;
@@ -472,6 +516,7 @@ namespace Ogre
     {
         OctreeSceneManager::clearScene();
         mTerrainPages.clear();
+		destroyLevelIndexes();
         // Octree has destroyed our root
         mTerrainRoot = 0;
     }
@@ -495,15 +540,14 @@ namespace Ogre
         // Insert page into list
         mTerrainPages[pageX][pageZ] = page;
         // Attach page to terrain root
-        mTerrainRoot->addChild(page->pageSceneNode);
+		if (page->pageSceneNode->getParentSceneNode() != mTerrainRoot)
+			mTerrainRoot->addChild(page->pageSceneNode);
 
     }
     //-------------------------------------------------------------------------
     void TerrainSceneManager::_renderVisibleObjects( void )
     {
 
-        mDestRenderSystem -> setLightingEnabled( false );
-
         OctreeSceneManager::_renderVisibleObjects();
 
     }
@@ -773,8 +817,15 @@ namespace Ogre
     void TerrainSceneManager::registerPageSource(const String& typeName, 
         TerrainPageSource* source)
     {
-        mPageSources.insert(
-            PageSourceMap::value_type(typeName, source));
+		std::pair<PageSourceMap::iterator, bool> retPair = 
+			mPageSources.insert(
+				PageSourceMap::value_type(typeName, source));
+		if (!retPair.second)
+		{
+			OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM, 
+				"The page source " + typeName + " is already registered",
+				"TerrainSceneManager::registerPageSource");
+		}
         LogManager::getSingleton().logMessage(
             "TerrainSceneManager: Registered a new PageSource for "
             "type " + typeName);
@@ -804,7 +855,7 @@ namespace Ogre
 
     }
     //-------------------------------------------------------------------------
-    int TerrainSceneManager::setDetailTextureRepeat(void)
+    int TerrainSceneManager::getDetailTextureRepeat(void)
     {
         return mOptions.detailTile;
     }
@@ -833,6 +884,29 @@ namespace Ogre
     {
         return mOptions.maxGeoMipMapLevel;
     }
+	//-----------------------------------------------------------------------
+	void TerrainSceneManager::initLevelIndexes()
+	{
+		if ( mLevelIndex.size() == 0 )
+		{
+			for ( int i = 0; i < 16; i++ )
+			{
+
+				mLevelIndex.push_back( new IndexMap() );
+
+			}
+
+		}
+	}
+	//-----------------------------------------------------------------------
+	void TerrainSceneManager::destroyLevelIndexes()
+	{
+		for ( int i = 0; i < mLevelIndex.size(); i++ )
+		{
+			delete mLevelIndex[i];
+		}
+		mLevelIndex.clear();
+	}
     //-------------------------------------------------------------------------
     //-------------------------------------------------------------------------
     RaySceneQuery* 
@@ -856,8 +930,7 @@ namespace Ogre
     //-------------------------------------------------------------------------
     void TerrainRaySceneQuery::execute(RaySceneQueryListener* listener)
     {
-        static WorldFragment worldFrag;
-        worldFrag.fragmentType = SceneQuery::WFT_SINGLE_INTERSECTION;
+        mWorldFrag.fragmentType = SceneQuery::WFT_SINGLE_INTERSECTION;
 
         const Vector3& dir = mRay.getDirection();
         const Vector3& origin = mRay.getOrigin();
@@ -868,11 +941,11 @@ namespace Ogre
                 origin.x, origin.z);
             if (height != -1 && (height <= origin.y && dir.y < 0) || (height >= origin.y && dir.y > 0))
             {
-                worldFrag.singleIntersection.x = origin.x;
-                worldFrag.singleIntersection.z = origin.z;
-                worldFrag.singleIntersection.y = height;
-                if (!listener->queryResult(&worldFrag, 
-                    (worldFrag.singleIntersection - origin).length()))
+                mWorldFrag.singleIntersection.x = origin.x;
+                mWorldFrag.singleIntersection.z = origin.z;
+                mWorldFrag.singleIntersection.y = height;
+                if (!listener->queryResult(&mWorldFrag, 
+                    (mWorldFrag.singleIntersection - origin).length()))
 					return;
             }
         }
@@ -880,10 +953,10 @@ namespace Ogre
         {
             // Perform arbitrary query
             if (static_cast<TerrainSceneManager*>(mParentSceneMgr)->intersectSegment(
-                origin, origin + (dir * 100000), &worldFrag.singleIntersection))
+                origin, origin + (dir * 100000), &mWorldFrag.singleIntersection))
             {
-                if (!listener->queryResult(&worldFrag, 
-                    (worldFrag.singleIntersection - origin).length()))
+                if (!listener->queryResult(&mWorldFrag, 
+                    (mWorldFrag.singleIntersection - origin).length()))
 					return;
             }
 
@@ -903,8 +976,10 @@ namespace Ogre
         return PageSourceIterator(mPageSources.begin(), mPageSources.end());
     }
 	//-------------------------------------------------------------------------
-	void TerrainSceneManager::setWorldGeometryRenderQueue(RenderQueueGroupID qid)
+	void TerrainSceneManager::setWorldGeometryRenderQueue(uint8 qid)
 	{
+		OctreeSceneManager::setWorldGeometryRenderQueue(qid);
+
 		for (TerrainPage2D::iterator pi = mTerrainPages.begin(); 
 			pi != mTerrainPages.end(); ++pi)
 		{
@@ -921,5 +996,49 @@ namespace Ogre
 
 	}
 
+	//-----------------------------------------------------------------------
+	const String TerrainSceneManagerFactory::FACTORY_TYPE_NAME = "TerrainSceneManager";
+	//-----------------------------------------------------------------------
+	TerrainSceneManagerFactory::TerrainSceneManagerFactory()
+	{
+	}
+	//-----------------------------------------------------------------------
+	TerrainSceneManagerFactory::~TerrainSceneManagerFactory()
+	{
+		for (TerrainPageSources::iterator i = mTerrainPageSources.begin();
+			i != mTerrainPageSources.end(); ++i)
+		{
+			delete *i;
+		}
+		mTerrainPageSources.clear();
+	}
+	//-----------------------------------------------------------------------
+	void TerrainSceneManagerFactory::initMetaData(void) const
+	{
+		mMetaData.typeName = FACTORY_TYPE_NAME;
+		mMetaData.description = "Scene manager which generally organises the scene on "
+			"the basis of an octree, but also supports terrain world geometry. ";
+		mMetaData.sceneTypeMask = ST_EXTERIOR_CLOSE; // previous compatiblity
+		mMetaData.worldGeometrySupported = true;
+	}
+	//-----------------------------------------------------------------------
+	SceneManager* TerrainSceneManagerFactory::createInstance(
+		const String& instanceName)
+	{
+		TerrainSceneManager* tsm = new TerrainSceneManager(instanceName);
+		// Create & register default sources (one per manager)
+		HeightmapTerrainPageSource* ps = new HeightmapTerrainPageSource();
+		mTerrainPageSources.push_back(ps);
+		tsm->registerPageSource("Heightmap", ps);
+
+		return tsm;
+
+	}
+	//-----------------------------------------------------------------------
+	void TerrainSceneManagerFactory::destroyInstance(SceneManager* instance)
+	{
+		delete instance;
+	}
+
 
 } //namespace
