@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #ifndef __D3D9RENDERWINDOW_H__
@@ -44,6 +48,7 @@ namespace Ogre
 		~D3D9RenderWindow();
 		void create(const String& name, unsigned int width, unsigned int height,
 	            bool fullScreen, const NameValuePairList *miscParams);
+		void setFullscreen(bool fullScreen, unsigned int width, unsigned int height);
 		void destroy(void);
 		bool isVisible() const;
 		bool isClosed() const { return mClosed; }
@@ -70,8 +75,8 @@ namespace Ogre
 		{ return &md3dpp; }
 
 		/// @copydoc RenderTarget::update
-		void update();
-
+		void update(bool swap);
+		
 		/** Create (or recreate) the D3D device or SwapChain for this window.
 		*/
 		void createD3DResources();
@@ -80,6 +85,13 @@ namespace Ogre
 		*/
 		void destroyD3DResources();
 	
+		/// Accessor for render surface
+		LPDIRECT3DSURFACE9 getRenderSurface() { return mpRenderSurface; }
+
+		/// Are we in the middle of switching between fullscreen and windowed
+		bool _getSwitchingFullscreen() const { return mSwitchingFullscreen; }
+		/// Indicate that fullscreen / windowed switching has finished
+		void _finishSwitchingFullscreen() { mSwitchingFullscreen = false; }
 	protected:
 		HINSTANCE mInstance;			// Process instance
 		D3D9Driver *mDriver;			// D3D9 driver
@@ -88,12 +100,7 @@ namespace Ogre
 		bool	mSizing;
 		bool	mClosed;
 		bool	mIsSwapChain;			// Is this a secondary window?
-
-		static LRESULT CALLBACK WndProc(
-			HWND hWnd,
-			UINT uMsg,
-			WPARAM wParam,
-			LPARAM lParam );
+		bool	mSwitchingFullscreen;	// Are we switching from fullscreen to windowed or vice versa
 
 		// -------------------------------------------------------
 		// DirectX-specific
@@ -106,7 +113,9 @@ namespace Ogre
 		LPDIRECT3DSURFACE9 mpRenderZBuffer;
 		D3DMULTISAMPLE_TYPE mFSAAType;
 		DWORD mFSAAQuality;
+		UINT mDisplayFrequency;
 		bool mVSync;
+		bool mUseNVPerfHUD;
 
 		// just check if the multisampling requested is supported by the device
 		bool _checkMultiSampleQuality(D3DMULTISAMPLE_TYPE type, DWORD *outQuality, D3DFORMAT format, UINT adapterNum, D3DDEVTYPE deviceType, BOOL fullScreen);
