@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #include "OgreD3D9RenderWindow.h"
@@ -36,67 +40,14 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include <d3d9.h>
 #include "OgreMemoryMacros.h"
 #include "OgreRoot.h"
+#include "OgreWindowEventUtilities.h"
 
 namespace Ogre
 {
-	// Window procedure callback
-	// This is a static member, so applies to all windows but we store the
-	// D3D9RenderWindow instance in the window data GetWindowLog/SetWindowLog
-	LRESULT D3D9RenderWindow::WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
-	{
-		if (uMsg == WM_CREATE)
-		{
-			// copy D3D9RenderWindow* from createwindow param to userdata slot
-			SetWindowLong(hWnd, GWL_USERDATA,
-				(LONG)(((LPCREATESTRUCT)lParam)->lpCreateParams));
-			return 0;
-		}
-
-		D3D9RenderWindow* win =
-			(D3D9RenderWindow*)GetWindowLong(hWnd, GWL_USERDATA);
-
-		if (!win)
-			return DefWindowProc(hWnd, uMsg, wParam, lParam);
-
-		switch( uMsg )
-		{
-		case WM_ACTIVATE:
-			win->mActive = (LOWORD(wParam) != WA_INACTIVE);
-			break;
-
-		case WM_ENTERSIZEMOVE:
-			win->mSizing = true;
-			break;
-
-		case WM_EXITSIZEMOVE:
-			win->windowMovedOrResized();
-			win->mSizing = false;
-			break;
-
-		case WM_MOVE:
-		case WM_SIZE:
-			if (!win->mSizing)
-				win->windowMovedOrResized();
-			break;
-
-		case WM_GETMINMAXINFO:
-			// Prevent the window from going smaller than some minimu size
-			((MINMAXINFO*)lParam)->ptMinTrackSize.x = 100;
-			((MINMAXINFO*)lParam)->ptMinTrackSize.y = 100;
-			break;
-
-		case WM_CLOSE:
-			win->destroy(); // cleanup and call DestroyWindow
-			win->mClosed = true;
-			return 0;
-		}
-
-		return DefWindowProc( hWnd, uMsg, wParam, lParam );
-	}
-
-	D3D9RenderWindow::D3D9RenderWindow(HINSTANCE instance, D3D9Driver *driver, LPDIRECT3DDEVICE9 deviceIfSwapChain):
-	mInstance(instance),
-		mDriver(driver)
+	D3D9RenderWindow::D3D9RenderWindow(HINSTANCE instance, D3D9Driver *driver, LPDIRECT3DDEVICE9 deviceIfSwapChain)
+        : mInstance(instance)
+        , mDriver(driver)
+        , mpRenderSurface(0)
 	{
 		mIsFullScreen = false;
 		mIsSwapChain = (deviceIfSwapChain != NULL);
@@ -105,11 +56,13 @@ namespace Ogre
 		mActive = false;
 		mSizing = false;
 		mClosed = false;
+		mSwitchingFullscreen = false;
+		mDisplayFrequency = 0;
 	}
 
 	D3D9RenderWindow::~D3D9RenderWindow()
 	{
-		destroyD3DResources();
+		destroy();
 	}
 
 	bool D3D9RenderWindow::_checkMultiSampleQuality(D3DMULTISAMPLE_TYPE type, DWORD *outQuality, D3DFORMAT format, UINT adapterNum, D3DDEVTYPE deviceType, BOOL fullScreen)
@@ -136,7 +89,6 @@ namespace Ogre
 		mFSAAType = D3DMULTISAMPLE_NONE;
 		mFSAAQuality = 0;
 		mVSync = false;
-		unsigned int displayFrequency = 0;
 		String title = name;
 		unsigned int colourDepth = 32;
 		int left = -1; // Defaults to screen center
@@ -144,6 +96,7 @@ namespace Ogre
 		bool depthBuffer = true;
 		String border = "";
 		bool outerSize = false;
+		mUseNVPerfHUD = false;
 
 		if(miscParams)
 		{
@@ -176,7 +129,7 @@ namespace Ogre
 			// displayFrequency
 			opt = miscParams->find("displayFrequency");
 			if(opt != miscParams->end())
-				displayFrequency = StringConverter::parseUnsignedInt(opt->second);
+				mDisplayFrequency = StringConverter::parseUnsignedInt(opt->second);
 			// colourDepth
 			opt = miscParams->find("colourDepth");
 			if(opt != miscParams->end())
@@ -201,13 +154,17 @@ namespace Ogre
 			opt = miscParams->find("outerDimensions");
 			if(opt != miscParams->end())
 				outerSize = StringConverter::parseBool(opt->second);
+			// NV perf HUD?
+			opt = miscParams->find("useNVPerfHUD");
+			if(opt != miscParams->end())
+				mUseNVPerfHUD = StringConverter::parseBool(opt->second);
+			 
 		}
 
 		// Destroy current window if any
 		if( mHWnd )
 			destroy();
 
-
 		if (!externalHandle)
 		{
 			DWORD dwStyle = WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
@@ -265,7 +222,7 @@ namespace Ogre
 
 			// Register the window class
 			// NB allow 4 bytes of window data for D3D9RenderWindow pointer
-			WNDCLASS wc = { 0, WndProc, 0, 0, hInst,
+			WNDCLASS wc = { 0, WindowEventUtilities::_WndProc, 0, 0, hInst,
 				LoadIcon(0, IDI_APPLICATION), LoadCursor(NULL, IDC_ARROW),
 				(HBRUSH)GetStockObject(BLACK_BRUSH), 0, "OgreD3D9Wnd" };
 			RegisterClass(&wc);
@@ -275,6 +232,8 @@ namespace Ogre
 			mIsExternal = false;
 			mHWnd = CreateWindow("OgreD3D9Wnd", title.c_str(), dwStyle,
 				mLeft, mTop, mWidth, mHeight, parentHWnd, 0, hInst, this);
+			
+			WindowEventUtilities::_addRenderWindow(this);
 		}
 		else
 		{
@@ -307,6 +266,81 @@ namespace Ogre
 		createD3DResources();
 
 		mActive = true;
+		mClosed = false;
+	}
+
+	void D3D9RenderWindow::setFullscreen(bool fullScreen, unsigned int width, unsigned int height)
+	{
+		if (fullScreen != mIsFullScreen || width != mWidth || height != mHeight)
+		{
+
+			if (fullScreen != mIsFullScreen)
+				mSwitchingFullscreen = true;
+
+			DWORD dwStyle = WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
+
+			bool oldFullscreen = mIsFullScreen;
+			mIsFullScreen = fullScreen;
+
+			if (fullScreen)
+			{
+				dwStyle |= WS_POPUP;
+
+				mTop = mLeft = 0;
+				mWidth = width;
+				mHeight = height;
+				// need different ordering here
+				md3dpp.Windowed = !fullScreen;
+				md3dpp.FullScreen_RefreshRateInHz = mIsFullScreen ? mDisplayFrequency : 0;
+				md3dpp.BackBufferHeight = height;
+				md3dpp.BackBufferWidth = width;
+				if (oldFullscreen)
+				{
+					// was previously fullscreen, just changing the resolution
+					SetWindowPos(mHWnd, HWND_TOPMOST, 0, 0, width, height, SWP_NOACTIVATE);
+				}
+				else
+				{
+					SetWindowPos(mHWnd, HWND_TOPMOST, 0, 0, width, height, SWP_NOACTIVATE);
+					//MoveWindow(mHWnd, mLeft, mTop, mWidth, mHeight, FALSE);
+					SetWindowLong(mHWnd, GWL_STYLE, dwStyle);
+				}
+			}
+			else
+			{
+				dwStyle |= WS_OVERLAPPEDWINDOW;
+				// Calculate window dimensions required
+				// to get the requested client area
+				RECT rc;
+				SetRect(&rc, 0, 0, width, height);
+				AdjustWindowRect(&rc, dwStyle, false);
+				md3dpp.Windowed = !fullScreen;
+				md3dpp.FullScreen_RefreshRateInHz = mIsFullScreen ? mDisplayFrequency : 0;
+				unsigned int winWidth = rc.right - rc.left;
+				unsigned int winHeight = rc.bottom - rc.top;
+
+				md3dpp.BackBufferHeight = height;
+				md3dpp.BackBufferWidth = width;
+
+				SetWindowLong(mHWnd, GWL_STYLE, dwStyle);
+				SetWindowPos(mHWnd, HWND_NOTOPMOST, 0, 0, winWidth, winHeight,
+					SWP_DRAWFRAME | SWP_FRAMECHANGED | SWP_NOMOVE | SWP_NOACTIVATE);
+
+				// Note that we also set the position in the restoreLostDevice method
+			}
+
+			if ((oldFullscreen && fullScreen) || mIsExternal)
+			{
+				// Have to release & trigger device reset
+				// NB don't use windowMovedOrResized since Win32 doesn't know
+				// about the size change yet
+				SAFE_RELEASE(mpRenderSurface);
+				static_cast<D3D9RenderSystem*>(Root::getSingleton().getRenderSystem())->_notifyDeviceLost();
+				// Notify viewports of resize
+				ViewportList::iterator it = mViewportList.begin();
+				while(it != mViewportList.end()) (*it++).second->_updateDimensions();
+			}
+		}
 	}
 
 	void D3D9RenderWindow::createD3DResources(void)
@@ -321,6 +355,8 @@ namespace Ogre
 				"D3D9RenderWindow::createD3DResources");
 		}
 
+		SAFE_RELEASE(mpRenderSurface);
+
 		// Set up the presentation parameters
 		HRESULT hr;
 		LPDIRECT3D9 pD3D = mDriver->getD3D();
@@ -329,16 +365,33 @@ namespace Ogre
 		ZeroMemory( &md3dpp, sizeof(D3DPRESENT_PARAMETERS) );
 		md3dpp.Windowed					= !mIsFullScreen;
 		md3dpp.SwapEffect				= D3DSWAPEFFECT_DISCARD;
-		md3dpp.BackBufferCount			= 1;
+		// triple buffer if VSync is on
+		md3dpp.BackBufferCount			= mVSync ? 2 : 1;
 		md3dpp.EnableAutoDepthStencil	= mIsDepthBuffered;
 		md3dpp.hDeviceWindow			= mHWnd;
 		md3dpp.BackBufferWidth			= mWidth;
 		md3dpp.BackBufferHeight			= mHeight;
+		md3dpp.FullScreen_RefreshRateInHz = mIsFullScreen ? mDisplayFrequency : 0;
+		
 
 		if (mVSync)
+		{
 			md3dpp.PresentationInterval = D3DPRESENT_INTERVAL_ONE;
+		}
 		else
+		{
+			// NB not using vsync in windowed mode in D3D9 can cause jerking at low 
+			// frame rates no matter what buffering modes are used (odd - perhaps a
+			// timer issue in D3D9 since GL doesn't suffer from this) 
+			// low is < 200fps in this context
+			if (!mIsFullScreen)
+			{
+				LogManager::getSingleton().logMessage("D3D9 : WARNING - "
+					"disabling VSync in windowed mode can cause timing issues at lower "
+					"frame rates, turn VSync on if you observe this problem.");
+			}
 			md3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
+		}
 
 		md3dpp.BackBufferFormat		= D3DFMT_R5G6B5;
 		if( mColourDepth > 16 )
@@ -387,11 +440,17 @@ namespace Ogre
 			hr = mpD3DDevice->CreateAdditionalSwapChain(
 				&md3dpp, &mpSwapChain);
 			if (FAILED(hr))
+			{
+				// Try a second time, may fail the first time due to back buffer count,
+				// which will be corrected by the runtime
+				hr = mpD3DDevice->CreateAdditionalSwapChain(
+					&md3dpp, &mpSwapChain);
+			}
+			if (FAILED(hr))
 			{
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
 					"Unable to create an additional swap chain",
 					"D3D9RenderWindow::createD3DResources");
-
 			}
 			// Store references to buffers for convenience
 			mpSwapChain->GetBackBuffer( 0, D3DBACKBUFFER_TYPE_MONO, &mpRenderSurface );
@@ -420,10 +479,6 @@ namespace Ogre
 			{
 				mpRenderZBuffer = 0;
 			}
-			// release immediately so we don't hog them
-			mpRenderSurface->Release();
-			// We'll need the depth buffer for rendering the swap chain
-			//mpRenderZBuffer->Release();
 		}
 		else
 		{
@@ -438,15 +493,46 @@ namespace Ogre
 				if (opti != options.end() && opti->second.currentValue == "Consistent")
 					extraFlags |= D3DCREATE_FPU_PRESERVE;
 
-				hr = pD3D->CreateDevice( mDriver->getAdapterNumber(), devType, mHWnd,
+#if OGRE_THREAD_SUPPORT
+				extraFlags |= D3DCREATE_MULTITHREADED;
+#endif
+				// Set default settings (use the one Ogre discovered as a default)
+				UINT adapterToUse = mDriver->getAdapterNumber();
+
+				if (mUseNVPerfHUD)
+				{
+					// Look for 'NVIDIA NVPerfHUD' adapter
+					// If it is present, override default settings
+					for (UINT adapter=0; adapter < mDriver->getD3D()->GetAdapterCount(); ++adapter)
+					{
+						D3DADAPTER_IDENTIFIER9 identifier;
+						HRESULT res;
+						res = mDriver->getD3D()->GetAdapterIdentifier(adapter,0,&identifier);
+						if (strstr(identifier.Description,"NVPerfHUD") != 0)
+						{
+							adapterToUse = adapter;
+							devType = D3DDEVTYPE_REF;
+							break;
+						}
+					}
+				}
+
+				hr = pD3D->CreateDevice(adapterToUse, devType, mHWnd,
 					D3DCREATE_HARDWARE_VERTEXPROCESSING | extraFlags, &md3dpp, &mpD3DDevice );
+				if (FAILED(hr))
+				{
+					// Try a second time, may fail the first time due to back buffer count,
+					// which will be corrected down to 1 by the runtime
+					hr = pD3D->CreateDevice( adapterToUse, devType, mHWnd,
+						D3DCREATE_HARDWARE_VERTEXPROCESSING | extraFlags, &md3dpp, &mpD3DDevice );
+				}
 				if( FAILED( hr ) )
 				{
-					hr = pD3D->CreateDevice( mDriver->getAdapterNumber(), devType, mHWnd,
+					hr = pD3D->CreateDevice( adapterToUse, devType, mHWnd,
 						D3DCREATE_MIXED_VERTEXPROCESSING | extraFlags, &md3dpp, &mpD3DDevice );
 					if( FAILED( hr ) )
 					{
-						hr = pD3D->CreateDevice( mDriver->getAdapterNumber(), devType, mHWnd,
+						hr = pD3D->CreateDevice( adapterToUse, devType, mHWnd,
 							D3DCREATE_SOFTWARE_VERTEXPROCESSING | extraFlags, &md3dpp, &mpD3DDevice );
 					}
 				}
@@ -454,7 +540,8 @@ namespace Ogre
 				if( FAILED( hr ) )
 				{
 					destroy();
-					OGRE_EXCEPT( hr, "Failed to create Direct3D9 Device: " + 
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
+						"Failed to create Direct3D9 Device: " + 
 						Root::getSingleton().getErrorDescription(hr), 
 						"D3D9RenderWindow::createD3DResources" );
 				}
@@ -465,7 +552,6 @@ namespace Ogre
 			mpD3DDevice->GetRenderTarget( 0, &mpRenderSurface );
 			mpD3DDevice->GetDepthStencilSurface( &mpRenderZBuffer );
 			// release immediately so we don't hog them
-			mpRenderSurface->Release();
 			mpRenderZBuffer->Release();
 		}
 
@@ -473,7 +559,6 @@ namespace Ogre
 
 	void D3D9RenderWindow::destroyD3DResources()
 	{
-		mpRenderSurface = 0;
 		if (mIsSwapChain)
 		{
 			SAFE_RELEASE(mpRenderZBuffer);
@@ -483,18 +568,23 @@ namespace Ogre
 		{
 			// ignore depth buffer, access device through driver
 			mpRenderZBuffer = 0;
-			LPDIRECT3DDEVICE9 mpD3DDevice = mDriver->getD3DDevice();
-			SAFE_RELEASE(mpD3DDevice);
-			mDriver->setD3DDevice(NULL);
 		}
+		SAFE_RELEASE(mpRenderSurface);
 	}
 
 	void D3D9RenderWindow::destroy()
 	{
+		destroyD3DResources();
+
 		if (mHWnd && !mIsExternal)
+		{
+			WindowEventUtilities::_removeRenderWindow(this);
 			DestroyWindow(mHWnd);
+		}
+
 		mHWnd = 0;
 		mActive = false;
+		mClosed = true;
 	}
 
 	bool D3D9RenderWindow::isVisible() const
@@ -541,6 +631,8 @@ namespace Ogre
 		if (mWidth == width && mHeight == height)
 			return;
 
+		SAFE_RELEASE( mpRenderSurface );
+
 		if (mIsSwapChain) 
 		{
 
@@ -569,7 +661,7 @@ namespace Ogre
 					&mpSwapChain);
 
 				if (FAILED(hr))
-					OGRE_EXCEPT( hr, "Reset window to last size failed", "D3D9RenderWindow::resize" );
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Reset window to last size failed", "D3D9RenderWindow::resize" );
 
 			}		
 			else 
@@ -591,10 +683,9 @@ namespace Ogre
 
 				if (FAILED(hr)) 
 				{
-					OGRE_EXCEPT( hr, "Failed to create depth stencil surface for Swap Chain", "D3D9RenderWindow::resize" );
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to create depth stencil surface for Swap Chain", "D3D9RenderWindow::resize" );
 				}
 
-				mpRenderSurface->Release();
 			}
 		}
 		// primary windows must reset the device
@@ -629,11 +720,13 @@ namespace Ogre
 			}
 			if( D3DERR_DEVICELOST == hr )
 			{
+				SAFE_RELEASE(mpRenderSurface);
+
 				static_cast<D3D9RenderSystem*>(
 					Root::getSingleton().getRenderSystem())->_notifyDeviceLost();
 			}
 			else if( FAILED(hr) )
-				OGRE_EXCEPT( hr, "Error Presenting surfaces", "D3D9RenderWindow::swapBuffers" );
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error Presenting surfaces", "D3D9RenderWindow::swapBuffers" );
 		}
 	}
 
@@ -641,7 +734,7 @@ namespace Ogre
 	{
 		// Valid attributes and their equvalent native functions:
 		// D3DDEVICE			: getD3DDevice
-		// HWND					: getWindowHandle
+		// WINDOW				: getWindowHandle
 
 		if( name == "D3DDEVICE" )
 		{
@@ -649,7 +742,7 @@ namespace Ogre
 			*pDev = getD3DDevice();
 			return;
 		}
-		else if( name == "HWND" )
+		else if( name == "WINDOW" )
 		{
 			HWND *pHwnd = (HWND*)pData;
 			*pHwnd = getWindowHandle();
@@ -695,7 +788,7 @@ namespace Ogre
 		// get display dimensions
 		// this will be the dimensions of the front buffer
 		if (FAILED(hr = mpD3DDevice->GetDisplayMode(0, &dm)))
-			OGRE_EXCEPT(hr, "Can't get display mode!", "D3D9RenderWindow::writeContentsToFile");
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get display mode!", "D3D9RenderWindow::writeContentsToFile");
 
 		desc.Width = dm.Width;
 		desc.Height = dm.Height;
@@ -708,13 +801,13 @@ namespace Ogre
 			&pTempSurf, 
 			NULL)))
 		{
-			OGRE_EXCEPT(hr, "Cannot create offscreen buffer 1!", "D3D9RenderWindow::writeContentsToFile");
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Cannot create offscreen buffer 1!", "D3D9RenderWindow::writeContentsToFile");
 		}
 
 		if (FAILED(hr = mpD3DDevice->GetFrontBufferData(0, pTempSurf)))
 		{
 			SAFE_RELEASE(pTempSurf);
-			OGRE_EXCEPT(hr, "Can't get front buffer!", "D3D9RenderWindow::writeContentsToFile");
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get front buffer!", "D3D9RenderWindow::writeContentsToFile");
 		}
 		
 		D3DLOCKED_RECT lockedRect;
@@ -723,13 +816,21 @@ namespace Ogre
 			if (FAILED(hr = pTempSurf->LockRect(&lockedRect, NULL, 
 			D3DLOCK_READONLY | D3DLOCK_NOSYSLOCK)))
 			{
-				OGRE_EXCEPT(hr, "can't lock rect!", "D3D9RenderWindow::writeContentsToFile");
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "can't lock rect!", "D3D9RenderWindow::writeContentsToFile");
 			} 
 		}
 		else
 		{
 			RECT srcRect;
-			GetWindowRect(mHWnd, &srcRect);
+			GetClientRect(mHWnd, &srcRect);
+			POINT point;
+			point.x = srcRect.left;
+			point.y = srcRect.top;
+			ClientToScreen(mHWnd, &point);
+			srcRect.top = point.y;
+			srcRect.left = point.x;
+			srcRect.bottom += point.y;
+			srcRect.right += point.x;
 
 			desc.Width = srcRect.right - srcRect.left;
 			desc.Height = srcRect.bottom - srcRect.top;
@@ -738,7 +839,7 @@ namespace Ogre
 
 			D3DLOCK_READONLY | D3DLOCK_NOSYSLOCK)))
 			{
-				OGRE_EXCEPT(hr, "can't lock rect!", "D3D9RenderWindow::writeContentsToFile");
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "can't lock rect!", "D3D9RenderWindow::writeContentsToFile");
 			} 
 		}
   
@@ -826,8 +927,9 @@ namespace Ogre
 		SAFE_RELEASE(pSurf);
 	}
 	//-----------------------------------------------------------------------------
-	void D3D9RenderWindow::update(void)
+	void D3D9RenderWindow::update(bool swap)
 	{
+
 		D3D9RenderSystem* rs = static_cast<D3D9RenderSystem*>(
 			Root::getSingleton().getRenderSystem());
 
@@ -844,7 +946,7 @@ namespace Ogre
 				// can't do anything about it here, wait until we get 
 				// D3DERR_DEVICENOTRESET; rendering calls will silently fail until 
 				// then (except Present, but we ignore device lost there too)
-				mpRenderSurface = 0;
+				SAFE_RELEASE(mpRenderSurface);
 				// need to release if swap chain
 				if (!mIsSwapChain)
 					mpRenderZBuffer = 0;
@@ -866,47 +968,27 @@ namespace Ogre
 					return;
 				}
 
-				// fixed: this only works for not swap chained
 				if (!mIsSwapChain) 
 				{
 					// re-qeuery buffers
 					mpD3DDevice->GetRenderTarget( 0, &mpRenderSurface );
 					mpD3DDevice->GetDepthStencilSurface( &mpRenderZBuffer );
 					// release immediately so we don't hog them
-					mpRenderSurface->Release();
 					mpRenderZBuffer->Release();
 				}
 				else 
 				{
-					mpSwapChain->GetBackBuffer( 0, D3DBACKBUFFER_TYPE_MONO, &mpRenderSurface );
-					if (mIsDepthBuffered) {
-						hr = mpD3DDevice->CreateDepthStencilSurface(
-							mWidth, mHeight,
-							md3dpp.AutoDepthStencilFormat,
-							md3dpp.MultiSampleType,
-							md3dpp.MultiSampleQuality, 
-							(md3dpp.Flags & D3DPRESENTFLAG_DISCARD_DEPTHSTENCIL),
-							&mpRenderZBuffer, NULL
-							);
-
-						if (FAILED(hr)) {
-							OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
-								"Unable to re-create depth buffer for the swap chain",
-								"D3D9RenderWindow::update");
-
-						}
-					} 
-					else
-					{
-						mpRenderZBuffer = 0;
-					}
-
-					// release immediately so we don't hog them
-					mpRenderSurface->Release();
+				    // Update dimensions incase changed
+		            ViewportList::iterator it = mViewportList.begin();
+		            while( it != mViewportList.end() )
+			            (*it++).second->_updateDimensions();
+					// Actual restoration of surfaces will happen in 
+					// D3D9RenderSystem::restoreLostDevice when it calls
+					// createD3DResources for each secondary window
 				}
 			}
 
 		}
-		RenderWindow::update();
+		RenderWindow::update(swap);
 	}
 }
