@@ -2,9 +2,9 @@
 -----------------------------------------------------------------------------
 This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
-For the latest info, see http://ogre.sourceforge.net/
+For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 #ifndef __GLRenderSystem_H__
@@ -30,6 +34,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreRenderSystem.h"
 #include "OgreGLHardwareBufferManager.h"
 #include "OgreGLGpuProgramManager.h"
+#include "OgreGLSLProgramFactory.h"
 #include "OgreVector4.h"
 
 
@@ -37,7 +42,7 @@ namespace Ogre {
     /**
       Implementation of GL as a rendering system.
      */
-    class GLRenderSystem : public RenderSystem
+    class _OgrePrivate GLRenderSystem : public RenderSystem
     {
     private:
         // Rendering loop control
@@ -80,11 +85,14 @@ namespace Ogre {
         void makeGLMatrix(GLfloat gl_matrix[16], const Matrix4& m);
  
         GLint getBlendMode(SceneBlendFactor ogreBlend) const;
+		GLint getTextureAddressingMode(TextureUnitState::TextureAddressingMode tam) const;
 
         void setLights();
 
         // Store last depth write state
         bool mDepthWrite;
+		// Store last stencil mask state
+		uint32 mStencilMask;
 		// Store last colour write state
 		bool mColourWrite[4];
 
@@ -105,11 +113,12 @@ namespace Ogre {
 
         // check if the GL system has already been initialized
         bool mGLInitialized;
-        // Initialise GL context
-        void initGL(void);
+        // Initialise GL system and capabilities
+        void initGL(RenderTarget *primary);
 
         HardwareBufferManager* mHardwareBufferManager;
         GLGpuProgramManager* mGpuProgramManager;
+		GLSLProgramFactory* mGLSLProgramFactory;
 
         unsigned short mCurrentLights;
 
@@ -118,15 +127,20 @@ namespace Ogre {
         GLGpuProgram* mCurrentVertexProgram;
         GLGpuProgram* mCurrentFragmentProgram;
 
-        /* The main GL context */
+		/* The main GL context - main thread only */
         GLContext *mMainContext;
-        /* The current GL context */
+        /* The current GL context  - main thread only*/
         GLContext *mCurrentContext;
-        /* Type that maps render targets to contexts */
-        typedef std::map<RenderTarget*,GLContext*> ContextMap;
-        /* Map of render target -> context mappings. This is used to find the
-         * GL context for a certain render target */
-        ContextMap mContextMap;
+		typedef std::list<GLContext*> GLContextList;
+		/// List of background thread contexts
+		GLContextList mBackgroundContextList;
+
+        /** Manager object for creating render textures.
+            Direct render to texture via GL_EXT_framebuffer_object is preferable 
+			to pbuffers, which depend on the GL support used and are generally 
+			unwieldy and slow. However, FBO support for stencil buffers is poor.
+        */
+        GLRTTManager *mRTTManager;
     public:
         // Default constructor / destructor
         GLRenderSystem();
@@ -181,10 +195,8 @@ namespace Ogre {
 		RenderWindow* createRenderWindow(const String &name, unsigned int width, unsigned int height, 
 			bool fullScreen, const NameValuePairList *miscParams = 0);
 
-		/// @copydoc RenderSystem::createRenderTexture
-		RenderTexture * createRenderTexture( const String & name, unsigned int width, unsigned int height,
-		 	TextureType texType = TEX_TYPE_2D, PixelFormat internalFormat = PF_X8R8G8B8, 
-			const NameValuePairList *miscParams = 0 ); 
+		/// @copydoc RenderSystem::createMultiRenderTarget
+		virtual MultiRenderTarget * createMultiRenderTarget(const String & name); 
 		
         /** See
           RenderSystem
@@ -198,7 +210,7 @@ namespace Ogre {
         /** See
           RenderSystem
          */
-        void convertColourValue(const ColourValue& colour, uint32* pDest);
+        VertexElementType getColourVertexElementType(void) const;
         /** See
           RenderSystem
          */
@@ -233,7 +245,16 @@ namespace Ogre {
         /** See
           RenderSystem
          */
-        void _setTexture(size_t unit, bool enabled, const String &texname);
+		void _setPointParameters(Real size, bool attenuationEnabled, 
+			Real constant, Real linear, Real quadratic, Real minSize, Real maxSize);
+        /** See
+          RenderSystem
+         */
+		void _setPointSpritesEnabled(bool enabled);
+		/** See
+          RenderSystem
+         */
+        void _setTexture(size_t unit, bool enabled, const TexturePtr &tex);
         /** See
           RenderSystem
          */
@@ -250,7 +271,15 @@ namespace Ogre {
         /** See
           RenderSystem
          */
-        void _setTextureAddressingMode(size_t stage, TextureUnitState::TextureAddressingMode tam);
+        void _setTextureAddressingMode(size_t stage, const TextureUnitState::UVWAddressingMode& uvw);
+        /** See
+          RenderSystem
+         */
+        void _setTextureBorderColour(size_t stage, const ColourValue& colour);
+		/** See
+		RenderSystem
+		*/
+		void _setTextureMipmapBias(size_t unit, float bias);
         /** See
           RenderSystem
          */
@@ -298,7 +327,7 @@ namespace Ogre {
         /** See
           RenderSystem
          */
-        void _setDepthBias(ushort bias);
+        void _setDepthBias(float constantBias, float slopeScaleBias);
         /** See
           RenderSystem
          */
@@ -307,6 +336,11 @@ namespace Ogre {
           RenderSystem
          */
         void _setFog(FogMode mode, const ColourValue& colour, Real density, Real start, Real end);
+        /** See
+          RenderSystem
+         */
+        void _convertProjectionMatrix(const Matrix4& matrix,
+            Matrix4& dest, bool forGpuProgram = false);
         /** See
           RenderSystem
          */
@@ -338,7 +372,7 @@ namespace Ogre {
         /** See
           RenderSystem
          */
-        void _setRasterisationMode(SceneDetailLevel level);
+        void _setPolygonMode(PolygonMode level);
         /** See
           RenderSystem
          */
@@ -383,6 +417,10 @@ namespace Ogre {
           RenderSystem
          */
         void bindGpuProgramParameters(GpuProgramType gptype, GpuProgramParametersSharedPtr params);
+		/** See
+		RenderSystem
+		*/
+		void bindGpuProgramPassIterationParameters(GpuProgramType gptype);
         /** See
           RenderSystem
          */
@@ -399,13 +437,16 @@ namespace Ogre {
         Real getVerticalTexelOffset(void);
         Real getMinimumDepthInputValue(void);
         Real getMaximumDepthInputValue(void);
+		void registerThread();
+		void unregisterThread();
+		void preExtraThreadsStarted();
+		void postExtraThreadsStarted();
 
         // ----------------------------------
         // GLRenderSystem specific members
         // ----------------------------------
-        /**
-         * One time initialization for the RenderState of a context. Things that
-         * only need to be set once, like the LightingModel can be defined here.
+        /** One time initialization for the RenderState of a context. Things that
+            only need to be set once, like the LightingModel can be defined here.
          */
         void _oneTimeContextInitialization();
         /** Switch GL context, dealing with involved internal cached states too
@@ -415,21 +456,16 @@ namespace Ogre {
          * Set current render target to target, enabling its GL context if needed
          */
         void _setRenderTarget(RenderTarget *target);
-        /**
-         * Register a render target->context mapping.
+        /** Unregister a render target->context mapping. If the context of target 
+            is the current context, change the context to the main context so it
+            can be destroyed safely. 
+            
+            @note This is automatically called by the destructor of 
+            GLContext.
          */
-        void _registerContext(RenderTarget *target, GLContext *context);
-        /**
-         * Unregister a render target->context mapping. If the context of target 
-         * is the current context, change the context to the main context so it
-         * can be destroyed safely.
-         */
-        void _unregisterContext(RenderTarget *target);
-        /**
-         * Get the main context. This is generally the context with which 
-         * a new context wants to share buffers and textures.
-         */
-        GLContext *_getMainContext();
+        void _unregisterContext(GLContext *context);
+		/** Returns the main context */
+		GLContext* _getMainContext() {return mMainContext;} 
     };
 }
 #endif
