@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 
@@ -31,6 +35,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreLogManager.h"
 #include "OgreStringConverter.h"
 #include "OgreGLXUtils.h"
+#include "OgreWindowEventUtilities.h"
 
 #include <iostream>
 #include <algorithm>
@@ -49,49 +54,61 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include <GL/glx.h>
 
 
-namespace Ogre {
+namespace Ogre
+{
 
+//-------------------------------------------------------------------------------------------------//
 GLXWindow::GLXWindow(Display *display) :
-		mDisplay(display),
-		mWindow(0),
-		mGlxContext(0),
-        mClosed(false),mFullScreen(false), mOldMode(-1),
-        mContext(0) {
+	mDisplay(display), mWindow(0), mGlxContext(0), mVisualInfo(0), mDelVisualInfo(false), 
+	mDelWindow(false), mClosed(false), mVisible(true), mFullScreen(false), mOldMode(-1),
+	mContext(0)
+{
 	mActive = false;
 }
 
-GLXWindow::~GLXWindow() {
+//-------------------------------------------------------------------------------------------------//
+GLXWindow::~GLXWindow() 
+{
+	if (mDelVisualInfo && mVisualInfo)
+		XFree(mVisualInfo);
+
 	if(mGlxContext)
 		glXDestroyContext(mDisplay, mGlxContext);
-	if(mWindow)
+	
+	if(mDelWindow && mWindow)
 		XDestroyWindow(mDisplay, mWindow);
-#ifndef NO_XRANDR
 
-	if(mFullScreen) {
+#ifndef NO_XRANDR
+	if(mFullScreen) 
+	{
 		// Restore original video mode.
 		Window rootWindow = DefaultRootWindow(mDisplay);
 		XRRScreenConfiguration *config;
 
 		// Get current screen info
 		config = XRRGetScreenInfo(mDisplay, rootWindow);
-		if(config) {
+		if(config) 
+		{
 			Rotation current_rotation;
 			XRRConfigCurrentConfiguration (config, &current_rotation);
 			//std::cerr << "Restore mode " << mOldMode << std::endl;
 			LogManager::getSingleton().logMessage("GLXWindow::~GLXWindow -- Leaving full screen mode");
 			XRRSetScreenConfig(mDisplay, config, rootWindow, mOldMode, current_rotation, CurrentTime);
 			XRRFreeScreenConfigInfo(config);
-		} else {
+		} 
+		else 
+		{
 			LogManager::getSingleton().logMessage("GLXWindow::~GLXWindow -- Could not switch from full screen mode: XRRGetScreenInfo failed");
 		}
 	}
 #endif
 }
 
+//-------------------------------------------------------------------------------------------------//
 void GLXWindow::create(const String& name, unsigned int width, unsigned int height,
 	            bool fullScreen, const NameValuePairList *miscParams)
 {
-	std::cerr << "GLXWindow::create" << std::endl;
+	LogManager::getSingleton().logMessage("GLXWindow::create");
 
 	String title = name;
 	size_t fsaa_samples = 0;
@@ -113,7 +130,7 @@ void GLXWindow::create(const String& name, unsigned int width, unsigned int heig
         // Unless parentWindowHandle is given in miscParams we're top-level
 	mTopLevel = true;
    
-	std::cerr << "Parsing miscParams" << std::endl;
+	LogManager::getSingleton().logMessage("Parsing miscParams");
 	if(miscParams)
 	{
 		// Parse miscellenous parameters
@@ -152,7 +169,6 @@ void GLXWindow::create(const String& name, unsigned int width, unsigned int heig
 
 			left = top = 0;
 			fullScreen = false; // Can't be full screen if embedded in an app!
-            mTopLevel = false;  // Can't be top-level if embedded
 		}
 		
 		opt = miscParams->find("externalWindowHandle");
@@ -177,7 +193,7 @@ void GLXWindow::create(const String& name, unsigned int width, unsigned int heig
 			
 			left = top = 0;
 			fullScreen = false; // Can't be full screen if embedded in an app!
-            mTopLevel = false;  // Can't be top-level if embedded         
+			mTopLevel = false;  // Can't be top-level if embedded         
 		}
 
 	}
@@ -188,15 +204,18 @@ void GLXWindow::create(const String& name, unsigned int width, unsigned int heig
 		LogManager::getSingleton().logMessage("GLXWindow::create -- FSAA only supported in fullscreen mode");
 		fsaa_samples = 0;
 	}
-    // Disable FSAA for now -- it doesn't work on NVIDIA
-    fsaa_samples = 0;
+	// Disable FSAA for now -- it doesn't work on NVIDIA
+	fsaa_samples = 0;
 
 #ifndef NO_XRANDR
 	// Attempt mode switch for fullscreen -- only if RANDR extension is there
 	int dummy;
-	if(fullScreen && ! XQueryExtension(mDisplay, "RANDR", &dummy, &dummy, &dummy)) {
-			LogManager::getSingleton().logMessage("GLXWindow::create -- Could not switch to full screen mode: No XRANDR extension found");
-	} else if(fullScreen) {
+	if(fullScreen && ! XQueryExtension(mDisplay, "RANDR", &dummy, &dummy, &dummy)) 
+	{
+		LogManager::getSingleton().logMessage("GLXWindow::create -- Could not switch to full screen mode: No XRANDR extension found");
+	} 
+	else if(fullScreen) 
+	{
 		// Use Xrandr extension to switch video modes. This is much better than
 		// XVidMode as you can't scroll away from the full-screen applications.
 		XRRScreenConfiguration *config;
@@ -217,9 +236,13 @@ void GLXWindow::create(const String& name, unsigned int width, unsigned int heig
 			int mode = -1;
 			int mode_width = INT_MAX;
 			int mode_height = INT_MAX;
-			for(size_t i=0; i<nsizes; i++) {
-				if(sizes[i].width >= width && sizes[i].height >= height &&
-				                sizes[i].width < mode_width && sizes[i].height < mode_height) {
+			for(size_t i=0; i<nsizes; i++) 
+			{
+				if(sizes[i].width >= static_cast<int>(width) && 
+					sizes[i].height >= static_cast<int>(height) &&
+				    sizes[i].width < mode_width && 
+					sizes[i].height < mode_height) 
+				{
 					mode = i;
 					mode_width = sizes[i].width;
 					mode_height = sizes[i].height;
@@ -240,7 +263,6 @@ void GLXWindow::create(const String& name, unsigned int width, unsigned int heig
 	}
 #endif
 
-	XVisualInfo* visualInfo = NULL;
 	if(extVisualHandler == NULL) // user didn't create visual ( and window ) himself 
 	{
 		// Apply some magic algorithm to get the best visual
@@ -257,17 +279,18 @@ void GLXWindow::create(const String& name, unsigned int width, unsigned int heig
 		XVisualInfo templ;
 		int nmatch;
 		templ.visualid = best_visual;
-		visualInfo = XGetVisualInfo(mDisplay, VisualIDMask, &templ, &nmatch);
-		if(visualInfo==0 || nmatch==0) {
-			OGRE_EXCEPT(999, "GLXWindow: error choosing visual", "GLXWindow::create");
+		mVisualInfo = XGetVisualInfo(mDisplay, VisualIDMask, &templ, &nmatch);
+		mDelVisualInfo = true;
+		if(mVisualInfo==0 || nmatch==0) {
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "GLXWindow: error choosing visual", "GLXWindow::create");
 		}
 
 		XSetWindowAttributes attr;
 		unsigned long mask;
 		attr.background_pixel = 0;
 		attr.border_pixel = 0;
-		attr.colormap = XCreateColormap(mDisplay,rootWindow,visualInfo->visual,AllocNone);
-		attr.event_mask = StructureNotifyMask | KeyPressMask | KeyReleaseMask | PointerMotionMask | ButtonPressMask | ButtonReleaseMask;
+		attr.colormap = XCreateColormap(mDisplay,rootWindow,mVisualInfo->visual,AllocNone);
+		attr.event_mask = StructureNotifyMask | VisibilityChangeMask;
 		if(fullScreen) {
 			mask = CWBackPixel | CWColormap | CWOverrideRedirect | CWSaveUnder | CWBackingStore | CWEventMask;
 			attr.override_redirect = True;
@@ -279,9 +302,10 @@ void GLXWindow::create(const String& name, unsigned int width, unsigned int heig
 			mask = CWBackPixel | CWBorderPixel | CWColormap | CWEventMask;
 	
 		// Create window on server
-		mWindow = XCreateWindow(mDisplay,parentWindow,left,top,width,height,0,visualInfo->depth,InputOutput,visualInfo->visual,mask,&attr);
+		mWindow = XCreateWindow(mDisplay,parentWindow,left,top,width,height,0,mVisualInfo->depth,InputOutput,mVisualInfo->visual,mask,&attr);
+		mDelWindow = true;
 		if(!mWindow) {
-			OGRE_EXCEPT(999, "GLXWindow: XCreateWindow failed", "GLXWindow::create");
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "GLXWindow: XCreateWindow failed", "GLXWindow::create");
 		}
 	
 		// Make sure the window is in normal state
@@ -328,11 +352,15 @@ void GLXWindow::create(const String& name, unsigned int width, unsigned int heig
 	
 		// Make sure the server is up to date and focus the window
 		XFlush(mDisplay);
+
+		//Register only Ogre created windows (users can register their own)
+		WindowEventUtilities::_addRenderWindow(this);
 	}
 	else
 	{
 		LogManager::getSingleton().logMessage("GLXWindow::create -- using external window handle");
-		visualInfo = extVisualHandler;
+		mVisualInfo = extVisualHandler;
+		mDelVisualInfo = false;
 	}
 
 	GLRenderSystem *rs = static_cast<GLRenderSystem*>(Root::getSingleton().getRenderSystem());
@@ -341,204 +369,201 @@ void GLXWindow::create(const String& name, unsigned int width, unsigned int heig
 	{
 		// Finally, create a GL context
 		// we want to share it with main
-		mGlxContext = glXCreateContext(mDisplay,visualInfo,NULL,True);
+		mGlxContext = glXCreateContext(mDisplay,mVisualInfo,NULL,True);
 	}
 		else
-			mGlxContext = glXCreateContext(mDisplay,visualInfo,mainContext->mCtx,True);
+			mGlxContext = glXCreateContext(mDisplay,mVisualInfo,mainContext->mCtx,True);
 	
-	if(!mGlxContext) {
-		OGRE_EXCEPT(999, "glXCreateContext failed", "GLXWindow::create");
-	}
-
-	// Free visual info
-	if (extVisualHandler == NULL)
-		XFree(visualInfo);
+	if(!mGlxContext)
+		OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "glXCreateContext failed", "GLXWindow::create");
 
 	mName = name;
 	mWidth = width;
 	mHeight = height;
 	mFullScreen = fullScreen;
 
-    // Create OGRE GL context
-    mContext = new GLXContext(mDisplay, mWindow, mGlxContext);
-    // Register the context with the rendersystem and associate it with this window
-    rs->_registerContext(this, mContext);
-	
-	if ( rs->_getMainContext() == 0 )
-		glXMakeCurrent(mDisplay,mWindow,mGlxContext);
+	//Get the attributes of the screen
+	XWindowAttributes temp;
+	XGetWindowAttributes(mDisplay, mWindow, &temp);
+	mLeft = temp.x;
+	mTop = temp.y;
+
+	// Create OGRE GL context
+	mContext = new GLXContext(mDisplay, mWindow, mGlxContext, mVisualInfo);
 }
 
+//-------------------------------------------------------------------------------------------------//
 void GLXWindow::destroy(void)
 {
-    // Unregister and destroy OGRE GLContext
-    GLRenderSystem *rs = static_cast<GLRenderSystem*>(Root::getSingleton().getRenderSystem());
-    rs->_unregisterContext(this);
-    delete mContext;
+	WindowEventUtilities::_removeRenderWindow(this);
 
-    // Destroy GL context
+	// Unregister and destroy OGRE GLContext
+	delete mContext;
+
+	// Destroy GL context
 	if(mGlxContext)
 		glXDestroyContext(mDisplay, mGlxContext);
-	if(mWindow)
+
+	if(mDelWindow && mWindow)
 		XDestroyWindow(mDisplay, mWindow);
+
+	mContext = 0;
 	mWindow = 0;
 	mGlxContext = 0;
 	mActive = false;
+	mVisible = false;
+	mClosed = true;
 
 	Root::getSingleton().getRenderSystem()->detachRenderTarget( this->getName() );
 }
 
+//-------------------------------------------------------------------------------------------------//
 bool GLXWindow::isActive() const
 {
 	return mActive;
 }
 
+//-------------------------------------------------------------------------------------------------//
 bool GLXWindow::isClosed() const
 {
 	return mClosed;
 }
 
+//-------------------------------------------------------------------------------------------------//
+bool GLXWindow::isVisible() const
+{
+	return mVisible;
+}
+
+//-------------------------------------------------------------------------------------------------//
+void GLXWindow::setVisible(bool visible)
+{
+	mVisible = visible;
+}
+
+//-------------------------------------------------------------------------------------------------//
 void GLXWindow::reposition(int left, int top)
 {
 	XMoveWindow(mDisplay,mWindow,left,top);
 }
 
+//-------------------------------------------------------------------------------------------------//
 void GLXWindow::resize(unsigned int width, unsigned int height)
 {
-    if (!mTopLevel) 
-        /// Embedded
-        resized(width, height);
-    else
-        /// Ogre handles window
-        XResizeWindow(mDisplay,mWindow,width,height);
+	// Check if the window size really changed
+	if(mWidth == width && mHeight == height)
+		return;
+
+	mWidth = width;
+	mHeight = height;
+
+	if (!mTopLevel)
+	{
+		for (ViewportList::iterator it = mViewportList.begin();	it != mViewportList.end(); ++it)
+			(*it).second->_updateDimensions();
+	}
+	else
+	{
+		XResizeWindow(mDisplay, mWindow, width, height); /// Ogre handles window
+	}
 }
 
+//-------------------------------------------------------------------------------------------------//
+void GLXWindow::windowMovedOrResized()
+{
+	//Get the new attributes of the screen
+	XWindowAttributes temp;
+	XGetWindowAttributes(mDisplay, mWindow, &temp);
+	mLeft = temp.x;
+	mTop  = temp.y;
+
+	//Only update viewport dimensions if they did actually change
+	if (static_cast<int>(mWidth) == temp.width && static_cast<int>(mHeight) == temp.height)
+		return;
+
+	mWidth = temp.width;
+	mHeight = temp.height;
+
+	// Notify viewports of resize
+	ViewportList::iterator it, itend;
+	itend = mViewportList.end();
+	for( it = mViewportList.begin(); it != itend; ++it )
+		(*it).second->_updateDimensions();
+}
+
+//-------------------------------------------------------------------------------------------------//
 void GLXWindow::swapBuffers(bool waitForVSync)
 {
 	glXSwapBuffers(mDisplay,mWindow);
 }
 
-void GLXWindow::processEvent(const XEvent &event)
-{
-	// Process only events for this window
-	switch(event.type) {
-	case ClientMessage:
-		if(event.xclient.display != mDisplay || event.xclient.window != mWindow)
-			// Not for me
-			break;
-		if(event.xclient.format == 32 && event.xclient.data.l[0] == (long)mAtomDeleteWindow)  {
-			// Window deleted -- oops, this does not work, ogre doesn't register the close
-			//mClosed = true;
-			//mActive = false;
-			//Root::getSingleton().getRenderSystem()->detachRenderTarget( this->getName() );
-		}
-		break;
-	case ConfigureNotify:
-		if(event.xconfigure.display != mDisplay || event.xconfigure.window != mWindow)
-			// Not for me
-			break;
-        resized(event.xconfigure.width,	event.xconfigure.height);
-		break;
-	case MapNotify:
-		if(event.xconfigure.display != mDisplay || event.xconfigure.window != mWindow)
-			// Not for me
-			break;
-		// Window was mapped to the screen
-		exposed(true);
-		break;
-	case UnmapNotify:
-		if(event.xconfigure.display != mDisplay || event.xconfigure.window != mWindow)
-			// Not for me
-			break;
-		// Window was unmapped from the screen (user switched
-		// to another workspace, for example)
-        exposed(false);
-		break;
-	}
-}
-
-void GLXWindow::exposed(bool active)
-{
-    mActive = active;
-}
-void GLXWindow::resized(size_t width, size_t height)
-{
-    // Check if the window size really changed
-    if(mWidth == width && mHeight == height)
-        return;
-    mWidth = width;
-    mHeight = height;
-
-    for (ViewportList::iterator it = mViewportList.begin();
-                    it != mViewportList.end(); ++it) {
-        (*it).second->_updateDimensions();
-    }
-}
-/// Pure virtual destructor must be defined
-GLXWindowInterface::~GLXWindowInterface()
-{
-}
-
+//-------------------------------------------------------------------------------------------------//
 void GLXWindow::getCustomAttribute( const String& name, void* pData )
 {
-	if( name == "GLXWINDOW" ) {
-		*static_cast<Window*>(pData) = mWindow;
-		return;
-	} else if( name == "GLXDISPLAY" ) {
+	if( name == "DISPLAY" ) 
+	{
 		*static_cast<Display**>(pData) = mDisplay;
 		return;
-	} else if( name == "GLXWINDOWINTERFACE" ) {
-		*static_cast<GLXWindowInterface**>(pData) = this;
-		return;
 	}
-	RenderWindow::getCustomAttribute(name, pData);
+	else if( name == "ATOM" ) 
+	{
+		*static_cast< ::Atom* >(pData) = mAtomDeleteWindow;
+		return;
+	} 
+	else if( name == "WINDOW" ) 
+	{
+		*static_cast<Window*>(pData) = mWindow;
+		return;
+	} 
+	else if( name == "GLCONTEXT" ) 
+	{
+		*static_cast<GLXContext**>(pData) = mContext;
+		return;
+	} 
 }
 
-
-
+//-------------------------------------------------------------------------------------------------//
 void GLXWindow::writeContentsToFile(const String& filename)
 {
-        ImageCodec::ImageData* imgData = new ImageCodec::ImageData;
-        imgData->width = mWidth;
-       imgData->height = mHeight;
-     imgData->format = PF_BYTE_RGB;
+	ImageCodec::ImageData* imgData = new ImageCodec::ImageData;
+	imgData->width = mWidth;
+	imgData->height = mHeight;
+	imgData->format = PF_BYTE_RGB;
 
-      // Allocate buffer
-        uchar* pBuffer = new uchar[mWidth * mHeight * 3];
+	// Allocate buffer
+	uchar* pBuffer = new uchar[mWidth * mHeight * 3];
 
-     // Read pixels
-     // I love GL: it does all the locking & colour conversion for us
-       glReadPixels(0,0, mWidth-1, mHeight-1, GL_RGB, GL_UNSIGNED_BYTE, pBuffer);
+	// Read pixels
+	// I love GL: it does all the locking & colour conversion for us
+	glReadPixels(0,0, mWidth, mHeight, GL_RGB, GL_UNSIGNED_BYTE, pBuffer);
 
-        // Wrap buffer in a memory stream
-        DataStreamPtr stream(new MemoryDataStream(pBuffer, mWidth * mHeight * 3, false));
+	// Wrap buffer in a memory stream
+	DataStreamPtr stream(new MemoryDataStream(pBuffer, mWidth * mHeight * 3, false));
 
-       // Need to flip the read data over in Y though
-     Image img;
-     img.loadRawData(stream, mWidth, mHeight, PF_BYTE_RGB );
-      img.flipAroundX();
+	// Need to flip the read data over in Y though
+	Image img;
+	img.loadRawData(stream, mWidth, mHeight, PF_BYTE_RGB );
+	img.flipAroundX();
 
-        MemoryDataStreamPtr streamFlipped(new MemoryDataStream(img.getData(), stream->size(), false));
+	MemoryDataStreamPtr streamFlipped(new MemoryDataStream(img.getData(), stream->size(), false));
 
-        // Get codec
-      size_t pos = filename.find_last_of(".");
-       String extension;
-      if( pos == String::npos )
-          OGRE_EXCEPT(
-            Exception::ERR_INVALIDPARAMS,
-         "Unable to determine image type for '" + filename + "' - invalid extension.",
-          "SDLWindow::writeContentsToFile" );
+	// Get codec
+	size_t pos = filename.find_last_of(".");
+	String extension;
+	if( pos == String::npos )
+		OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS, "Unable to determine image type for '" 
+			+ filename + "' - invalid extension.", "SDLWindow::writeContentsToFile" );
 
-       while( pos != filename.length() - 1 )
-          extension += filename[++pos];
+	while( pos != filename.length() - 1 )
+		extension += filename[++pos];
 
-     // Get the codec
-       Codec * pCodec = Codec::getCodec(extension);
+	// Get the codec
+	Codec * pCodec = Codec::getCodec(extension);
 
-      // Write out
-       Codec::CodecDataPtr codecDataPtr(imgData);
-     pCodec->codeToFile(streamFlipped, filename, codecDataPtr);
+	// Write out
+	Codec::CodecDataPtr codecDataPtr(imgData);
+	pCodec->codeToFile(streamFlipped, filename, codecDataPtr);
 
-        delete [] pBuffer;
+	delete [] pBuffer;
 }
-
 }
