@@ -8,18 +8,18 @@
 #include "OgreWin32GLSupport.h"
 #include "OgreGLTexture.h"
 #include "OgreWin32Window.h"
-
-#define HW_RTT
-
-#ifdef HW_RTT
+#include <GL/wglext.h>
 #include "OgreWin32RenderTexture.h"
-#endif
 
 using namespace Ogre;
 
+GLenum wglewContextInit (Ogre::GLSupport *glSupport);
+
 namespace Ogre {
-	Win32GLSupport::Win32GLSupport():
-		mInitialWindow(0), mHasPixelFormatARB(0)
+	Win32GLSupport::Win32GLSupport()
+        : mInitialWindow(0)
+        , mHasPixelFormatARB(false)
+        , mHasMultisample(false)
     {
 		// immediately test WGL_ARB_pixel_format and FSAA support
 		// so we can set configuration options appropriately
@@ -50,6 +50,7 @@ namespace Ogre {
 		ConfigOption optDisplayFrequency;
 		ConfigOption optVSync;
 		ConfigOption optFSAA;
+		ConfigOption optRTTMode;
 
 		// FS setting possiblities
 		optFullScreen.name = "Full Screen";
@@ -77,11 +78,11 @@ namespace Ogre {
 
 		optColourDepth.name = "Colour Depth";
 		optColourDepth.immutable = false;
-		optColourDepth.currentValue = "";
+		optColourDepth.currentValue.clear();
 
 		optDisplayFrequency.name = "Display Frequency";
 		optDisplayFrequency.immutable = false;
-		optDisplayFrequency.currentValue = "";
+        optDisplayFrequency.currentValue.clear();
 
 		optVSync.name = "VSync";
 		optVSync.immutable = false;
@@ -96,12 +97,21 @@ namespace Ogre {
 			optFSAA.possibleValues.push_back(StringConverter::toString(*it));
 		optFSAA.currentValue = "0";
 
+		optRTTMode.name = "RTT Preferred Mode";
+		optRTTMode.possibleValues.push_back("FBO");
+		optRTTMode.possibleValues.push_back("PBuffer");
+		optRTTMode.possibleValues.push_back("Copy");
+		optRTTMode.currentValue = "FBO";
+		optRTTMode.immutable = false;
+
+
 		mOptions[optFullScreen.name] = optFullScreen;
 		mOptions[optVideoMode.name] = optVideoMode;
 		mOptions[optColourDepth.name] = optColourDepth;
 		mOptions[optDisplayFrequency.name] = optDisplayFrequency;
 		mOptions[optVSync.name] = optVSync;
 		mOptions[optFSAA.name] = optFSAA;
+		mOptions[optRTTMode.name] = optRTTMode;
 
 		refreshConfig();
 	}
@@ -112,14 +122,14 @@ namespace Ogre {
 		ConfigOptionMap::iterator moptColourDepth = mOptions.find("Colour Depth");
 		ConfigOptionMap::iterator moptDisplayFrequency = mOptions.find("Display Frequency");
 		if(optVideoMode == mOptions.end() || moptColourDepth == mOptions.end() || moptDisplayFrequency == mOptions.end())
-			OGRE_EXCEPT(999, "Can't find mOptions!", "Win32GLSupport::refreshConfig");
+			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Can't find mOptions!", "Win32GLSupport::refreshConfig");
 		ConfigOption* optColourDepth = &moptColourDepth->second;
 		ConfigOption* optDisplayFrequency = &moptDisplayFrequency->second;
 
 		const String& val = optVideoMode->second.currentValue;
 		String::size_type pos = val.find('x');
 		if (pos == String::npos)
-			OGRE_EXCEPT(999, "Invalid Video Mode provided", "Win32GLSupport::refreshConfig");
+			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Invalid Video Mode provided", "Win32GLSupport::refreshConfig");
 		DWORD width = StringConverter::parseUnsignedInt(val.substr(0, pos));
 		DWORD height = StringConverter::parseUnsignedInt(val.substr(pos+1, String::npos));
 
@@ -173,7 +183,7 @@ namespace Ogre {
 	String Win32GLSupport::validateConfig()
 	{
 		// TODO, DX9
-		return String("");
+		return StringUtil::BLANK;
 	}
 
 	RenderWindow* Win32GLSupport::createWindow(bool autoCreateWindow, GLRenderSystem* renderSystem, const String& windowTitle)
@@ -182,16 +192,16 @@ namespace Ogre {
         {
             ConfigOptionMap::iterator opt = mOptions.find("Full Screen");
             if (opt == mOptions.end())
-                OGRE_EXCEPT(999, "Can't find full screen options!", "Win32GLSupport::createWindow");
+                OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Can't find full screen options!", "Win32GLSupport::createWindow");
             bool fullscreen = (opt->second.currentValue == "Yes");
 
             opt = mOptions.find("Video Mode");
             if (opt == mOptions.end())
-                OGRE_EXCEPT(999, "Can't find video mode options!", "Win32GLSupport::createWindow");
+                OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Can't find video mode options!", "Win32GLSupport::createWindow");
             String val = opt->second.currentValue;
             String::size_type pos = val.find('x');
             if (pos == String::npos)
-                OGRE_EXCEPT(999, "Invalid Video Mode provided", "Win32GLSupport::createWindow");
+                OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Invalid Video Mode provided", "Win32GLSupport::createWindow");
 
 			unsigned int w = StringConverter::parseUnsignedInt(val.substr(0, pos));
             unsigned int h = StringConverter::parseUnsignedInt(val.substr(pos + 1));
@@ -200,21 +210,29 @@ namespace Ogre {
 			NameValuePairList winOptions;
 			opt = mOptions.find("Colour Depth");
 			if (opt == mOptions.end())
-				OGRE_EXCEPT(999, "Can't find Colour Depth options!", "Win32GLSupport::createWindow");
+				OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Can't find Colour Depth options!", "Win32GLSupport::createWindow");
 			unsigned int colourDepth =
 				StringConverter::parseUnsignedInt(opt->second.currentValue);
 			winOptions["colourDepth"] = StringConverter::toString(colourDepth);
 
 			opt = mOptions.find("VSync");
 			if (opt == mOptions.end())
-				OGRE_EXCEPT(999, "Can't find VSync options!", "Win32GLSupport::createWindow");
+				OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Can't find VSync options!", "Win32GLSupport::createWindow");
 			bool vsync = (opt->second.currentValue == "Yes");
 			winOptions["vsync"] = StringConverter::toString(vsync);
 			renderSystem->setWaitForVerticalBlank(vsync);
 
+			opt = mOptions.find("Display Frequency");
+			if (opt != mOptions.end())
+			{
+				unsigned int displayFrequency =
+					StringConverter::parseUnsignedInt(opt->second.currentValue);
+				winOptions["displayFrequency"] = StringConverter::toString(displayFrequency);
+			}
+
 			opt = mOptions.find("FSAA");
 			if (opt == mOptions.end())
-				OGRE_EXCEPT(999, "Can't find FSAA options!", "Win32GLSupport::createWindow");
+				OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Can't find FSAA options!", "Win32GLSupport::createWindow");
 			unsigned int multisample =
 				StringConverter::parseUnsignedInt(opt->second.currentValue);
 			winOptions["FSAA"] = StringConverter::toString(multisample);
@@ -233,7 +251,7 @@ namespace Ogre {
 	{
 		ConfigOptionMap::iterator opt = mOptions.find("Display Frequency");
 		if (opt == mOptions.end())
-			OGRE_EXCEPT(999, "Can't find Colour Depth options!", "Win32GLSupport::newWindow");
+			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Can't find Display Frequency options!", "Win32GLSupport::newWindow");
 		unsigned int displayFrequency = StringConverter::parseUnsignedInt(opt->second.currentValue);
 
 		Win32Window* window = new Win32Window(*this);
@@ -252,12 +270,17 @@ namespace Ogre {
 	void Win32GLSupport::stop()
 	{
 		LogManager::getSingleton().logMessage("*** Stopping Win32GL Subsystem ***");
+		mInitialWindow = 0; // Since there is no removeWindow, although there should be...
 	}
 
-	void Win32GLSupport::initialiseExtensions() {
+	void Win32GLSupport::initialiseExtensions()
+	{
 		assert(mInitialWindow);
 		// First, initialise the normal extensions
 		GLSupport::initialiseExtensions();
+		// wglew init
+		wglewContextInit(this);
+
 		// Check for W32 specific extensions probe function
 		PFNWGLGETEXTENSIONSSTRINGARBPROC _wglGetExtensionsStringARB = 
 			(PFNWGLGETEXTENSIONSSTRINGARBPROC)wglGetProcAddress("wglGetExtensionsStringARB");
@@ -278,20 +301,12 @@ namespace Ogre {
         }
 	}
 
-	void Win32GLSupport::initialiseCapabilities(RenderSystemCapabilities &caps) 
-	{
-		if( checkExtension("WGL_ARB_pbuffer") ) 
-		{
-			// If yes, add rendersystem flag RSC_HWRENDER_TO_TEXTURE	
-			caps.setCapability(RSC_HWRENDER_TO_TEXTURE);
-		}
-	}
 
 	void* Win32GLSupport::getProcAddress(const String& procname)
 	{
         	return (void*)wglGetProcAddress( procname.c_str() );
 	}
-
+/*
 	RenderTexture * Win32GLSupport::createRenderTexture( const String & name, 
 		unsigned int width, unsigned int height,
 		TextureType texType, PixelFormat internalFormat, 
@@ -308,6 +323,7 @@ namespace Ogre {
 			return new GLRenderTexture(name, width, height, texType, internalFormat, miscParams);
 	}
 
+*/
 	void Win32GLSupport::initialiseWGL()
 	{
 		// wglGetProcAddress does not work without an active OpenGL context,
@@ -323,7 +339,11 @@ namespace Ogre {
 		// take this opportunity to enumerate the valid FSAA modes.
 		
 		LPCSTR dummyText = "OgreWglDummy";
+#ifdef OGRE_STATIC_LIB
+		HINSTANCE hinst = GetModuleHandle( NULL );
+#else
 		HINSTANCE hinst = GetModuleHandle("RenderSystem_GL.dll");
+#endif
 		
 		WNDCLASS dummyClass;
 		memset(&dummyClass, 0, sizeof(WNDCLASS));
@@ -339,7 +359,7 @@ namespace Ogre {
 
 		// if a simple CreateWindow fails, then boy are we in trouble...
 		if (hwnd == NULL)
-			OGRE_EXCEPT(0, "CreateWindow() failed", "Win32GLSupport::initializeWGL");
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "CreateWindow() failed", "Win32GLSupport::initializeWGL");
 
 
 		// no chance of failure and no need to release thanks to CS_OWNDC
@@ -363,6 +383,8 @@ namespace Ogre {
 		HGLRC hrc = wglCreateContext(hdc);
 		if (hrc)
 		{
+			HGLRC oldrc = wglGetCurrentContext();
+			HDC oldhdc = wglGetCurrentDC();
 			// if wglMakeCurrent fails, wglGetProcAddress will return null
 			wglMakeCurrent(hdc, hrc);
 			
@@ -371,8 +393,6 @@ namespace Ogre {
 				wglGetProcAddress("wglGetExtensionsStringARB");
 			
 			// check for pixel format and multisampling support
-			bool hasMultisample = false;
-			
 			if (_wglGetExtensionsStringARB)
 			{
 				std::istringstream wglexts(_wglGetExtensionsStringARB(hdc));
@@ -382,42 +402,53 @@ namespace Ogre {
 					if (ext == "WGL_ARB_pixel_format")
 						mHasPixelFormatARB = true;
 					else if (ext == "WGL_ARB_multisample")
-						hasMultisample = true;
+						mHasMultisample = true;
 				}
 			}
 
-			if (mHasPixelFormatARB && hasMultisample)
+			if (mHasPixelFormatARB && mHasMultisample)
 			{
-				// enumerate all 32-bit formats w/ multisampling
-				int iattr[] = {
+				// enumerate all formats w/ multisampling
+				static const int iattr[] = {
 					WGL_DRAW_TO_WINDOW_ARB, GL_TRUE,
 					WGL_SUPPORT_OPENGL_ARB, GL_TRUE,
 					WGL_DOUBLE_BUFFER_ARB, GL_TRUE,
 					WGL_SAMPLE_BUFFERS_ARB, GL_TRUE,
 					WGL_ACCELERATION_ARB, WGL_FULL_ACCELERATION_ARB,
+                    /* We are no matter about the colour, depth and stencil buffers here
 					WGL_COLOR_BITS_ARB, 24,
 					WGL_ALPHA_BITS_ARB, 8,
 					WGL_DEPTH_BITS_ARB, 24,
 					WGL_STENCIL_BITS_ARB, 8,
+                    */
 					WGL_SAMPLES_ARB, 2,
 					0
 				};
 				int formats[256];
 				unsigned int count;
-				wglChoosePixelFormatARB(hdc, iattr, 0, 256, formats, &count);
-				
-				// determine what multisampling levels are offered
-				int query = WGL_SAMPLES_ARB, samples;
-				for (unsigned int i = 0; i < count; ++i)
-				{
-					if (wglGetPixelFormatAttribivARB(hdc, formats[i],
-													0, 1, &query, &samples))
-						mFSAALevels.push_back(samples);
-				}
-				remove_duplicates(mFSAALevels);
+                // cheating here.  wglChoosePixelFormatARB procc address needed later on
+                // when a valid GL context does not exist and glew is not initialized yet.
+                __wglewChoosePixelFormatARB =
+                    (PFNWGLCHOOSEPIXELFORMATARBPROC)wglGetProcAddress("wglChoosePixelFormatARB");
+                if (__wglewChoosePixelFormatARB(hdc, iattr, 0, 256, formats, &count))
+                {
+                    // determine what multisampling levels are offered
+                    int query = WGL_SAMPLES_ARB, samples;
+                    for (unsigned int i = 0; i < count; ++i)
+                    {
+                        PFNWGLGETPIXELFORMATATTRIBIVARBPROC _wglGetPixelFormatAttribivARB =
+                            (PFNWGLGETPIXELFORMATATTRIBIVARBPROC)
+                            wglGetProcAddress("wglGetPixelFormatAttribivARB");
+                        if (_wglGetPixelFormatAttribivARB(hdc, formats[i], 0, 1, &query, &samples))
+                        {
+                            mFSAALevels.push_back(samples);
+                        }
+                    }
+                    remove_duplicates(mFSAALevels);
+                }
 			}
 			
-			wglMakeCurrent(0, 0);
+			wglMakeCurrent(oldhdc, oldrc);
 			wglDeleteContext(hrc);
 		}
 
@@ -448,8 +479,8 @@ namespace Ogre {
 
 		if (multisample)
 		{
-			// only available at 32bpp with driver support
-			if (colourDepth < 32 || !mHasPixelFormatARB)
+			// only available with driver support
+			if (!mHasMultisample || !mHasPixelFormatARB)
 				return false;
 			
 			int iattr[] = {
@@ -458,8 +489,8 @@ namespace Ogre {
 				WGL_DOUBLE_BUFFER_ARB, GL_TRUE,
 				WGL_SAMPLE_BUFFERS_ARB, GL_TRUE,
 				WGL_ACCELERATION_ARB, WGL_FULL_ACCELERATION_ARB,
-				WGL_COLOR_BITS_ARB, 24,
-				WGL_ALPHA_BITS_ARB, 8,
+				WGL_COLOR_BITS_ARB, pfd.cColorBits,
+				WGL_ALPHA_BITS_ARB, pfd.cAlphaBits,
 				WGL_DEPTH_BITS_ARB, 24,
 				WGL_STENCIL_BITS_ARB, 8,
 				WGL_SAMPLES_ARB, multisample,
@@ -467,7 +498,11 @@ namespace Ogre {
 			};
 
 			UINT nformats;
-			wglChoosePixelFormatARB(hdc, iattr, NULL, 1, &format, &nformats);
+            assert(__wglewChoosePixelFormatARB && "failed to get proc address for ChoosePixelFormatARB");
+            // ChoosePixelFormatARB proc address was obtained when setting up a dummy GL context in initialiseWGL()
+            // since glew hasn't been initialized yet, we have to cheat and use the previously obtained address
+			if (!__wglewChoosePixelFormatARB(hdc, iattr, NULL, 1, &format, &nformats) || nformats <= 0)
+                return false;
 		}
 		else
 		{
@@ -476,4 +511,13 @@ namespace Ogre {
 
 		return (format != 0 && SetPixelFormat(hdc, format, &pfd));
 	}
+
+	bool Win32GLSupport::supportsPBuffers()
+	{
+		return __WGLEW_ARB_pbuffer != GL_FALSE;
+	}
+    GLPBuffer *Win32GLSupport::createPBuffer(PixelComponentType format, size_t width, size_t height)
+	{
+		return new Win32PBuffer(format, width, height);
+	}
 }
