@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 
@@ -32,6 +36,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreException.h"
 #include "OgreWin32GLSupport.h"
 #include "OgreWin32Context.h"
+#include "OgreWindowEventUtilities.h"
 
 namespace Ogre {
 
@@ -41,7 +46,10 @@ namespace Ogre {
 	{
 		mIsFullScreen = false;
 		mHWnd = 0;
+		mGlrc = 0;
 		mIsExternal = false;
+		mIsExternalGLControl = false;
+		mIsExternalGLContext = false;
 		mSizing = false;
 		mClosed = false;
 		mDisplayFrequency = 0;
@@ -60,7 +68,11 @@ namespace Ogre {
 		if (mHWnd)
 			destroy();
 
+#ifdef OGRE_STATIC_LIB
+		HINSTANCE hInst = GetModuleHandle( NULL );
+#else
 		HINSTANCE hInst = GetModuleHandle("RenderSystem_GL.dll");
+#endif
 
 		mHWnd = 0;
 		mName = name;
@@ -79,7 +91,7 @@ namespace Ogre {
 		String title = name;
 		bool vsync = false;
 		int fsaa = 0;
-		String border = "";
+		String border;
 		bool outerSize = false;
 
 		if(miscParams)
@@ -114,7 +126,18 @@ namespace Ogre {
 					mIsExternal = true;
 					mIsFullScreen = false;
 				}
+
+				if ((opt = miscParams->find("externalGLControl")) != end) {
+				  mIsExternalGLControl = StringConverter::parseBool(opt->second);
+				}
 			}
+			if ((opt = miscParams->find("externalGLContext")) != end)
+			{
+				mGlrc = (HGLRC)StringConverter::parseUnsignedLong(opt->second);
+				if( mGlrc )
+					mIsExternalGLContext = true;
+			}
+
 			// window border style
 			opt = miscParams->find("border");
 			if(opt != miscParams->end())
@@ -124,20 +147,15 @@ namespace Ogre {
 			if(opt != miscParams->end())
 				outerSize = StringConverter::parseBool(opt->second);
 
-			if (mIsFullScreen)
-			{
-				// only available with fullscreen
-				if ((opt = miscParams->find("displayFrequency")) != end)
-					mDisplayFrequency = StringConverter::parseUnsignedInt(opt->second);
-				if ((opt = miscParams->find("colourDepth")) != end)
-					mColourDepth = StringConverter::parseUnsignedInt(opt->second);
-			}
-			else
-			{
-				// incompatible with fullscreen
-				if ((opt = miscParams->find("parentWindowHandle")) != end)
-					parent = (HWND)StringConverter::parseUnsignedInt(opt->second);
-			}
+			// only available with fullscreen
+			if ((opt = miscParams->find("displayFrequency")) != end)
+				mDisplayFrequency = StringConverter::parseUnsignedInt(opt->second);
+			if ((opt = miscParams->find("colourDepth")) != end)
+				mColourDepth = StringConverter::parseUnsignedInt(opt->second);
+
+			// incompatible with fullscreen
+			if ((opt = miscParams->find("parentWindowHandle")) != end)
+				parent = (HWND)StringConverter::parseUnsignedInt(opt->second);
 		}
 
 		if (!mIsExternal)
@@ -156,13 +174,20 @@ namespace Ogre {
 			}
 			else
 			{
-				if (border == "none")
-					dwStyle |= WS_POPUP;
-				else if (border == "fixed")
-					dwStyle |= WS_OVERLAPPED | WS_BORDER | WS_CAPTION |
-					WS_SYSMENU | WS_MINIMIZEBOX;
+				if (parent)
+				{
+					dwStyle |= WS_CHILD;
+				}
 				else
-					dwStyle |= WS_OVERLAPPEDWINDOW;
+				{
+					if (border == "none")
+						dwStyle |= WS_POPUP;
+					else if (border == "fixed")
+						dwStyle |= WS_OVERLAPPED | WS_BORDER | WS_CAPTION |
+						WS_SYSMENU | WS_MINIMIZEBOX;
+					else
+						dwStyle |= WS_OVERLAPPEDWINDOW;
+				}
 
 				int screenw = GetSystemMetrics(SM_CXSCREEN);
 				int screenh = GetSystemMetrics(SM_CYSCREEN);
@@ -192,7 +217,7 @@ namespace Ogre {
 			}
 
 			// register class and create window
-			WNDCLASS wc = { CS_OWNDC, WndProc, 0, 0, hInst,
+			WNDCLASS wc = { CS_OWNDC, WindowEventUtilities::_WndProc, 0, 0, hInst,
 				LoadIcon(NULL, IDI_APPLICATION), LoadCursor(NULL, IDC_ARROW),
 				(HBRUSH)GetStockObject(BLACK_BRUSH), NULL, "OgreGLWindow" };
 			RegisterClass(&wc);
@@ -201,6 +226,8 @@ namespace Ogre {
 			mHWnd = CreateWindowEx(dwStyleEx, "OgreGLWindow", title.c_str(),
 				dwStyle, mLeft, mTop, outerw, outerh, parent, 0, hInst, this);
 
+			WindowEventUtilities::_addRenderWindow(this);
+
 			StringUtil::StrStreamType str;
 			str << "Created Win32Window '"
 				<< mName << "' : " << mWidth << "x" << mHeight
@@ -219,6 +246,11 @@ namespace Ogre {
 				{
 					dm.dmDisplayFrequency = mDisplayFrequency;
 					dm.dmFields |= DM_DISPLAYFREQUENCY;
+					if (ChangeDisplaySettings(&dm, CDS_FULLSCREEN | CDS_TEST) != DISP_CHANGE_SUCCESSFUL)
+					{
+						LogManager::getSingleton().logMessage(LML_NORMAL, "ChangeDisplaySettings with user display frequency failed");
+						dm.dmFields ^= DM_DISPLAYFREQUENCY;
+					}
 				}
 				if (ChangeDisplaySettings(&dm, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
 					LogManager::getSingleton().logMessage(LML_CRITICAL, "ChangeDisplaySettings failed");
@@ -240,69 +272,160 @@ namespace Ogre {
 
 		mHDC = GetDC(mHWnd);
 
-		if (!mGLSupport.selectPixelFormat(mHDC, mColourDepth, fsaa))
+		if (!mIsExternalGLControl)
 		{
-			if (fsaa == 0)
-				OGRE_EXCEPT(0, "selectPixelFormat failed", "Win32Window::create");
+			if (!mGLSupport.selectPixelFormat(mHDC, mColourDepth, fsaa))
+			{
+				if (fsaa == 0)
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "selectPixelFormat failed", "Win32Window::create");
 
-			LogManager::getSingleton().logMessage(LML_NORMAL, "FSAA level not supported, falling back");
-			if (!mGLSupport.selectPixelFormat(mHDC, mColourDepth, 0))
-				OGRE_EXCEPT(0, "selectPixelFormat failed", "Win32Window::create");
+				LogManager::getSingleton().logMessage(LML_NORMAL, "FSAA level not supported, falling back");
+				if (!mGLSupport.selectPixelFormat(mHDC, mColourDepth, 0))
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "selectPixelFormat failed", "Win32Window::create");
+			}
+		}
+		if (!mIsExternalGLContext)
+		{
+			mGlrc = wglCreateContext(mHDC);
+			if (!mGlrc)
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "wglCreateContext", "Win32Window::create");
+		}
+		if (!wglMakeCurrent(mHDC, mGlrc))
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "wglMakeCurrent", "Win32Window::create");
+
+		// Do not change vsync if the external window has the OpenGL control
+		if (!mIsExternalGLControl) {
+			// Don't use wglew as if this is the first window, we won't have initialised yet
+			PFNWGLSWAPINTERVALEXTPROC _wglSwapIntervalEXT = 
+				(PFNWGLSWAPINTERVALEXTPROC)wglGetProcAddress("wglSwapIntervalEXT");
+			if (_wglSwapIntervalEXT)
+				_wglSwapIntervalEXT(vsync? 1 : 0);
 		}
 
-		mGlrc = wglCreateContext(mHDC);
-		if (!mGlrc)
-			OGRE_EXCEPT(0, "wglCreateContext", "Win32Window::create");
-		if (!wglMakeCurrent(mHDC, mGlrc))
-			OGRE_EXCEPT(0, "wglMakeCurrent", "Win32Window::create");
-
-		wglSwapIntervalEXT(vsync? 1 : 0);
-
-        if (old_context)
+        if (old_context && old_context != mGlrc)
         {
             // Restore old context
 		    if (!wglMakeCurrent(old_hdc, old_context))
-			    OGRE_EXCEPT(0, "wglMakeCurrent() failed", "Win32Window::create");
+			    OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "wglMakeCurrent() failed", "Win32Window::create");
 
             // Share lists with old context
 		    if (!wglShareLists(old_context, mGlrc))
-			    OGRE_EXCEPT(0, "wglShareLists() failed", " Win32Window::create");
+			    OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "wglShareLists() failed", " Win32Window::create");
         }
 
 		// Create RenderSystem context
 		mContext = new Win32Context(mHDC, mGlrc);
-		// Register the context with the rendersystem and associate it with this window
-		GLRenderSystem *rs = static_cast<GLRenderSystem*>(Root::getSingleton().getRenderSystem());
-		rs->_registerContext(this, mContext);
 
 		mActive = true;
 	}
 
+	void Win32Window::setFullscreen(bool fullScreen, unsigned int width, unsigned int height)
+	{
+		if (mIsFullScreen != fullScreen || width != mWidth || height != mHeight)
+		{
+			mIsFullScreen = fullScreen;
+			DWORD dwStyle = WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
+
+			if (mIsFullScreen)
+			{
+				dwStyle |= WS_POPUP;
+
+				DEVMODE dm;
+				dm.dmSize = sizeof(DEVMODE);
+				dm.dmBitsPerPel = mColourDepth;
+				dm.dmPelsWidth = width;
+				dm.dmPelsHeight = height;
+				dm.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
+				if (mDisplayFrequency)
+				{
+					dm.dmDisplayFrequency = mDisplayFrequency;
+					dm.dmFields |= DM_DISPLAYFREQUENCY;
+					if (ChangeDisplaySettings(&dm, CDS_FULLSCREEN | CDS_TEST) != DISP_CHANGE_SUCCESSFUL)
+					{
+						LogManager::getSingleton().logMessage(LML_NORMAL, "ChangeDisplaySettings with user display frequency failed");
+						dm.dmFields ^= DM_DISPLAYFREQUENCY;
+					}
+				}
+				else
+				{
+					// try a few
+					dm.dmDisplayFrequency = 100;
+					dm.dmFields |= DM_DISPLAYFREQUENCY;
+					if (ChangeDisplaySettings(&dm, CDS_FULLSCREEN | CDS_TEST) != DISP_CHANGE_SUCCESSFUL)
+					{
+						dm.dmDisplayFrequency = 75;
+						if (ChangeDisplaySettings(&dm, CDS_FULLSCREEN | CDS_TEST) != DISP_CHANGE_SUCCESSFUL)
+						{
+							dm.dmFields ^= DM_DISPLAYFREQUENCY;
+						}
+					}
+
+				}
+				if (ChangeDisplaySettings(&dm, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
+					LogManager::getSingleton().logMessage(LML_CRITICAL, "ChangeDisplaySettings failed");
+
+				SetWindowLong(mHWnd, GWL_STYLE, dwStyle);
+				SetWindowPos(mHWnd, HWND_TOPMOST, 0, 0, width, height,
+					SWP_NOACTIVATE);
+				mWidth = width;
+				mHeight = height;
+
+
+			}
+			else
+			{
+				dwStyle |= WS_OVERLAPPEDWINDOW;
+
+				// drop out of fullscreen
+				ChangeDisplaySettings(NULL, 0);
+
+				// calculate overall dimensions for requested client area
+				RECT rc = { 0, 0, width, height };
+				AdjustWindowRect(&rc, dwStyle, false);
+				unsigned int winWidth = rc.right - rc.left;
+				unsigned int winHeight = rc.bottom - rc.top;
+
+				int screenw = GetSystemMetrics(SM_CXSCREEN);
+				int screenh = GetSystemMetrics(SM_CYSCREEN);
+				int left = (screenw - winWidth) / 2;
+				int top = (screenh - winHeight) / 2;
+
+
+				SetWindowLong(mHWnd, GWL_STYLE, dwStyle);
+				SetWindowPos(mHWnd, HWND_NOTOPMOST, left, top, winWidth, winHeight,
+					SWP_DRAWFRAME | SWP_FRAMECHANGED | SWP_NOACTIVATE);
+				mWidth = width;
+				mHeight = height;
+
+			}
+
+		}
+	}
+
 	void Win32Window::destroy(void)
 	{
 		if (!mHWnd)
 			return;
 
 		// Unregister and destroy OGRE GLContext
-		if (mContext)
-		{
-			GLRenderSystem *rs = static_cast<GLRenderSystem*>(Root::getSingleton().getRenderSystem());
-			rs->_unregisterContext(this);
-			delete mContext;
-			mContext = 0;
-		}
-		if (mGlrc)
+		delete mContext;
+
+		if (!mIsExternalGLContext && mGlrc)
 		{
 			wglDeleteContext(mGlrc);
 			mGlrc = 0;
 		}
 		if (!mIsExternal)
 		{
+			WindowEventUtilities::_removeRenderWindow(this);
+
 			if (mIsFullScreen)
 				ChangeDisplaySettings(NULL, 0);
 			DestroyWindow(mHWnd);
 		}
+
 		mActive = false;
+		mClosed = true;
 		mHDC = 0; // no release thanks to CS_OWNDC wndclass style
 		mHWnd = 0;
 	}
@@ -367,7 +490,9 @@ namespace Ogre {
 
 	void Win32Window::swapBuffers(bool waitForVSync)
 	{
-		SwapBuffers(mHDC);
+	  if (!mIsExternalGLControl) {
+	  	SwapBuffers(mHDC);
+	  }
 	}
 
 	void Win32Window::writeContentsToFile(const String& filename)
@@ -383,7 +508,9 @@ namespace Ogre {
 
 		// Read pixels
 		// I love GL: it does all the locking & colour conversion for us
-		glReadPixels(0,0, mWidth-1, mHeight-1, GL_RGB, GL_UNSIGNED_BYTE, pBuffer);
+		if (mIsFullScreen)
+			glReadBuffer(GL_FRONT);
+		glReadPixels(0,0, mWidth, mHeight, GL_RGB, GL_UNSIGNED_BYTE, pBuffer);
 
 		// Wrap buffer in a memory stream
 		DataStreamPtr stream(new MemoryDataStream(pBuffer, mWidth * mHeight * 3, false));
@@ -417,96 +544,47 @@ namespace Ogre {
 		delete [] pBuffer;
 	}
 
-	// Window procedure callback
-	// This is a static member, so applies to all windows but we store the
-	// Win32Window instance in the window data GetWindowLog/SetWindowLog
-	LRESULT Win32Window::WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
-	{
-
-		if (uMsg == WM_CREATE)
-		{
-			// Store pointer to Win32Window in user data area
-			SetWindowLong(hWnd, GWL_USERDATA,
-				(LONG)(((LPCREATESTRUCT)lParam)->lpCreateParams));
-			return 0;
-		}
-
-		// look up window instance
-		// note: it is possible to get a WM_SIZE before WM_CREATE
-		Win32Window* win = (Win32Window*)GetWindowLong(hWnd, GWL_USERDATA);
-		if (!win)
-			return DefWindowProc(hWnd, uMsg, wParam, lParam);
-
-		switch( uMsg )
-		{
-		case WM_ACTIVATE:
-			if (win->mIsFullScreen)
-			{
-				if (LOWORD(wParam) == WA_INACTIVE)
-				{
-					win->mActive = false;
-					ChangeDisplaySettings(NULL, 0);
-					ShowWindow(hWnd, SW_SHOWMINNOACTIVE);
-				}
-				else
-				{
-					win->mActive = true;
-					ShowWindow(hWnd, SW_SHOWNORMAL);
-
-					DEVMODE dm;
-					dm.dmSize = sizeof(DEVMODE);
-					dm.dmBitsPerPel = win->mColourDepth;
-					dm.dmPelsWidth = win->mWidth;
-					dm.dmPelsHeight = win->mHeight;
-					dm.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
-					if (win->mDisplayFrequency)
-					{
-						dm.dmDisplayFrequency = win->mDisplayFrequency;
-						dm.dmFields |= DM_DISPLAYFREQUENCY;
-					}
-					ChangeDisplaySettings(&dm, CDS_FULLSCREEN);
-				}
-			}
-			break;
-
-		case WM_ENTERSIZEMOVE:
-			win->mSizing = true;
-			break;
-
-		case WM_EXITSIZEMOVE:
-			win->windowMovedOrResized();
-			win->mSizing = false;
-			break;
-
-		case WM_MOVE:
-		case WM_SIZE:
-			if (!win->mSizing)
-				win->windowMovedOrResized();
-			break;
-
-		case WM_GETMINMAXINFO:
-			// Prevent the window from going smaller than some minimu size
-			((MINMAXINFO*)lParam)->ptMinTrackSize.x = 100;
-			((MINMAXINFO*)lParam)->ptMinTrackSize.y = 100;
-			break;
-
-		case WM_CLOSE:
-			win->destroy(); // will call DestroyWindow
-			win->mClosed = true;
-			return 0;
-		}
-
-		return DefWindowProc( hWnd, uMsg, wParam, lParam );
-	}
-
 	void Win32Window::getCustomAttribute( const String& name, void* pData )
 	{
-		if( name == "HWND" )
+		if( name == "GLCONTEXT" ) {
+			*static_cast<GLContext**>(pData) = mContext;
+			return;
+		} else if( name == "WINDOW" )
 		{
 			HWND *pHwnd = (HWND*)pData;
 			*pHwnd = getWindowHandle();
 			return;
-		}
+		} 
 	}
 
+	void Win32Window::setActive( bool state )
+	{
+		mActive = state;
+
+		if( mIsFullScreen )
+		{
+			if( state == false )
+			{	//Restore Desktop
+				ChangeDisplaySettings(NULL, 0);
+				ShowWindow(mHWnd, SW_SHOWMINNOACTIVE);
+			}
+			else
+			{	//Restore App
+				ShowWindow(mHWnd, SW_SHOWNORMAL);
+
+				DEVMODE dm;
+				dm.dmSize = sizeof(DEVMODE);
+				dm.dmBitsPerPel = mColourDepth;
+				dm.dmPelsWidth = mWidth;
+				dm.dmPelsHeight = mHeight;
+				dm.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
+				if (mDisplayFrequency)
+				{
+					dm.dmDisplayFrequency = mDisplayFrequency;
+					dm.dmFields |= DM_DISPLAYFREQUENCY;
+				}
+				ChangeDisplaySettings(&dm, CDS_FULLSCREEN);
+			}
+		}
+	}
 }
