@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 
@@ -31,6 +35,9 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreStringConverter.h"
 #include "OgreHardwareBufferManager.h"
 #include "OgreException.h"
+#include "OgreAnimation.h"
+#include "OgreAnimationTrack.h"
+#include "OgreKeyFrame.h"
 
 namespace Ogre {
 
@@ -43,10 +50,12 @@ namespace Ogre {
     {
     }
     //---------------------------------------------------------------------
-    void XMLMeshSerializer::importMesh(const String& filename, Mesh* pMesh)
+    void XMLMeshSerializer::importMesh(const String& filename, 
+		VertexElementType colourElementType, Mesh* pMesh)
     {
         LogManager::getSingleton().logMessage("XMLMeshSerializer reading mesh data from " + filename + "...");
         mpMesh = pMesh;
+		mColourElementType = colourElementType;
         mXMLDoc = new TiXmlDocument(filename);
         mXMLDoc->LoadFile();
 
@@ -58,7 +67,8 @@ namespace Ogre {
         elem = rootElem->FirstChildElement("sharedgeometry");
         if (elem)
         {
-            if(StringConverter::parseInt(elem->Attribute("vertexcount")) > 0)
+            const char *claimedVertexCount_ = elem->Attribute("vertexcount");
+            if(!claimedVertexCount_ || StringConverter::parseInt(claimedVertexCount_) > 0)
             {
                 mpMesh->sharedVertexData = new VertexData();
                 readGeometry(elem, mpMesh->sharedVertexData);
@@ -90,7 +100,22 @@ namespace Ogre {
 		if (elem)
 			readSubMeshNames(elem, mpMesh);
 
-        delete mXMLDoc;
+		// submesh extremes
+		elem = rootElem->FirstChildElement("extremes");
+		if (elem)
+			readExtremes(elem, mpMesh);
+
+		// poses
+		elem = rootElem->FirstChildElement("poses");
+		if (elem)
+			readPoses(elem, mpMesh);
+
+		// animations
+		elem = rootElem->FirstChildElement("animations");
+		if (elem)
+			readAnimations(elem, mpMesh);
+
+		delete mXMLDoc;
 
         LogManager::getSingleton().logMessage("XMLMeshSerializer import successful.");
         
@@ -104,7 +129,6 @@ namespace Ogre {
 
         mXMLDoc = new TiXmlDocument();
         mXMLDoc->InsertEndChild(TiXmlElement("mesh"));
-        TiXmlElement* rootNode = mXMLDoc->RootElement();
 
         LogManager::getSingleton().logMessage("Populating DOM...");
 
@@ -163,7 +187,8 @@ namespace Ogre {
 
                 while (bi.hasMoreElements())
                 {
-                    writeBoneAssignment(boneAssignNode, &(bi.getNext()));
+					const VertexBoneAssignment& assign = bi.getNext();
+                    writeBoneAssignment(boneAssignNode, &assign);
                 }
 
                 LogManager::getSingleton().logMessage("Shared geometry bone assignments exported.");
@@ -177,11 +202,12 @@ namespace Ogre {
 		}
         // Write submesh names
         writeSubMeshNames(rootNode, pMesh);
-
-
-
-
-
+		// Write poses
+		writePoses(rootNode, pMesh);
+		// Write animations
+		writeAnimations(rootNode, pMesh);
+        // Write extremes
+        writeExtremes(rootNode, pMesh);
     }
     //---------------------------------------------------------------------
     void XMLMeshSerializer::writeSubMesh(TiXmlElement* mSubMeshesNode, const SubMesh* s)
@@ -237,7 +263,7 @@ namespace Ogre {
         facesNode->SetAttribute("count", 
             StringConverter::toString(numFaces));
         // Write each face in turn
-        ushort i;
+        size_t i;
 		unsigned int* pInt;
 		unsigned short* pShort;
 		HardwareIndexBufferSharedPtr ibuf = s->indexData->indexBuffer;
@@ -285,6 +311,9 @@ namespace Ogre {
             writeGeometry(geomNode, s->vertexData);
         }
 
+        // texture aliases
+        writeTextureAliases(subMeshNode, s);
+
         // Bone assignments
         if (mpMesh->hasSkeleton())
         {
@@ -295,7 +324,8 @@ namespace Ogre {
                 subMeshNode->InsertEndChild(TiXmlElement("boneassignments"))->ToElement();
             while (bi.hasMoreElements())
             {
-                writeBoneAssignment(boneAssignNode, &bi.getNext());
+				const VertexBoneAssignment& assign = bi.getNext();
+                writeBoneAssignment(boneAssignNode, &assign);
             }
         }
         LogManager::getSingleton().logMessage("Dedicated geometry bone assignments exported.");
@@ -348,6 +378,12 @@ namespace Ogre {
 				case VES_NORMAL:
 					vbNode->SetAttribute("normals","true");
                     break;
+				case VES_TANGENT:
+					vbNode->SetAttribute("tangents","true");
+					break;
+				case VES_BINORMAL:
+					vbNode->SetAttribute("binormals","true");
+					break;
 				case VES_DIFFUSE:
 					vbNode->SetAttribute("colours_diffuse","true");
                     break;
@@ -360,6 +396,7 @@ namespace Ogre {
                         StringConverter::toString(VertexElement::getTypeCount(elem.getType())));
                     ++numTextureCoords;
                     break;
+
                 default:
                     break;
                 }
@@ -397,6 +434,22 @@ namespace Ogre {
 						dataNode->SetAttribute("y", StringConverter::toString(pFloat[1]));
 						dataNode->SetAttribute("z", StringConverter::toString(pFloat[2]));
 						break;
+					case VES_TANGENT:
+						elem.baseVertexPointerToElement(pVert, &pFloat);
+						dataNode = 
+							vertexNode->InsertEndChild(TiXmlElement("tangent"))->ToElement();
+						dataNode->SetAttribute("x", StringConverter::toString(pFloat[0]));
+						dataNode->SetAttribute("y", StringConverter::toString(pFloat[1]));
+						dataNode->SetAttribute("z", StringConverter::toString(pFloat[2]));
+						break;
+					case VES_BINORMAL:
+						elem.baseVertexPointerToElement(pVert, &pFloat);
+						dataNode = 
+							vertexNode->InsertEndChild(TiXmlElement("binormal"))->ToElement();
+						dataNode->SetAttribute("x", StringConverter::toString(pFloat[0]));
+						dataNode->SetAttribute("y", StringConverter::toString(pFloat[1]));
+						dataNode->SetAttribute("z", StringConverter::toString(pFloat[2]));
+						break;
 					case VES_DIFFUSE:
 						elem.baseVertexPointerToElement(pVert, &pColour);
 						dataNode = 
@@ -482,6 +535,29 @@ namespace Ogre {
             StringConverter::toString(assign->weight));
 
 
+    }
+    //---------------------------------------------------------------------
+    void XMLMeshSerializer::writeTextureAliases(TiXmlElement* mSubmeshesNode, const SubMesh* subMesh)
+    {
+        if (!subMesh->hasTextureAliases())
+            return; // do nothing
+
+        TiXmlElement* textureAliasesNode = 
+            mSubmeshesNode->InsertEndChild(TiXmlElement("textures"))->ToElement();
+
+        // use ogre map iterator
+        SubMesh::AliasTextureIterator aliasIterator = subMesh->getAliasTextureIterator();
+
+        while (aliasIterator.hasMoreElements())
+        {
+            TiXmlElement* aliasTextureNode = 
+                textureAliasesNode->InsertEndChild(TiXmlElement("texture"))->ToElement();
+            // iterator key is alias and value is texture name
+            aliasTextureNode->SetAttribute("alias", aliasIterator.peekNextKey());
+            aliasTextureNode->SetAttribute("name", aliasIterator.peekNextValue());
+            aliasIterator.moveNext();
+        }
+
     }
     //---------------------------------------------------------------------
     void XMLMeshSerializer::readSubMeshes(TiXmlElement* mSubmeshesNode)
@@ -527,17 +603,29 @@ namespace Ogre {
             
             // Faces
             TiXmlElement* faces = smElem->FirstChildElement("faces");
+            size_t actualCount = 0;
+            for (TiXmlElement *faceElem = faces->FirstChildElement(); faceElem != 0; faceElem = faceElem->NextSiblingElement())
+            {
+                    actualCount++;
+            }
+            const char *claimedCount_ = faces->Attribute("count");
+            if (claimedCount_ && StringConverter::parseInt(claimedCount_)!=actualCount)
+            {
+				StringUtil::StrStreamType str;
+                str << "WARNING: face count (" << actualCount << ") " <<
+					"is not as claimed (" << claimedCount_ << ")";
+				LogManager::getSingleton().logMessage(str.str());
+            }
+
+
+            // Faces
             if (sm->operationType == RenderOperation::OT_TRIANGLE_LIST)
             {
-                // tri list
-                sm->indexData->indexCount = 
-                    StringConverter::parseInt(faces->Attribute("count")) * 3;
-            }
-            else
-            {
-                // tri strip or fan
-                sm->indexData->indexCount = 
-                    StringConverter::parseInt(faces->Attribute("count")) + 2;
+                    // tri list
+                    sm->indexData->indexCount = actualCount * 3;
+            } else {
+                    // tri strip or fan
+                    sm->indexData->indexCount = actualCount + 2;
             }
 
             // Allocate space
@@ -600,6 +688,11 @@ namespace Ogre {
                 }
             }
 
+            // texture aliases
+            TiXmlElement* textureAliasesNode = smElem->FirstChildElement("textures");
+            if(textureAliasesNode)
+                readTextureAliases(textureAliasesNode, sm);
+
             // Bone assignments
             TiXmlElement* boneAssigns = smElem->FirstChildElement("boneassignments");
             if(boneAssigns)
@@ -616,9 +709,15 @@ namespace Ogre {
         float *pFloat;
         ARGB *pCol;
 
-        vertexData->vertexCount = StringConverter::parseInt(mGeometryNode->Attribute("vertexcount"));
+        const char *claimedVertexCount_ = mGeometryNode->Attribute("vertexcount");
+        ptrdiff_t claimedVertexCount;
+        if (claimedVertexCount_)
+        {
+                claimedVertexCount =
+                        StringConverter::parseInt(claimedVertexCount_);
+        }
         // Skip empty 
-        if (vertexData->vertexCount <= 0) return;
+        if (claimedVertexCount_ && claimedVertexCount <= 0) return;
         
 
         VertexDeclaration* decl = vertexData->vertexDeclaration;
@@ -653,19 +752,33 @@ namespace Ogre {
                 decl->addElement(bufCount, offset, VET_FLOAT3, VES_NORMAL);
                 offset += VertexElement::getTypeSize(VET_FLOAT3);
             }
+			attrib = vbElem->Attribute("tangents");
+			if (attrib && StringConverter::parseBool(attrib))
+			{
+				// Add element
+				decl->addElement(bufCount, offset, VET_FLOAT3, VES_TANGENT);
+				offset += VertexElement::getTypeSize(VET_FLOAT3);
+			}
+			attrib = vbElem->Attribute("binormals");
+			if (attrib && StringConverter::parseBool(attrib))
+			{
+				// Add element
+				decl->addElement(bufCount, offset, VET_FLOAT3, VES_BINORMAL);
+				offset += VertexElement::getTypeSize(VET_FLOAT3);
+			}
             attrib = vbElem->Attribute("colours_diffuse");
             if (attrib && StringConverter::parseBool(attrib))
             {
                 // Add element
-                decl->addElement(bufCount, offset, VET_COLOUR, VES_DIFFUSE);
-                offset += VertexElement::getTypeSize(VET_COLOUR);
+                decl->addElement(bufCount, offset, mColourElementType, VES_DIFFUSE);
+                offset += VertexElement::getTypeSize(mColourElementType);
             }
             attrib = vbElem->Attribute("colours_specular");
             if (attrib && StringConverter::parseBool(attrib))
             {
                 // Add element
-                decl->addElement(bufCount, offset, VET_COLOUR, VES_SPECULAR);
-                offset += VertexElement::getTypeSize(VET_COLOUR);
+                decl->addElement(bufCount, offset, mColourElementType, VES_SPECULAR);
+                offset += VertexElement::getTypeSize(mColourElementType);
             }
             attrib = vbElem->Attribute("texture_coords");
             if (attrib && StringConverter::parseInt(attrib))
@@ -675,7 +788,9 @@ namespace Ogre {
                 {
                     // NB set is local to this buffer, but will be translated into a 
                     // global set number across all vertex buffers
-                    attrib = vbElem->Attribute("texture_coord_dimensions_" + StringConverter::toString(tx));
+					StringUtil::StrStreamType str;
+					str << "texture_coord_dimensions_" << tx;
+                    attrib = vbElem->Attribute(str.str().c_str());
                     unsigned short dims;
                     if (attrib)
                     {
@@ -693,6 +808,22 @@ namespace Ogre {
                     offset += VertexElement::getTypeSize(vtype);
                 }
             } 
+
+            // calculate how many vertexes there actually are
+            int actualVertexCount = 0;
+            for (TiXmlElement * vertexElem = vbElem->FirstChildElement(); vertexElem != 0; vertexElem = vertexElem->NextSiblingElement())
+            {
+                    actualVertexCount++;
+            }
+            if (claimedVertexCount_ && actualVertexCount!=claimedVertexCount)
+            {
+				StringUtil::StrStreamType str;
+				str << "WARNING: vertex count (" << actualVertexCount 
+					<< ") is not as claimed (" << claimedVertexCount_ << ")";
+				LogManager::getSingleton().logMessage(str.str());
+            }
+
+            vertexData->vertexCount = actualVertexCount;
             // Now create the vertex buffer
             HardwareVertexBufferSharedPtr vbuf = HardwareBufferManager::getSingleton().
                 createVertexBuffer(offset, vertexData->vertexCount, 
@@ -772,6 +903,38 @@ namespace Ogre {
                         *pFloat++ = StringConverter::parseReal(
                             xmlElem->Attribute("z"));
                         break;
+					case VES_TANGENT:
+						xmlElem = vertexElem->FirstChildElement("tangent");
+						if (!xmlElem)
+						{
+							OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Missing <tangent> element.",
+								"XMLSerializer::readGeometry");
+						}
+						elem.baseVertexPointerToElement(pVert, &pFloat);
+
+						*pFloat++ = StringConverter::parseReal(
+							xmlElem->Attribute("x"));
+						*pFloat++ = StringConverter::parseReal(
+							xmlElem->Attribute("y"));
+						*pFloat++ = StringConverter::parseReal(
+							xmlElem->Attribute("z"));
+						break;
+					case VES_BINORMAL:
+						xmlElem = vertexElem->FirstChildElement("binormal");
+						if (!xmlElem)
+						{
+							OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Missing <binormal> element.",
+								"XMLSerializer::readGeometry");
+						}
+						elem.baseVertexPointerToElement(pVert, &pFloat);
+
+						*pFloat++ = StringConverter::parseReal(
+							xmlElem->Attribute("x"));
+						*pFloat++ = StringConverter::parseReal(
+							xmlElem->Attribute("y"));
+						*pFloat++ = StringConverter::parseReal(
+							xmlElem->Attribute("z"));
+						break;
                     case VES_DIFFUSE:
                         xmlElem = vertexElem->FirstChildElement("colour_diffuse");
                         if (!xmlElem)
@@ -784,7 +947,7 @@ namespace Ogre {
 							ColourValue cv;
 							cv = StringConverter::parseColourValue(
 								xmlElem->Attribute("value"));
-							*pCol++ = cv.getAsARGB();
+							*pCol++ = VertexElement::convertColourValue(cv, mColourElementType);
 						}
                         break;
                     case VES_SPECULAR:
@@ -799,7 +962,7 @@ namespace Ogre {
 							ColourValue cv;
 							cv = StringConverter::parseColourValue(
 								xmlElem->Attribute("value"));
-							*pCol++ = cv.getAsARGB();
+							*pCol++ = VertexElement::convertColourValue(cv, mColourElementType);
 						}
                         break;
                     case VES_TEXTURE_COORDINATES:
@@ -894,6 +1057,26 @@ namespace Ogre {
 
         LogManager::getSingleton().logMessage("Bone assignments done.");
     }
+    //---------------------------------------------------------------------
+    void XMLMeshSerializer::readTextureAliases(TiXmlElement* mTextureAliasesNode, SubMesh* subMesh)
+    {
+        LogManager::getSingleton().logMessage("Reading sub mesh texture aliases...");
+
+        // Iterate over all children (texture entries)
+        for (TiXmlElement* elem = mTextureAliasesNode->FirstChildElement();
+             elem != 0; elem = elem->NextSiblingElement())
+        {
+            // pass alias and texture name to submesh
+            // read attribute "alias"
+            String alias = elem->Attribute("alias");
+            // read attribute "name"
+            String name = elem->Attribute("name");
+
+            subMesh->addTextureAlias(alias, name);
+        }
+
+        LogManager::getSingleton().logMessage("Texture aliases done.");
+    }
     //---------------------------------------------------------------------
 	void XMLMeshSerializer::readSubMeshNames(TiXmlElement* mMeshNamesNode, Mesh *sm)
 	{
@@ -1057,6 +1240,37 @@ namespace Ogre {
 
 	}
     //---------------------------------------------------------------------
+	void XMLMeshSerializer::writeExtremes(TiXmlElement* mMeshNode, const Mesh* m)
+	{
+		TiXmlElement* extremesNode = NULL;
+		int idx = 0;
+		for (Mesh::SubMeshIterator i = ((Mesh &)*m).getSubMeshIterator ();
+			 i.hasMoreElements (); i.moveNext (), ++idx)
+		{
+			SubMesh *sm = i.peekNext ();
+			if (sm->extremityPoints.empty())
+				continue; // do nothing
+
+			if (!extremesNode)
+				extremesNode = mMeshNode->InsertEndChild(TiXmlElement("extremes"))->ToElement();
+
+			TiXmlElement* submeshNode =
+				extremesNode->InsertEndChild(TiXmlElement("submesh_extremes"))->ToElement();
+
+			submeshNode->SetAttribute("index",  StringConverter::toString(idx));
+
+			for (std::vector<Vector3>::const_iterator v = sm->extremityPoints.begin ();
+				 v != sm->extremityPoints.end (); ++v)
+			{
+				TiXmlElement* vert = submeshNode->InsertEndChild(
+					TiXmlElement("position"))->ToElement();
+				vert->SetAttribute("x", StringConverter::toString(v->x));
+				vert->SetAttribute("y", StringConverter::toString(v->y));
+				vert->SetAttribute("z", StringConverter::toString(v->z));
+			}
+		}
+	}
+	//---------------------------------------------------------------------
 	void XMLMeshSerializer::readLodInfo(TiXmlElement*  lodNode)
 	{
 		
@@ -1189,6 +1403,475 @@ namespace Ogre {
 		}
         
 	}
+	//-----------------------------------------------------------------------------
+	void XMLMeshSerializer::readExtremes(TiXmlElement* extremesNode, Mesh *m)
+	{
+		LogManager::getSingleton().logMessage("Reading extremes...");
+
+		// Iterate over all children (submesh_extreme list)
+		for (TiXmlElement* elem = extremesNode->FirstChildElement();
+			 elem != 0; elem = elem->NextSiblingElement())
+		{
+			int index = StringConverter::parseInt(elem->Attribute("index"));
+
+			SubMesh *sm = m->getSubMesh(index);
+			sm->extremityPoints.clear ();
+			for (TiXmlElement* vert = elem->FirstChildElement();
+				 vert != 0; vert = vert->NextSiblingElement())
+			{
+				Vector3 v;
+				v.x = StringConverter::parseReal(vert->Attribute("x"));
+				v.y = StringConverter::parseReal(vert->Attribute("y"));
+				v.z = StringConverter::parseReal(vert->Attribute("z"));
+				sm->extremityPoints.push_back (v);
+			}
+		}
+
+		LogManager::getSingleton().logMessage("Extremes done.");
+	}
+	//-----------------------------------------------------------------------------
+	void XMLMeshSerializer::readPoses(TiXmlElement* posesNode, Mesh *m)
+	{
+		TiXmlElement* poseNode = posesNode->FirstChildElement("pose");
+
+		while (poseNode)
+		{
+			const char* target = poseNode->Attribute("target");
+			if (!target)
+			{
+				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+					"Required attribute 'target' missing on pose", 
+					"XMLMeshSerializer::readPoses");
+			}
+			unsigned short targetID;
+			if(String(target) == "mesh")
+			{
+				targetID = 0;
+			}
+			else
+			{
+				// submesh, get index
+				const char* val = poseNode->Attribute("index");
+				if (!val)
+				{
+					OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+						"Required attribute 'index' missing on pose", 
+						"XMLMeshSerializer::readPoses");
+				}
+				unsigned short submeshIndex = static_cast<unsigned short>(
+					StringConverter::parseUnsignedInt(val));
+
+				targetID = submeshIndex + 1;
+			}
+
+			String name;
+			const char* val = poseNode->Attribute("name");
+			if (val)
+				name = val;
+			Pose* pose = m->createPose(targetID, name);
+
+			TiXmlElement* poseOffsetNode = poseNode->FirstChildElement("poseoffset");
+			while (poseOffsetNode)
+			{
+				uint index = StringConverter::parseUnsignedInt(
+					poseOffsetNode->Attribute("index"));
+				Vector3 offset;
+				offset.x = StringConverter::parseReal(poseOffsetNode->Attribute("x"));
+				offset.y = StringConverter::parseReal(poseOffsetNode->Attribute("y"));
+				offset.z = StringConverter::parseReal(poseOffsetNode->Attribute("z"));
+
+				pose->addVertex(index, offset);
+
+				poseOffsetNode = poseOffsetNode->NextSiblingElement();
+			}
+
+			poseNode = poseNode->NextSiblingElement();
+
+		}
+
+
+	}
+	//-----------------------------------------------------------------------------
+	void XMLMeshSerializer::readAnimations(TiXmlElement* mAnimationsNode, Mesh *pMesh)
+	{
+		TiXmlElement* animElem = mAnimationsNode->FirstChildElement("animation");
+		while (animElem)
+		{
+			String name = animElem->Attribute("name");
+			const char* charLen = animElem->Attribute("length");
+			Real len = StringConverter::parseReal(charLen);
+
+			Animation* anim = pMesh->createAnimation(name, len);
+
+			TiXmlElement* tracksNode = animElem->FirstChildElement("tracks");
+			if (tracksNode)
+			{
+				readTracks(tracksNode, pMesh, anim);
+			}
+
+			animElem = animElem->NextSiblingElement();
+
+		}
+
+
+	}
+	//-----------------------------------------------------------------------------
+	void XMLMeshSerializer::readTracks(TiXmlElement* tracksNode, Mesh *m, Animation* anim)
+	{
+		TiXmlElement* trackNode = tracksNode->FirstChildElement("track");
+		while (trackNode)
+		{
+			String target = trackNode->Attribute("target");
+			unsigned short targetID;
+			VertexData* vertexData = 0;
+			if(target == "mesh")
+			{
+				targetID = 0;
+				vertexData = m->sharedVertexData;
+			}
+			else
+			{
+				// submesh, get index
+				const char* val = trackNode->Attribute("index");
+				if (!val)
+				{
+					OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+						"Required attribute 'index' missing on submesh track", 
+						"XMLMeshSerializer::readTracks");
+				}
+				unsigned short submeshIndex = static_cast<unsigned short>(
+					StringConverter::parseUnsignedInt(val));
+
+				targetID = submeshIndex + 1;
+				vertexData = m->getSubMesh(submeshIndex)->vertexData;
+
+			}
+
+			if (!vertexData)
+			{
+				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+					"Track cannot be created for " + target + " since VertexData "
+					"does not exist at the specified index", 
+					"XMLMeshSerializer::readTracks");
+			}
+
+			// Get type
+			VertexAnimationType animType = VAT_NONE;
+			String strAnimType = trackNode->Attribute("type");
+			if (strAnimType == "morph")
+			{
+				animType = VAT_MORPH;
+			}
+			else if (strAnimType == "pose")
+			{
+				animType = VAT_POSE;
+			}
+			else
+			{
+				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+					"Unrecognised animation track type '" + strAnimType + "'",
+					"XMLMeshSerializer::readTracks");
+			}
+
+			// Create track
+			VertexAnimationTrack* track = 
+				anim->createVertexTrack(targetID, vertexData, animType);
+
+			TiXmlElement* keyframesNode = trackNode->FirstChildElement("keyframes");
+			if (keyframesNode)
+			{
+				if (track->getAnimationType() == VAT_MORPH)
+				{
+					readMorphKeyFrames(keyframesNode, track, vertexData->vertexCount);
+				}
+				else // VAT_POSE
+				{
+					readPoseKeyFrames(keyframesNode, track);
+				}
+			}
+
+			trackNode = trackNode->NextSiblingElement();
+		}
+	}
+	//-----------------------------------------------------------------------------
+	void XMLMeshSerializer::readMorphKeyFrames(TiXmlElement* keyframesNode, 
+		VertexAnimationTrack* track, size_t vertexCount)
+	{
+		TiXmlElement* keyNode = keyframesNode->FirstChildElement("keyframe");
+		while (keyNode)
+		{
+			const char* val = keyNode->Attribute("time");
+			if (!val)
+			{
+				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+					"Required attribute 'time' missing on keyframe", 
+					"XMLMeshSerializer::readKeyFrames");
+			}
+			Real time = StringConverter::parseReal(val);
+
+			VertexMorphKeyFrame* kf = track->createVertexMorphKeyFrame(time);
+
+			// create a vertex buffer
+			HardwareVertexBufferSharedPtr vbuf = 
+				HardwareBufferManager::getSingleton().createVertexBuffer(
+				VertexElement::getTypeSize(VET_FLOAT3), vertexCount, 
+				HardwareBuffer::HBU_STATIC, true);
+
+			float* pFloat = static_cast<float*>(
+				vbuf->lock(HardwareBuffer::HBL_DISCARD));
+
+
+			TiXmlElement* posNode = keyNode->FirstChildElement("position");
+			for (size_t v = 0; v < vertexCount; ++v)
+			{
+				if (!posNode)
+				{
+					OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+						"Not enough 'position' elements under keyframe", 
+						"XMLMeshSerializer::readKeyFrames");
+
+				}
+
+				*pFloat++ = StringConverter::parseReal(
+					posNode->Attribute("x"));
+				*pFloat++ = StringConverter::parseReal(
+					posNode->Attribute("y"));
+				*pFloat++ = StringConverter::parseReal(
+					posNode->Attribute("z"));
+
+
+				posNode = posNode->NextSiblingElement("position");
+			}
+
+			vbuf->unlock();
+
+			kf->setVertexBuffer(vbuf);
+				
+
+			keyNode = keyNode->NextSiblingElement();
+		}
+
+
+	}
+	//-----------------------------------------------------------------------------
+	void XMLMeshSerializer::readPoseKeyFrames(TiXmlElement* keyframesNode, VertexAnimationTrack* track)
+	{
+		TiXmlElement* keyNode = keyframesNode->FirstChildElement("keyframe");
+		while (keyNode)
+		{
+			const char* val = keyNode->Attribute("time");
+			if (!val)
+			{
+				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+					"Required attribute 'time' missing on keyframe", 
+					"XMLMeshSerializer::readKeyFrames");
+			}
+			Real time = StringConverter::parseReal(val);
+
+			VertexPoseKeyFrame* kf = track->createVertexPoseKeyFrame(time);
+
+			// Read all pose references
+			TiXmlElement* poseRefNode = keyNode->FirstChildElement("poseref");
+			while (poseRefNode)
+			{
+				const char* val = poseRefNode->Attribute("poseindex");
+				if (!val)
+				{
+					OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+						"Required attribute 'poseindex' missing on poseref", 
+						"XMLMeshSerializer::readPoseKeyFrames");
+				}
+				unsigned short poseIndex = StringConverter::parseUnsignedInt(val);
+				Real influence = 1.0f;
+				val = poseRefNode->Attribute("influence");
+				if (val)
+				{
+					influence = StringConverter::parseReal(val);
+				}
+
+				kf->addPoseReference(poseIndex, influence);
+
+				poseRefNode = poseRefNode->NextSiblingElement();
+			}
+
+			keyNode = keyNode->NextSiblingElement();
+		}
+
+	}
+	//-----------------------------------------------------------------------------
+	void XMLMeshSerializer::writePoses(TiXmlElement* meshNode, const Mesh* m)
+	{
+		if (m->getPoseCount() == 0)
+			return;
+
+		TiXmlElement* posesNode = 
+			meshNode->InsertEndChild(TiXmlElement("poses"))->ToElement();
+
+		Mesh::ConstPoseIterator poseIt = m->getPoseIterator();
+		while (poseIt.hasMoreElements())
+		{
+			const Pose* pose = poseIt.getNext();
+			TiXmlElement* poseNode = 
+				posesNode->InsertEndChild(TiXmlElement("pose"))->ToElement();
+			unsigned short target = pose->getTarget();
+			if (target == 0)
+			{
+				// Main mesh
+				poseNode->SetAttribute("target", "mesh");
+			}
+			else
+			{
+				// Submesh - rebase index
+				poseNode->SetAttribute("target", "submesh");
+				poseNode->SetAttribute("index", 
+					StringConverter::toString(target - 1));
+			}
+			poseNode->SetAttribute("name", pose->getName());
+
+			Pose::ConstVertexOffsetIterator vit = pose->getVertexOffsetIterator();
+			while (vit.hasMoreElements())
+			{
+				TiXmlElement* poseOffsetElement = poseNode->InsertEndChild(
+					TiXmlElement("poseoffset"))->ToElement();
+
+				poseOffsetElement->SetAttribute("index", 
+					StringConverter::toString(vit.peekNextKey()));
+
+				Vector3 offset = vit.getNext();
+				poseOffsetElement->SetAttribute("x", StringConverter::toString(offset.x));
+				poseOffsetElement->SetAttribute("y", StringConverter::toString(offset.y));
+				poseOffsetElement->SetAttribute("z", StringConverter::toString(offset.z));
+
+
+			}
+
+		}
+
+	}
+	//-----------------------------------------------------------------------------
+	void XMLMeshSerializer::writeAnimations(TiXmlElement* meshNode, const Mesh* m)
+	{
+		// Skip if no animation
+		if (!m->hasVertexAnimation())
+			return;
+
+		TiXmlElement* animationsNode = 
+			meshNode->InsertEndChild(TiXmlElement("animations"))->ToElement();
+
+		for (unsigned short a = 0; a < m->getNumAnimations(); ++a)
+		{
+			Animation* anim = m->getAnimation(a);
+
+			TiXmlElement* animNode = 
+				animationsNode->InsertEndChild(TiXmlElement("animation"))->ToElement();
+			animNode->SetAttribute("name", anim->getName());
+			animNode->SetAttribute("length", 
+				StringConverter::toString(anim->getLength()));
+
+			TiXmlElement* tracksNode = 
+				animNode->InsertEndChild(TiXmlElement("tracks"))->ToElement();
+			Animation::VertexTrackIterator iter = anim->getVertexTrackIterator();
+			while(iter.hasMoreElements())
+			{
+				const VertexAnimationTrack* track = iter.getNext();
+				TiXmlElement* trackNode = 
+					tracksNode->InsertEndChild(TiXmlElement("track"))->ToElement();
+
+				unsigned short targetID = track->getHandle();
+				if (targetID == 0)
+				{
+					trackNode->SetAttribute("target", "mesh");
+				}
+				else
+				{
+					trackNode->SetAttribute("target", "submesh");
+					trackNode->SetAttribute("index", 
+						StringConverter::toString(targetID-1));
+				}
+
+				if (track->getAnimationType() == VAT_MORPH)
+				{
+					trackNode->SetAttribute("type", "morph");
+					writeMorphKeyFrames(trackNode, track);
+				}
+				else
+				{
+					trackNode->SetAttribute("type", "pose");
+					writePoseKeyFrames(trackNode, track);
+				}
+
+
+			}
+		}
+
+		
+	}
+	//-----------------------------------------------------------------------------
+	void XMLMeshSerializer::writeMorphKeyFrames(TiXmlElement* trackNode, const VertexAnimationTrack* track)
+	{
+		TiXmlElement* keyframesNode = 
+			trackNode->InsertEndChild(TiXmlElement("keyframes"))->ToElement();
+
+		size_t vertexCount = track->getAssociatedVertexData()->vertexCount;
+
+		for (unsigned short k = 0; k < track->getNumKeyFrames(); ++k)
+		{
+			VertexMorphKeyFrame* kf = track->getVertexMorphKeyFrame(k);
+			TiXmlElement* keyNode = 
+				keyframesNode->InsertEndChild(TiXmlElement("keyframe"))->ToElement();
+			keyNode->SetAttribute("time", 
+				StringConverter::toString(kf->getTime()));
+
+			HardwareVertexBufferSharedPtr vbuf = kf->getVertexBuffer();
+			float* pFloat = static_cast<float*>(
+				vbuf->lock(HardwareBuffer::HBL_READ_ONLY));
+
+			for (size_t v = 0; v < vertexCount; ++v)
+			{
+				TiXmlElement* posNode = 
+					keyNode->InsertEndChild(TiXmlElement("position"))->ToElement();
+				posNode->SetAttribute("x", StringConverter::toString(*pFloat++));
+				posNode->SetAttribute("y", StringConverter::toString(*pFloat++));
+				posNode->SetAttribute("z", StringConverter::toString(*pFloat++));
+			}
+
+		}
+	}
+	//-----------------------------------------------------------------------------
+	void XMLMeshSerializer::writePoseKeyFrames(TiXmlElement* trackNode, const VertexAnimationTrack* track)
+	{
+		TiXmlElement* keyframesNode = 
+			trackNode->InsertEndChild(TiXmlElement("keyframes"))->ToElement();
+
+		for (unsigned short k = 0; k < track->getNumKeyFrames(); ++k)
+		{
+			VertexPoseKeyFrame* kf = track->getVertexPoseKeyFrame(k);
+			TiXmlElement* keyNode = 
+				keyframesNode->InsertEndChild(TiXmlElement("keyframe"))->ToElement();
+			keyNode->SetAttribute("time", 
+				StringConverter::toString(kf->getTime()));
+
+			VertexPoseKeyFrame::PoseRefIterator poseIt = kf->getPoseReferenceIterator();
+			while (poseIt.hasMoreElements())
+			{
+				const VertexPoseKeyFrame::PoseRef& poseRef = poseIt.getNext();
+				TiXmlElement* poseRefNode = 
+					keyNode->InsertEndChild(TiXmlElement("poseref"))->ToElement();
+
+				poseRefNode->SetAttribute("poseindex", 
+					StringConverter::toString(poseRef.poseIndex));
+				poseRefNode->SetAttribute("influence", 
+					StringConverter::toString(poseRef.influence));
+
+			}
+
+		}
+
+
+	}
+
+
+
 
 }
 
