@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2005 The OGRE Team
+Copyright (c) 2000-2006 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 This program is free software; you can redistribute it and/or modify it under
@@ -20,6 +20,10 @@ You should have received a copy of the GNU Lesser General Public License along w
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/lesser.txt.
+
+You may alternatively use this source under the terms of a specific version of
+the OGRE Unrestricted License provided you have obtained such a license from
+Torus Knot Software Ltd.
 -----------------------------------------------------------------------------
 */
 
@@ -43,23 +47,30 @@ struct XmlOptions
     String dest;
     String sourceExt;
     String destExt;
+    String logFile;
     bool interactiveMode;
     unsigned short numLods;
     Real lodDist;
     Real lodPercent;
     size_t lodFixed;
+    size_t nuextremityPoints;
     bool usePercent;
     bool generateEdgeLists;
     bool generateTangents;
+	VertexElementSemantic tangentSemantic;
     bool reorganiseBuffers;
 	bool optimiseAnimations;
+	bool quietMode;
+	bool d3d;
+	bool gl;
+	Serializer::Endian endian;
 };
 
 void help(void)
 {
     // Print help message
     cout << endl << "OgreXMLConvert: Converts data between XML and OGRE binary formats." << endl;
-    cout << "Provided for OGRE by Steve Streeting 2002" << endl << endl;
+    cout << "Provided for OGRE by Steve Streeting" << endl << endl;
     cout << "Usage: OgreXMLConverter [options] sourcefile [destfile] " << endl;
 	cout << endl << "Available options:" << endl;
     cout << "-i             = interactive mode - prompt for options" << endl;
@@ -71,7 +82,15 @@ void help(void)
     cout << "-e             = DON'T generate edge lists (for stencil shadows)" << endl;
     cout << "-r             = DON'T reorganise vertex buffers to OGRE recommended format." << endl;
     cout << "-t             = Generate tangents (for normal mapping)" << endl;
+	cout << "-td [uvw|tangent]" << endl;
+	cout << "           = Tangent vertex semantic destination (default tangent)" << endl;
     cout << "-o             = DON'T optimise out redundant tracks & keyframes" << endl;
+	cout << "-d3d           = Prefer D3D packed colour formats (default on Windows)" << endl;
+	cout << "-gl            = Prefer GL packed colour formats (default on non-Windows)" << endl;
+	cout << "-E endian      = Set endian mode 'big' 'little' or 'native' (default)" << endl;
+	cout << "-x num         = Generate no more than num eXtremes for every submesh (default 0)" << endl;
+	cout << "-q             = Quiet mode, less output" << endl;
+    cout << "-log filename  = name of the log file (default: 'OgreXMLConverter.log')" << endl;
     cout << "sourcefile     = name of file to convert" << endl;
     cout << "destfile       = optional name of file to write to. If you don't" << endl;
     cout << "                 specify this OGRE works it out through the extension " << endl;
@@ -92,11 +111,15 @@ XmlOptions parseArgs(int numArgs, char **args)
     opts.lodFixed = 0;
     opts.lodPercent = 20;
     opts.numLods = 0;
+    opts.nuextremityPoints = 0;
     opts.usePercent = true;
     opts.generateEdgeLists = true;
     opts.generateTangents = false;
+	opts.tangentSemantic = VES_TANGENT;
     opts.reorganiseBuffers = true;
 	opts.optimiseAnimations = true;
+    opts.quietMode = false;
+	opts.endian = Serializer::ENDIAN_NATIVE;
 
     // ignore program name
     char* source = 0;
@@ -111,16 +134,29 @@ XmlOptions parseArgs(int numArgs, char **args)
     unOpt["-r"] = false;
     unOpt["-t"] = false;
     unOpt["-o"] = false;
+	unOpt["-q"] = false;
+	unOpt["-d3d"] = false;
+	unOpt["-gl"] = false;
     binOpt["-l"] = "";
     binOpt["-d"] = "";
     binOpt["-p"] = "";
     binOpt["-f"] = "";
+    binOpt["-E"] = "";
+    binOpt["-x"] = "";
+    binOpt["-log"] = "OgreXMLConverter.log";
+	binOpt["-td"] = "";
 
     int startIndex = findCommandLineOpts(numArgs, args, unOpt, binOpt);
     UnaryOptionList::iterator ui;
     BinaryOptionList::iterator bi;
 
-    ui = unOpt.find("-i");
+	ui = unOpt.find("-q");
+	if (ui->second)
+	{
+		opts.quietMode = true;
+	}
+
+	ui = unOpt.find("-i");
     if (ui->second)
     {
         opts.interactiveMode = true;
@@ -144,6 +180,14 @@ XmlOptions parseArgs(int numArgs, char **args)
         {
             opts.generateTangents = true;
         }
+		bi = binOpt.find("-td");
+		if (!bi->second.empty())
+		{
+			if (bi->second == "uvw")
+				opts.tangentSemantic = VES_TEXTURE_COORDINATES;
+			else
+				opts.tangentSemantic = VES_TANGENT;
+		}
 
         ui = unOpt.find("-o");
         if (ui->second)
@@ -177,7 +221,44 @@ XmlOptions parseArgs(int numArgs, char **args)
             opts.lodFixed = StringConverter::parseInt(bi->second);
             opts.usePercent = false;
         }
-    }
+
+        bi = binOpt.find("-x");
+        if (!bi->second.empty())
+        {
+            opts.nuextremityPoints = StringConverter::parseInt(bi->second);
+        }
+
+        bi = binOpt.find("-log");
+        if (!bi->second.empty())
+        {
+            opts.logFile = bi->second;
+        }
+
+        bi = binOpt.find("-E");
+        if (!bi->second.empty())
+        {
+            if (bi->second == "big")
+                opts.endian = Serializer::ENDIAN_BIG;
+            else if (bi->second == "little")
+                opts.endian = Serializer::ENDIAN_LITTLE;
+            else 
+                opts.endian = Serializer::ENDIAN_NATIVE;
+        }
+
+		ui = unOpt.find("-d3d");
+		if (ui->second)
+		{
+			opts.d3d = true;
+		}
+
+		ui = unOpt.find("-gl");
+		if (ui->second)
+		{
+			opts.gl = true;
+			opts.d3d = false;
+		}
+
+	}
     // Source / dest
     if (numArgs > startIndex)
         source = args[startIndex];
@@ -221,35 +302,41 @@ XmlOptions parseArgs(int numArgs, char **args)
 	StringUtil::toLowerCase(ext);
     opts.destExt = ext;
 
-    cout << endl;
-    cout << "-- OPTIONS --" << endl;
-    cout << "source file      = " << opts.source << endl;
-    cout << "destination file = " << opts.dest << endl;
-    cout << "interactive mode = " << StringConverter::toString(opts.interactiveMode) << endl;
-    if (opts.numLods == 0)
-    {
-        cout << "lod levels       = none (or use existing)" << endl;
-    }
-    else
-    {
-        cout << "lod levels       = " << opts.numLods << endl;
-        cout << "lod distance     = " << opts.lodDist << endl;
-        if (opts.usePercent)
+    if (!opts.quietMode) 
+	{
+        cout << endl;
+        cout << "-- OPTIONS --" << endl;
+        cout << "source file      = " << opts.source << endl;
+        cout << "destination file = " << opts.dest << endl;
+            cout << "log file         = " << opts.logFile << endl;
+        cout << "interactive mode = " << StringConverter::toString(opts.interactiveMode) << endl;
+        if (opts.numLods == 0)
         {
-            cout << "lod reduction    = " << opts.lodPercent << "%" << endl;
+            cout << "lod levels       = none (or use existing)" << endl;
         }
         else
         {
-            cout << "lod reduction    = " << opts.lodFixed << " verts" << endl;
+            cout << "lod levels       = " << opts.numLods << endl;
+            cout << "lod distance     = " << opts.lodDist << endl;
+            if (opts.usePercent)
+            {
+                cout << "lod reduction    = " << opts.lodPercent << "%" << endl;
+            }
+            else
+            {
+                cout << "lod reduction    = " << opts.lodFixed << " verts" << endl;
+            }
         }
+        if (opts.nuextremityPoints)
+            cout << "Generate extremes per submesh = " << opts.nuextremityPoints << endl;
+        cout << "Generate edge lists  = " << opts.generateEdgeLists << endl;
+        cout << "Generate tangents = " << opts.generateTangents << endl;
+        cout << "Reorganise vertex buffers = " << opts.reorganiseBuffers << endl;
+    	cout << "Optimise animations = " << opts.optimiseAnimations << endl;
+    	
+        cout << "-- END OPTIONS --" << endl;
+        cout << endl;
     }
-    cout << "Generate edge lists  = " << opts.generateEdgeLists << endl;
-    cout << "Generate tangents = " << opts.generateTangents << endl;
-    cout << "Reorganise vertex buffers = " << opts.reorganiseBuffers << endl;
-	cout << "Optimise animations = " << opts.optimiseAnimations << endl;
-	
-    cout << "-- END OPTIONS --" << endl;
-    cout << endl;
 
 
     return opts;
@@ -297,6 +384,7 @@ void XMLToBinary(XmlOptions opts)
     if (!doc->LoadFile())
     {
         cout << "Unable to open file " << opts.source << " - fatal error." << endl;
+        delete doc;
         exit (1);
     }
     TiXmlElement* root = doc->RootElement();
@@ -305,7 +393,13 @@ void XMLToBinary(XmlOptions opts)
         delete doc;
         MeshPtr newMesh = MeshManager::getSingleton().createManual("conversion", 
             ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
-        xmlMeshSerializer->importMesh(opts.source, newMesh.getPointer());
+		VertexElementType colourElementType;
+		if (opts.d3d)
+			colourElementType = VET_COLOUR_ARGB;
+		else
+			colourElementType = VET_COLOUR_ABGR;
+
+        xmlMeshSerializer->importMesh(opts.source, colourElementType, newMesh.getPointer());
 
         // Re-jig the buffers?
         if (opts.reorganiseBuffers)
@@ -317,7 +411,7 @@ void XMLToBinary(XmlOptions opts)
                 // Automatic
                 VertexDeclaration* newDcl = 
                     newMesh->sharedVertexData->vertexDeclaration->getAutoOrganisedDeclaration(
-                        newMesh->hasSkeleton());
+                        newMesh->hasSkeleton(), newMesh->hasVertexAnimation());
                 if (*newDcl != *(newMesh->sharedVertexData->vertexDeclaration))
                 {
                     // Usages don't matter here since we're onlly exporting
@@ -329,7 +423,6 @@ void XMLToBinary(XmlOptions opts)
             }
             // Dedicated geometry
             Mesh::SubMeshIterator smIt = newMesh->getSubMeshIterator();
-            unsigned short idx = 0;
             while (smIt.hasMoreElements())
             {
                 SubMesh* sm = smIt.getNext();
@@ -338,7 +431,7 @@ void XMLToBinary(XmlOptions opts)
                     // Automatic
                     VertexDeclaration* newDcl = 
                         sm->vertexData->vertexDeclaration->getAutoOrganisedDeclaration(
-                            newMesh->hasSkeleton());
+                            newMesh->hasSkeleton(), newMesh->hasVertexAnimation());
                     if (*newDcl != *(sm->vertexData->vertexDeclaration))
                     {
                         // Usages don't matter here since we're onlly exporting
@@ -530,14 +623,17 @@ void XMLToBinary(XmlOptions opts)
 
         if (opts.generateEdgeLists)
         {
-            std::cout << "Generating edge lists...." << std::endl;
+            if (!opts.quietMode) 
+			{
+                std::cout << "Generating edge lists...." << std::endl;
+            }
             newMesh->buildEdgeList();
         }
 
         if (opts.generateTangents)
         {
             unsigned short srcTex, destTex;
-            bool existing = newMesh->suggestTangentVectorBuildParams(srcTex, destTex);
+            bool existing = newMesh->suggestTangentVectorBuildParams(opts.tangentSemantic, srcTex, destTex);
             if (existing)
             {
                 std::cout << "\nThis mesh appears to already have a set of 3D texture coordinates, " <<
@@ -564,13 +660,25 @@ void XMLToBinary(XmlOptions opts)
             }
             if (opts.generateTangents)
             {
-                std::cout << "Generating tangent vectors...." << std::endl;
-                newMesh->buildTangentVectors(srcTex, destTex);
+                if (!opts.quietMode) 
+				{
+                    std::cout << "Generating tangent vectors...." << std::endl;
+                }
+                newMesh->buildTangentVectors(opts.tangentSemantic, srcTex, destTex);
             }
         }
 
+        if (opts.nuextremityPoints)
+        {
+            Mesh::SubMeshIterator smIt = newMesh->getSubMeshIterator();
+            while (smIt.hasMoreElements())
+            {
+                SubMesh* sm = smIt.getNext();
+                sm->generateExtremes (opts.nuextremityPoints);
+            }
+        }
 
-        meshSerializer->exportMesh(newMesh.getPointer(), opts.dest);
+        meshSerializer->exportMesh(newMesh.getPointer(), opts.dest, opts.endian);
     }
     else if (!stricmp(root->Value(), "skeleton"))
     {
@@ -582,9 +690,12 @@ void XMLToBinary(XmlOptions opts)
 		{
 			newSkel->optimiseAllAnimations();
 		}
-        skeletonSerializer->exportSkeleton(newSkel.getPointer(), opts.dest);
+        skeletonSerializer->exportSkeleton(newSkel.getPointer(), opts.dest, opts.endian);
+    }
+    else
+    {
+        delete doc;
     }
-
 
 }
 
@@ -618,8 +729,10 @@ int main(int numargs, char** args)
         return -1;
     }
 
+    XmlOptions opts = parseArgs(numargs, args);
+
     logMgr = new LogManager();
-	logMgr->createLog("OgreXMLConverter.log"); 
+	logMgr->createLog(opts.logFile, false, !opts.quietMode); 
     rgm = new ResourceGroupManager();
     mth = new Math();
     meshMgr = new MeshManager();
@@ -634,10 +747,6 @@ int main(int numargs, char** args)
 
 
 
-    logMgr->createLog("OgreXMLConverter.log");
-
-    XmlOptions opts = parseArgs(numargs, args);
-
     if (opts.sourceExt == "mesh")
     {
         meshToXML(opts);
